/*##############################################################################
   
   Developed by NetMob Group, University of Trento, Italy

   University of Trento
   DIT - Department of Informatics and Telecommunications 
   Computer Networks and Mobility Research Program
   http://netmob.unitn.it/ 
   
   This file is part of the WilmaGate system, created as part
   of the WILMA Project, sponsored by the Autonomous
   Province of Trento and participated by ITC-irst, the University
   of Trento and Alpikom Spa. More info at

     http://www.wilmaproject.org/		

   To download the project:
      - in tarball format
        http://netmob.unitn.it/wilmagate.html
      - connect to CVS repository
        :pserver:cvs@rtm.unitn.it:/home/cvsroot
   In both previous cases, the download is password protected. To obtain
   the password you must send a mail to 

      wilmagate@dit.unitn.it

################################################################################

   The WilmaGate Licence of Use


   Copyright (c) 2003-2005
                - Universita` di Trento,
                  Dipartimento di Informatica e Telecomunicazioni (DIT)
                  Computer Networks and Mobility Research Program


   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (hereinafter
   the "WilmaGate") directly from the copyright owner, to use the WilmaGate
   without restriction of purpose or time, including without limitation the
   rights to use, copy, modify, and merge the WilmaGate, subject to the
   following conditions:

   The above copyright notice and this permission notice shall be included
   in all copies or substantial portions of the WilmaGate. 

   The WilmaGate shall not be sub-licenced either commercially or free of
   charge, as is or in part or as a part of any other package. 

   The licensee agrees to properly credit WilmaGate and its copyright 
   owners in any technical, scientific or divulgative publication, 
   whatever publishing means are used, that is based entirely or in part 
   on WilmaGate use, as well as in any booklet, web-site or brochure that 
   advertise services based on WilmaGate use. 

   If the WilmaGate is modified, the copyright owner shall be notified.

   THE WilmaGate IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

##############################################################################*/

#include <iostream>
#include <iomanip>
#include <csignal>

#include "exceptions.h"
#include "access.h"
#include "timers.h"

using namespace std;
using namespace Common;



char * fastsequence = "EESSSSSEe";
int waitingtime = 1;
char * slowsequence = "eesssssSs";

pthread_t thread;
time_t initialTime;
AccessRights acc;
CoarseTimer ct;

class Other: public TimerListener {
private:
	int sequenceIndex;
public:
	Other (void) { sequenceIndex = 0; }
	bool timerTick (void);
};

bool Other::timerTick (void)
{
	if ( ! slowsequence[sequenceIndex] )
		return false;
	cout << setw(4) << time(NULL) - initialTime << "\t\tSLOW: Action "
		<< slowsequence[sequenceIndex] << "." << endl;
	switch ( slowsequence[sequenceIndex] ) {
	case 'S':
		acc.beginShared ();
		break;
	case 's':
		acc.endShared ();
		break;
	case 'E':
		acc.beginExclusive ();
		break;
	case 'e':
		acc.endExclusive ();
		break;
	}
	sequenceIndex++;
	return true;
}

void accelerateSlowSequence (int)
{
	ct.setPeriod (0);
}

int main (void)
{
	try {
		signal (SIGUSR1, accelerateSlowSequence);
		Other other;
		ct.registerListener (&other);

		initialTime = time (0);
		ct.start (1);
		for ( int i = 0; fastsequence[i]; i++ ) {
			cout << setw(4) << time(NULL) - initialTime << "\tFAST: Waiting for "
				<< fastsequence[i] << "..." << endl;
			switch ( fastsequence[i] ) {
			case 'S':
				acc.beginShared ();
				break;
			case 's':
				acc.endShared ();
				break;
			case 'E':
				acc.beginExclusive ();
				break;
			case 'e':
				acc.endExclusive ();
				break;
			}
			cout << setw(4) << time(NULL) - initialTime << "\tFAST: ... Got it." << endl;
		}
		cout << setw(4) << time(NULL) - initialTime
			<< "\t*** Fast sequence ended. Accelerating slow sequence ***" << endl;
		raise (SIGUSR1);
		ct.wait ();
	}
	catch ( WilmaGateException &e ) {
		cerr << e << '\n';
		return 1;
	}
	cout << setw(4) << time(NULL) - initialTime
		<< "\t*** Sequences completed successfully ***" << endl;
	return 0;
}
