/*##############################################################################
   
   Developed by NetMob Group, University of Trento, Italy

   University of Trento
   DIT - Department of Informatics and Telecommunications 
   Computer Networks and Mobility Research Program
   http://netmob.unitn.it/ 
   
   This file is part of the WilmaGate system, created as part
   of the WILMA Project, sponsored by the Autonomous
   Province of Trento and participated by ITC-irst, the University
   of Trento and Alpikom Spa. More info at

     http://www.wilmaproject.org/		

   To download the project:
      - in tarball format
        http://netmob.unitn.it/wilmagate.html
      - connect to CVS repository
        :pserver:cvs@rtm.unitn.it:/home/cvsroot
   In both previous cases, the download is password protected. To obtain
   the password you must send a mail to 

      wilmagate@dit.unitn.it

################################################################################

   The WilmaGate Licence of Use


   Copyright (c) 2003-2005
                - Universita` di Trento,
                  Dipartimento di Informatica e Telecomunicazioni (DIT)
                  Computer Networks and Mobility Research Program


   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (hereinafter
   the "WilmaGate") directly from the copyright owner, to use the WilmaGate
   without restriction of purpose or time, including without limitation the
   rights to use, copy, modify, and merge the WilmaGate, subject to the
   following conditions:

   The above copyright notice and this permission notice shall be included
   in all copies or substantial portions of the WilmaGate. 

   The WilmaGate shall not be sub-licenced either commercially or free of
   charge, as is or in part or as a part of any other package. 

   The licensee agrees to properly credit WilmaGate and its copyright 
   owners in any technical, scientific or divulgative publication, 
   whatever publishing means are used, that is based entirely or in part 
   on WilmaGate use, as well as in any booklet, web-site or brochure that 
   advertise services based on WilmaGate use. 

   If the WilmaGate is modified, the copyright owner shall be notified.

   THE WilmaGate IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

##############################################################################*/

#include <iostream>
#include <cstring>
#include <unistd.h>

#include "socket.h"

using namespace Common;

const IPv4Address destIP = "127.0.0.1";
const uint16_t destPort = 62544;
char *UDPmessage = "Ciao!";
char *TCPmessage = "Ciao!\nCuc!\n";

class Receiver: public PacketListener {
public:
	virtual SuggestedAction receivedPacket (uint8_t *message, int size, int socketID, IPv4Address senderIP, uint16_t senderPort);
	virtual SuggestedAction failed (int socketID);
};

SocketListener::SuggestedAction Receiver::receivedPacket (uint8_t *message, int size, int socketID,
								IPv4Address senderIP, uint16_t senderPort)
{
	message[size] = '\0';
	cout << "Message of size " << size << " from " << senderIP << ':' << senderPort << " on socket "
			<< socketID << "\n>\t" << message << endl;
	return CONTINUE;
}

SocketListener::SuggestedAction Receiver::failed (int socketID)
{
	cerr << "Socket " << socketID << " closed.\n";
	return (socketID == 1) ? STOP : DELETE;
}

class StreamReceiver: public StreamListener {
public:
	virtual void connectionOpen (TCPSocket *s);
	virtual SuggestedAction failed (int socketID);
};

void StreamReceiver::connectionOpen (TCPSocket *s)
{
	string st;
	while ( s->readString (st) )
		cout << "String received from " << s->getRemoteIP() << ':' << s->getRemotePort()
			<< "\n>\t" << st << endl;
}

SocketListener::SuggestedAction StreamReceiver::failed (int socketID)
{
	cerr << "Socket " << socketID << " closed.\n";
	return (socketID == 1) ? STOP : DELETE;
}

class ConnectionReceiver: public ConnectionListener {
private:
	PacketListener &receiver;
	StreamListener &sreceiver;
	int nconn;
public:
	ConnectionReceiver (PacketListener &r, StreamListener &s);
	virtual SuggestedAction failed (int socketID);
	virtual SuggestedAction incomingConnection (TCPSocket *s, int socketID);
};

ConnectionReceiver::ConnectionReceiver (PacketListener &r, StreamListener &s):
	receiver (r),
	sreceiver (s)
{
	nconn = 2;
}

SocketListener::SuggestedAction ConnectionReceiver::incomingConnection (TCPSocket *s, int socketID)
{
	cerr << "Incoming connection from " << s->getRemoteIP() << ':' << s->getRemotePort() << endl;
	if ( nconn == 2 ) {
		s->registerListener (receiver, nconn++);
		s->startReceiving ();
	}
	else {
		s->registerStreamListener (sreceiver);
		s->startSyncReceiving ();
	}
	return CONTINUE;
}

SocketListener::SuggestedAction ConnectionReceiver::failed (int socketID)
{
	cerr << "Error from listening socket " << socketID << "!\n";
	return STOP;
}

int main (void)
{
	try {
		Receiver rec;

		cerr << "Testing UDP socket...\n";
		UDPSocket skin, skout;
		skin.bind (destIP, destPort);
		skin.registerListener (rec, 1);
		skout.setDestination (destIP, destPort);
		skin.startReceiving ();
		for ( int i = 0; i < 5; i++ ) {
			sleep (1);
			cout << "Sending message" << endl;
			if ( ! skout.send (UDPmessage, strlen(UDPmessage)) ) {
				cerr << "Send error!\n";
				exit (2);
			}
		}
		sleep (1);
		skin.stopReceiving ();

		cerr << "Testing TCP socket...\n";
		StreamReceiver srec;
		ConnectionReceiver crec (rec, srec);
		TCPSocket tskin;
		cerr << "Binding...\n";
		tskin.bind (destIP, destPort);
		cerr << "Registering listener...\n";
		tskin.registerConnectionListener (crec, 0);
		cerr << "Listening...\n";
		tskin.startListening ();
		cerr << "Setting destination...\n";
		for ( int j = 0; j < 2; j++ ) {
			TCPSocket *tskout = new TCPSocket;
			cerr << "Connection " << j << " to " << destIP << ':' << destPort << endl;
			if ( ! tskout->setDestination (destIP, destPort) ) {
				cerr << "Connect error!\n";
				exit (2);
			}
			for ( int i = 0; i < 5; i++ ) {
				sleep (1);
				cout << "Sending message" << endl;
				if ( ! tskout->send (TCPmessage, strlen(TCPmessage)) ) {
					cerr << "Send error!\n";
					exit (2);
				}
			}
			tskout->close ();
			delete tskout;
		}
		tskin.stopListening ();
		sleep (1);
	}
	catch ( SocketException &e ) {
		cerr << e << '\n';
	}
	catch ( ... ) {
		cerr << "Unexpected exception!\n";
	}

	sleep(1);

	return 0;
}
