///
/// \file auth_statustable_interface.h
///	\author Danilo Severina
///
///	\brief Interface between %Authentication and %StatusTable.
///	
/// The file contains the definitions of functions that are used by the
///	Plugin sub-classes to access the Status Table to read and/or write 
///	data. Not all of functions that read and write informations in Status
/// Table must be used by plugins, so this class has function of interface:
///	only the methods that are described here can be used byb plugins.
/// There are functions that can be used by the plugins but they are not 
///	defined in this file: they are defined virtual in StatusTableInterface and
///	the interface beetwen tha plugins and the Status Table derive from the
/// previous class.
///
///	$Id: auth_statustable_interface.h,v 1.3 2005/12/13 17:18:53 cvs Exp $
///

/*##############################################################################
   
   Developed by NetMob Group, University of Trento, Italy

   University of Trento
   DIT - Department of Informatics and Telecommunications 
   Computer Networks and Mobility Research Program
   http://netmob.unitn.it/ 
   
   This file is part of the WilmaGate system, created as part
   of the WILMA Project, sponsored by the Autonomous
   Province of Trento and participated by ITC-irst, the University
   of Trento and Alpikom Spa. More info at

     http://www.wilmaproject.org/		

   To download the project:
      - in tarball format
        http://netmob.unitn.it/wilmagate.html
      - connect to CVS repository
        :pserver:cvs@rtm.unitn.it:/home/cvsroot
   In both previous cases, the download is password protected. To obtain
   the password you must send a mail to 

      wilmagate@dit.unitn.it

################################################################################

   The WilmaGate Licence of Use


   Copyright (c) 2003-2005
                - Universita` di Trento,
                  Dipartimento di Informatica e Telecomunicazioni (DIT)
                  Computer Networks and Mobility Research Program


   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (hereinafter
   the "WilmaGate") directly from the copyright owner, to use the WilmaGate
   without restriction of purpose or time, including without limitation the
   rights to use, copy, modify, and merge the WilmaGate, subject to the
   following conditions:

   The above copyright notice and this permission notice shall be included
   in all copies or substantial portions of the WilmaGate. 

   The WilmaGate shall not be sub-licenced either commercially or free of
   charge, as is or in part or as a part of any other package. 

   The licensee agrees to properly credit WilmaGate and its copyright 
   owners in any technical, scientific or divulgative publication, 
   whatever publishing means are used, that is based entirely or in part 
   on WilmaGate use, as well as in any booklet, web-site or brochure that 
   advertise services based on WilmaGate use. 

   If the WilmaGate is modified, the copyright owner shall be notified.

   THE WilmaGate IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

##############################################################################*/

#ifndef _AUTHENTICATOR_STATUSTABLE_INTERFACE_
#define	_AUTHENTICATOR_STATUSTABLE_INTERFACE_

#include <iostream>
#include "statustable_interface.h"
//#include "token.h"
//#include "addresses.h"
//#include "packets.h"

using namespace std;

namespace Gatekeeper {

///
///	\brief Interface between %Authentication and %StatusTable.
/// 
/// The class contains the definitions of functions that are used by the
///	Authentication class to access the Status Table to read and/or write 
///	data. Not all of functions that read and write informations in Status
/// Table must be used %Authentication, so this class has function of interface:
///	only the methods that are described here can be used by %Authentication
/// There are functions that can be used by the %Authentication but they are not 
///	defined in this file: they are defined virtual in StatusTableInterface and
///	the interface beetwen the %Authentication and the %StatusTable derives from the
/// previous class.
///
class AuthStatusTableInterface : virtual public StatusTableInterface{

private:

public:

		///
		/// \brief Virtual destructor.
		///
		virtual ~AuthStatusTableInterface (void) {}
	///
	/// \brief Set identification data for the user.
	///
	/// \param ip IP address assigned to the user.
	/// \param name Name of the user, as provided by the authentication server.
	/// \param email Email of the user.
	///
	virtual void setUserData (IPv4Address ip, const char *name, const char *email) = 0;

};

}

#endif

