/*##############################################################################
   
   Developed by NetMob Group, University of Trento, Italy

  University of Trento
   DIT - Department of Informatics and Telecommunications 
   Computer Networks and Mobility Research Program
   http://netmob.unitn.it/ 
   
   This file is part of the WilmaGate system, created as part
   of the WILMA Project, sponsored by the Autonomous
   Province of Trento and participated by ITC-irst, the University
   of Trento and Alpikom Spa. More info at

     http://www.wilmaproject.org/		

   To download the project:
      - in tarball format
        http://netmob.unitn.it/wilmagate.html
      - connect to CVS repository
        :pserver:cvs@rtm.unitn.it:/home/cvsroot
   In both previous cases, the download is password protected. To obtain
   the password you must send a mail to 

      wilmagate@dit.unitn.it

################################################################################

   The WilmaGate Licence of Use


   Copyright (c) 2003-2005
                - Universita` di Trento,
                  Dipartimento di Informatica e Telecomunicazioni (DIT)
                  Computer Networks and Mobility Research Program


   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (hereinafter
   the "WilmaGate") directly from the copyright owner, to use the WilmaGate
   without restriction of purpose or time, including without limitation the
   rights to use, copy, modify, and merge the WilmaGate, subject to the
   following conditions:

   The above copyright notice and this permission notice shall be included
   in all copies or substantial portions of the WilmaGate. 

   The WilmaGate shall not be sub-licenced either commercially or free of
   charge, as is or in part or as a part of any other package. 

   The licensee agrees to properly credit WilmaGate and its copyright 
   owners in any technical, scientific or divulgative publication, 
   whatever publishing means are used, that is based entirely or in part 
   on WilmaGate use, as well as in any booklet, web-site or brochure that 
   advertise services based on WilmaGate use. 

   If the WilmaGate is modified, the copyright owner shall be notified.

   THE WilmaGate IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

##############################################################################*/

#include <sstream>
#include <cerrno>
extern "C" {
	#include <syslog.h>
}
#include "../Common/utilities.h"
#include "authentication.h"

namespace Gatekeeper {

Authentication::Authentication(AuthStatusTableInterface &astio,
							   ProviderListInterface &plio,
							   IPv4Address listenIP, uint16_t listenPort, int ren)
			throw (WilmaGateException): 
asti(astio), pli(plio), renewalTime (ren)
{
	for ( int i = 0; i < 256; tcpconn[i++] = NULL );
	if ( !skTCPin.bind(listenIP, listenPort) )
		throw WilmaGateException ("Error in binding auth port", 112233);
	skTCPin.registerConnectionListener(*this, 1000);
}

Authentication::~Authentication()
{}


SocketListener::SuggestedAction Authentication::receivedPacket (uint8_t *message, int size, int socketID,
								IPv4Address senderIP, uint16_t senderPort)
{
	//char macstring [40];
	char part[1000], part2[10];
	message[size] = '\0';
	istringstream ist (reinterpret_cast<char*>(message));
	ist.getline (part, 1000, '\t');
	if ( !strncmp (part, "authenticate",12) ) {
		StatusTableInterface::authTypes type;
		if (strcmp (part, "authenticate")==0)
			type=StatusTableInterface::HTTP;
		else if (strcmp (part, "authenticate_sip")==0)
			type=StatusTableInterface::SIP;
		else
			type=StatusTableInterface::UNKNOWN;
		IPv4Address IP;
		string name, email;
		ist.getline (part, 1000, '\t');
		IP = part;
		ist.getline (part, 1000, '\t');
		name = part;
		ist.getline (part, 1000, '\t');
		email = part;
		ist.getline (part, 1000, '\t');
		
		*part2 = '\0';
		if ( !ist.eof() )
			ist.getline (part2, 10, '\t');
		bool dhcpauth = *part2 >= '1' && *part2 <= '9';
		Token *ptk = asti.getToken (IP);
		ostringstream os;
		if ( ptk && *ptk == part ) {
			//logger() << "authenticate_" << type << " received";
			asti.authorizeUserFor(IP,type,name.c_str(),email.c_str());
			/*if ( asti.getStatus(IP) != StatusTableInterface::AUTHORIZED )
				syslog (LOG_INFO, "AUTH IP %s MAC %s NAME %s EMAIL %s", inet_ntoa(IP.asInaddr()),
						asti.getMAC(IP)->toString(macstring), name.c_str(), email.c_str());
			*/
			asti.setExpirationAuth (IP, renewalTime, dhcpauth);
			ptk = asti.getToken (IP);
			os << "ok\t" << *ptk << '\t' << renewalTime/2 << '\n' << asti.getURL(IP) << endl << endl;
		}
		else {
			logger() << "Received wrong token: \n";// \"" << part << "\" instead of \"" << *ptk << "\".\n";
			os << "no" << endl;
		}
		tcpconn[socketID]->send (os.str().c_str(), os.str().length());
	}
	else if ( ! strncmp (part, "revoke",6) ) {
		StatusTableInterface::authTypes type;
		if (strcmp (part, "revoke")==0)
			type=StatusTableInterface::HTTP;
		else if (strcmp (part, "revoke_sip")==0)
			type=StatusTableInterface::SIP;
		else
			type=StatusTableInterface::UNKNOWN;
		IPv4Address IP;
		ist.getline (part, 1000, '\t');
		IP = part;
		ist.getline (part, 1000, '\t');
		Token *ptk = asti.getToken (IP);
		ostringstream os;
		if ( *ptk == part && asti.getStatus (IP) == StatusTableInterface::AUTHORIZED ) {
      			//remove authorization only if revoke is for the actual type of authentication 
			if ((type==StatusTableInterface::SIP && asti.isSIPauthenticated(IP)) ||
       			    (type==StatusTableInterface::HTTP && asti.isHTTPauthenticated(IP))){
				asti.removeAuthType(IP,type);				
				//if user is no more authenticated, de-authenticate
				//since we allow only one kind of authorization at a time, this check is useless...	
				//if(!asti.isAuthenticated(IP))
				asti.setStatus (IP, StatusTableInterface::TEMPORARY_LEASE, false);
				//logger() << "Deauthenticate user " << IP << endl;
			}
			//replay with a positive answer to the client
			os << "ok" << endl;
			/*
			syslog (LOG_INFO, "ENDAUTH IP %s MAC %s", inet_ntoa(IP.asInaddr()),
-					asti.getMAC(IP)->toString(macstring));
			*/
		}
		else
			os << "no" << endl;
		tcpconn[socketID]->send (os.str().c_str(), os.str().length());
	}
	else {
		tcpconn[socketID] = NULL;
		return DELETE;
	}
	return CONTINUE;
}


SocketListener::SuggestedAction Authentication::incomingConnection (TCPSocket *s, int socketID)
{
	int i;
	for ( i = 0; i < 256 && tcpconn[i]; i++ );
	if ( i == 256 ) {
		logger() << "Auth: Too many open sockets to accommodate, refusing connection.\n";
		return DELETE;
	}
	tcpconn[i] = s;
	s->registerListener (*this, i);
	s->startReceiving ();
	return CONTINUE;
}

SocketListener::SuggestedAction Authentication::failed (int socketID)
{
	logger () << "Auth: Socket " << socketID << " closure or error: " << errno << '\n';
	return (socketID == 1000) ? CONTINUE : DELETE;
}

}
