/*##############################################################################
   
   Developed by NetMob Group, University of Trento, Italy

   University of Trento
   DIT - Department of Informatics and Telecommunications 
   Computer Networks and Mobility Research Program
   http://netmob.unitn.it/ 
   
   This file is part of the WilmaGate system, created as part
   of the WILMA Project, sponsored by the Autonomous
   Province of Trento and participated by ITC-irst, the University
   of Trento and Alpikom Spa. More info at

     http://www.wilmaproject.org/		

   To download the project:
      - in tarball format
        http://netmob.unitn.it/wilmagate.html
      - connect to CVS repository
	:pserver:cvs@rtm.unitn.it:/home/cvsroot
   In both previous cases, the download is password protected. To obtain
   the password you must send a mail to 

      wilmagate@dit.unitn.it

################################################################################

   The WilmaGate Licence of Use


   Copyright (c) 2003-2005
                - Universita` di Trento,
                  Dipartimento di Informatica e Telecomunicazioni (DIT)
                  Computer Networks and Mobility Research Program


   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (hereinafter
   the "WilmaGate") directly from the copyright owner, to use the WilmaGate
   without restriction of purpose or time, including without limitation the
   rights to use, copy, modify, and merge the WilmaGate, subject to the
   following conditions:

   The above copyright notice and this permission notice shall be included
   in all copies or substantial portions of the WilmaGate. 

   The WilmaGate shall not be sub-licenced either commercially or free of
   charge, as is or in part or as a part of any other package. 

   The licensee agrees to properly credit WilmaGate and its copyright 
   owners in any technical, scientific or divulgative publication, 
   whatever publishing means are used, that is based entirely or in part 
   on WilmaGate use, as well as in any booklet, web-site or brochure that 
   advertise services based on WilmaGate use. 

   If the WilmaGate is modified, the copyright owner shall be notified.

   THE WilmaGate IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

##############################################################################*/

//General Struct of Message Element:
//	-------------------------
//	| Type | Lenght | Value |
//	-------------------------
//	|  16  |   16   |   n   |
//	-------------------------
//

#include "../Common/utilities.h"
#include "capwap_plugin_A.h"

using namespace std;
using namespace Common;

namespace Gatekeeper {

	CapwapPluginA::CapwapPluginA(PluginStatusTableInterface &pluginStatusTableInterfaceObject, 
				     WTPTableInterface &pluginWTPTableInterfaceObject,
				     IPv4Address gateIpAddress,
				     IPv4Address wayIpAddress,
				     const char *domainName): 
		gkIP(gateIpAddress),
		gwIP(wayIpAddress),
		domain(domainName),
		psti(pluginStatusTableInterfaceObject),
		pwtpti(pluginWTPTableInterfaceObject)
	{
		inudp=pwtpti.Getinudpport();
		outudp=pwtpti.Getoutudpport();
	}

	CapwapPluginA::~CapwapPluginA (void)
	{
	}

	
	CAPWAPMessage::MessageTypes CapwapPluginA::check (EthernetFrame *ethf)
	{
		CAPWAPMessage::MessageTypes type = CAPWAPMessage::NONE;
		CAPWAPMessage::MessageElementType MEtype = CAPWAPMessage::ME_NONE;
		bool scan=true;
		uint16_t p=0;
		bitset<1> cme; //Check message element
		
		if ( (ethf->isProtocol (EthernetFrame::IP_PROTOCOL))
				   && (ethf->IPPayload().protocol == IPPacket::UDP_PROTOCOL) )
				   //&& (ethf->IPPayload().UDPPayload().destinationPort == UDPORT) ) //UDP Port!!!
		{
			CAPWAPMessage *m = reinterpret_cast<CAPWAPMessage*>(ethf->IPPayload().UDPPayload().payload);
			
		//Version Verify
			if(m->getVersion()!=0)
			{
				//logger()<<"[CAPWAP] CAPWAP Error Version"<<endl;
				return type;
			}

		//Only Discovery Request and Join Request
			switch ( m->MessageType ) {
				case CAPWAPMessage::DISCOVERYREQUEST:
				//Check message element
					//logger()<<"[CAPWAP] Check DISCOVERYREQUEST"<<endl;
					while ( p < ((m->MsgElementLength)-7) && scan) 
					{
						scan=false;
						MEtype = static_cast<CAPWAPMessage::MessageElementType>(m->MessageElement[p]);

						if (MEtype==CAPWAPMessage::DISCOVERYTYPE){
							cme[0]=1; 
							p+=2;
							p+=static_cast<uint16_t>(m->MessageElement[p])+2;
							scan=true;

						}
						if( (cme[0]) ){
							type = CAPWAPMessage::DISCOVERYREQUEST;
							//logger()<<"[CAPWAP]  --- OK "<<endl;
						}
						
					}
					break;

				case CAPWAPMessage::JOINREQUEST:
				//logger()<<"[CAPWAP] Check JOINREQUEST"<<endl;
					while ( p < ((m->MsgElementLength)-7) && scan) 
					{
						scan=false;
						MEtype = static_cast<CAPWAPMessage::MessageElementType>(m->MessageElement[p]);
						if (MEtype==CAPWAPMessage::SESSIONID){
							cme[0]=1;
							p+=2;
							p+=static_cast<uint16_t>(m->MessageElement[p])+2;
							scan=true;
						}
						if( (cme[0]) ){
							type = CAPWAPMessage::JOINREQUEST;
							//logger()<<"[CAPWAP]  --- OK"<<endl;
						}
					}
					break;

				default: 
					break;
			}
		
		}
// */
	return type;
	}

//-------------------------------------------------------------------------------------------------START Message Elemnt
	void CapwapPluginA::setACDescriptor (uint8_t** data, int* len)
	{
		int p;
		uint8_t* bufpck;
// /* 
		if(*len!=0){
			p=*len;
			bufpck=new uint8_t [p + 2+2+12]; //Type+Length+Value
			memmove(bufpck,*data,p); //copy previous message element
			delete [] *data;
		
			//LEN TOT
			*len=p+2+2+12;
		}
		else{
			p=0;
			bufpck=new uint8_t[2+2+12];
			
			//LEN TOT
			*len=2+2+12;
		}

	//Type
		*reinterpret_cast<uint16_t*>(bufpck+p) = CAPWAPMessage::ACDESCRIPTOR; 
		p += 2;
		
	//Lenght
		*reinterpret_cast<uint16_t*>(bufpck+p) = 12; 
		p += 2;

	//Stations: Number of station (client) actually associated to Gatekepper 
		*reinterpret_cast<uint16_t*>(bufpck+p) = 0; //psti.getNumberUser(); !!! Rivedi non e disponibile il metodo!!!
		p += 2;

	//Limit: Max number of stations supported
		*reinterpret_cast<uint16_t*>(bufpck+p) = 255; 
		p += 2;

	//Active WTPs: Number of WTP actually associated to Gatekepper
		*reinterpret_cast<uint16_t*>(bufpck+p) = pwtpti.GetNumberWTP();
		p += 2;

	//Max WTPs: Max number  of WTP supported
		*reinterpret_cast<uint16_t*>(bufpck+p) =pwtpti.GetMaxNumberWTP();
		p += 2;

	//Security
		*reinterpret_cast<uint8_t*>(bufpck+p) = 0;
		p += 1;

	//Radio Mac Field
		*reinterpret_cast<uint8_t*>(bufpck+p) = 0;
		p += 1;

	//Wireless Field
		*reinterpret_cast<uint8_t*>(bufpck+p) = 0;
		p += 1;

	//Reserved
		*reinterpret_cast<uint8_t*>(bufpck+p) = 0;
		p += 1;
		
		*data=bufpck;
	}

	void CapwapPluginA::setACName (uint8_t** data, int* len)
	{
		int p;
		uint8_t* bufpck;
		uint16_t lenstr=strlen( pwtpti.GetACName().c_str() );
// /*
		if(*len!=0){
			p=*len;
			bufpck=new uint8_t[p + 4+ lenstr]; //Type+Length+Value
			memmove(bufpck,*data,p);
			delete [] (*data);
			
			//LEN TOT
			*len = p+(4+lenstr);
		}
		else{
			p=0;
			bufpck=new uint8_t[(4+lenstr)];

		//LEN TOT
			*len =(4+lenstr );
		}

	//Type
		*reinterpret_cast<uint16_t*>(bufpck+p) = CAPWAPMessage::ACNAME;
		p += 2;
		
	//Lenght
		*reinterpret_cast<uint16_t*>(bufpck+p) = lenstr;
		p += 2;

	//AC name
		memmove (reinterpret_cast<char*>(bufpck+p), pwtpti.GetACName().c_str(), lenstr );
		p += lenstr;
// */
		*data=bufpck;
	}

	void CapwapPluginA::setResultCode (uint8_t** data, int* len, int code)
	{
		int p;
		uint8_t* bufpck;
// /*
		if(*len!=0){
			p=*len;
			bufpck=new uint8_t[p + 2+2+4]; //Type+Length+Value
			memmove(bufpck,*data,p);
			delete [] *data; 

		//LEN TOT
			*len= p+8; 
		}
		else{
			p=0;
			bufpck=new uint8_t[8];

		//LEN TOT
			*len= 8;
		}

	//Type
		*reinterpret_cast<uint16_t*>(bufpck+p) = CAPWAPMessage::RESULTCODE; 
		p += 2;
	
	//Lenght
		*reinterpret_cast<uint16_t*>(bufpck+p) = 4; 
		p += 2;

	//Result Code 
	//0 - Success
	//3 - Failure
		*reinterpret_cast<uint32_t*>(bufpck+p) = code; 
		p += 4;

// */
		*data=bufpck;
	}

	void CapwapPluginA::setSessionID (uint8_t** data, int* len, uint32_t SID)
	{
		int p;
		uint8_t* bufpck;
// /*
		if(*len!=0){
			p=*len;
			bufpck=new uint8_t[p + 8]; //Type+Length+Value
			memmove(bufpck,*data,p); //copia message element precedenti
			delete [] *data;
			
		//LEN TOT
			*len = p+8; 
		}
		else{
			p=0;
			bufpck=new uint8_t[8];

		//LEN TOT
			*len=8;
		}

	//Type
		*reinterpret_cast<uint16_t*>(bufpck+p) = CAPWAPMessage::SESSIONID; 
		p += 2;
	
	//Lenght
		*reinterpret_cast<uint16_t*>(bufpck+p) = 4; 
		p += 2;

	//Session ID
		*reinterpret_cast<uint32_t*>(bufpck+p) = SID; 
		p += 4;

		*data=bufpck;
	}
//-------------------------------------------------------------------------------------------------END Message Elemnt
//-------------------------------------------------------------------------------------------------START CAPWAP CONTROL PCK
	void CapwapPluginA::setDISCOVERYRESPONSE(EthernetFrame* ethf, EthernetFrame* e)
	{
		memmove (e, ethf,( sizeof(EthernetFrame)+sizeof(IPPacket)+sizeof(UDPDatagram)+CAPWAP_LEN+(ethf->IPPayload().UDPPayload().CAPWAPPayload().MsgElementLength)-7));
		//memmove (e, ethf, sizeof(EthernetFrame)+sizeof(IPPacket)+sizeof(UDPDatagram)+CAPWAP_LEN+1);
		
		
		//Vedi SendCONFIGURATIONUPDATEREQUEST in CAPWAP_plugin B
		// forgia il apcchetto da zero e poi dai all'indirizzo ethf il nuovo
		//pacchetto forgiato 
		//IPPacket *ipk;
		//UDPDatagram *udpk;
		//CAPWAPMessage *m;
		
		//EthernetFrame *ethout = new (ETHERMAXSIZE) EthernetFrame;		
		//e = new (ETHERMAXSIZE) EthernetFrame;
		e->setReplyHeader (ethf);
		IPPacket *ipk = &(e->IPPayload());
		ipk->setReplyHeader (&(ethf->IPPayload()));
		ipk->source = gkIP;
		UDPDatagram *udpk = &(ipk->UDPPayload());
		udpk->setReplyHeader (&(ethf->IPPayload().UDPPayload()));
		udpk->destinationPort=htons(outudp);
		udpk->sourcePort=htons(inudp);
		
		//CAPWAPMessage *m = new CAPWAPMessage();
		//udpk->payload = m;
		CAPWAPMessage *m = reinterpret_cast<CAPWAPMessage*>(udpk->payload);
		
		//udpk->payload = reinterpret_cast<>(m);
		//Set CAPWAP HEADER
		setCAPWAPHdr(m);

	//Set CAPWAP CONTROL HEADER
		(m->MessageType) = CAPWAPMessage::DISCOVERYRESPONSE;
	//(m->SeqNum): it is the same of REQUEST 
		
		//m->SeqNum=0xaa; //8-bit
		//m->MsgElementLength=0xbbbb; //16-bit
		m->Flags=0; //8-bit
		m->TimeStamp=0;  //32-bit
		//uint32_t bufint32=0xffffffff;
		//(m->TimeStamp) = bufint32;
	
	//Set Message Element
		uint8_t* bufme;
		int totlen=0;
		
		setACDescriptor(&bufme,&totlen);		
		setACName(&bufme,&totlen);
		
		/*		
		logger()<<"[CAPWAP] Type (1): "<<static_cast<uint16_t>(bufme[0])<<endl;
		logger()<<"[CAPWAP] Lenght (12): "<<static_cast<uint16_t>(bufme[2])<<endl;
		logger()<<"[CAPWAP] Type (4): "<<static_cast<uint16_t>(bufme[16])<<endl;
		logger()<<"[CAPWAP] Lenght (7): "<<static_cast<uint16_t>(bufme[16+2])<<endl;
		logger()<<"[CAPWAP] Data (ACUnify): "<<(bufme[16+4])<<(bufme[16+5])<<(bufme[16+6])<<(bufme[16+7])
		<<(bufme[16+8])<<(bufme[16+9])<<(bufme[16+10])<<(bufme[16+11])<<endl;
		*/
		
		(m->MsgElementLength) = 7 + totlen;
		//(uint8_t*) (m->MessageElement[0])  = new uint8_t[totlen];
		memmove(m->MessageElement,bufme,totlen);
		/*
		logger()<<"[CAPWAP] totlen (27): "<<totlen<<endl;
		logger()<<"[CAPWAP] Type (1): "<<static_cast<uint16_t>(m->MessageElement[0])<<endl;
		logger()<<"[CAPWAP] Lenght (12): "<<static_cast<uint16_t>(m->MessageElement[2])<<endl;
		logger()<<"[CAPWAP] Type (4): "<<static_cast<uint16_t>(m->MessageElement[16])<<endl;
		logger()<<"[CAPWAP] Lenght (7): "<<static_cast<uint16_t>(m->MessageElement[16+2])<<endl;
		//logger()<<"[CAPWAP] Data (ACUnify): "<<(m->MessageElement[16+4])<<(m->MessageElement[16+5])<<(m->MessageElement[16+6])<<(m->MessageElement[16+7])
		//<<(bufme[16+8])<<(m->MessageElement[16+9])<<(m->MessageElement[16+10])<<(m->MessageElement[16+11])<<endl;
		*/
		//int l=sizeof(CAPWAPMessage)+totlen;
		//memset (reinterpret_cast<char*>(m)+l, 0, l);

		ipk->length = htons ( CAPWAP_LEN + totlen + sizeof(IPPacket) + sizeof(UDPDatagram));
		ipk->calculateHeaderChecksum ();

		udpk->length = htons ( CAPWAP_LEN + totlen + sizeof(UDPDatagram));
		//udpk->calculateChecksum (ipk->source, ipk->destination, CAPWAP_LEN + totlen + sizeof(UDPDatagram));
		udpk->checksum=0;
		delete [] bufme;
		//delete [] ethf;
	}

	void CapwapPluginA::setJOINRESPONSE(EthernetFrame* ethf, EthernetFrame* e, uint32_t code, uint32_t SID)
	{
		//logger()<<"[CAPWAP] Dentro setJOINRESPONSE"<<endl;
		memmove (e, ethf, sizeof(EthernetFrame)+sizeof(IPPacket)+sizeof(UDPDatagram)+CAPWAP_LEN+(ethf->IPPayload().UDPPayload().CAPWAPPayload().MsgElementLength)-7); 
		//memmove (e, ethf, sizeof(EthernetFrame)+sizeof(IPPacket)+sizeof(UDPDatagram)+CAPWAP_LEN+1);
		//e = new (ETHERMAXSIZE) EthernetFrame;
		e->setReplyHeader (ethf);
		IPPacket *ipk = &(e->IPPayload());
		//e->setReplyHeader (ethf);
		//IPPacket *ipk = &(e->IPPayload());
		ipk->setReplyHeader (&(ethf->IPPayload()));
		ipk->source = gkIP;
		UDPDatagram *udpk = &(ipk->UDPPayload());
		udpk->setReplyHeader (&(ethf->IPPayload().UDPPayload()));
		udpk->destinationPort=htons(outudp);
		udpk->sourcePort=htons(inudp);
		CAPWAPMessage *m = reinterpret_cast<CAPWAPMessage*>(udpk->payload);

	//Set CAPWAP HEADER
		setCAPWAPHdr(m);

	//Set CAPWAP CONTROL HEADER
		(m->MessageType) = CAPWAPMessage::JOINRESPONSE;
		(m->Flags) = (0);
		(m->TimeStamp) = (0);

	//Set Message Element
		uint8_t* bufme=NULL;
		int totlen=0;
		
		setResultCode(&bufme,&totlen, code);
		setSessionID(&bufme,&totlen, SID);

		(m->MsgElementLength) = 7+totlen;
		//m->MessageElement=new uint8_t[totlen];
		memmove(m->MessageElement,bufme,totlen);
		//m->MessageElement=&bufme[1];

		ipk->length = htons ( CAPWAP_LEN + totlen + sizeof(IPPacket) + sizeof(UDPDatagram));
		ipk->calculateHeaderChecksum ();

		udpk->length = htons ( CAPWAP_LEN + totlen + sizeof(UDPDatagram));
		//udpk->calculateChecksum (ipk->source, ipk->destination, CAPWAP_LEN + totlen + sizeof(UDPDatagram));
		udpk->checksum=0;
		delete [] bufme;
		//delete [] ethf;
	}

//-------------------------------------------------------------------------------------------------END CAPWAP CONTROL PCK

	void CapwapPluginA::setCAPWAPHdr(CAPWAPMessage *m)
	{
		uint32_t zero=0;
		m->F1=zero;
		m->F2=zero;
		setHLEN(m,CAPWAP_LEN);
	}

	void CapwapPluginA::setVersion(CAPWAPMessage *m, uint8_t Version)
	{
		m->F1|=(Version&0x0f)<<28;
	}

	void CapwapPluginA::setRID(CAPWAPMessage *m, uint8_t RID)
	{
		m->F1|=(RID&0x1f)<<23;
	}

	void CapwapPluginA::setHLEN(CAPWAPMessage *m, uint8_t HLEN)
	{
		m->F1|=(HLEN&0x1f)<<18;
	}

	void CapwapPluginA::setWBID(CAPWAPMessage *m, uint8_t WBID)
	{
		m->F1|=(WBID&0x1f)<<13;
	}

	void CapwapPluginA::setT(CAPWAPMessage *m, uint8_t T)
	{
		m->F1|=(T&0x01)<<12;
	}

	void CapwapPluginA::setF(CAPWAPMessage *m, uint8_t F)
	{
		m->F1|=(F&0x01)<<11;
	}

	void CapwapPluginA::setL(CAPWAPMessage *m, uint8_t L)
	{
		m->F1|=(L&0x01)<<10;
	}

	void CapwapPluginA::setW(CAPWAPMessage *m, uint8_t W)
	{
		m->F1|=(W&0x01)<<9;
	}

	void CapwapPluginA::setM(CAPWAPMessage *m, uint8_t M)
	{
		m->F1|=(M&0x01)<<8;
	}

	void CapwapPluginA::setFlags(CAPWAPMessage *m, uint8_t Flags)
	{
		m->F1|=(Flags&0xff)<<0;
	}

	void CapwapPluginA::setFragmentId(CAPWAPMessage *m, uint16_t FragmentId)
	{
		m->F2|=(FragmentId&0xffff)<<16;
	}

	void CapwapPluginA::setFragmentOffset(CAPWAPMessage *m, uint16_t FragmentOffset)
	{
		m->F2|=(FragmentOffset&0x1fff)<<3;
	}

	void CapwapPluginA::setRsvd(CAPWAPMessage *m, uint8_t Rsvd)
	{
		m->F2|=(Rsvd&0x7)<<0;
	}

	uint32_t CapwapPluginA::getSID(uint8_t *me, uint16_t maxlen)
	{
		bool scan=true;
		uint16_t i=0;
		CAPWAPMessage::MessageElementType MEtype = CAPWAPMessage::ME_NONE;
		while(scan)
		{
			MEtype = static_cast<CAPWAPMessage::MessageElementType>(me[i]);
			//logger()<<"[CAPWAP] Dentro getSID, (static_cast<uint16_t>(me[i])): "<<(static_cast<uint16_t>(me[i]))<<endl;
			
			if(MEtype!=CAPWAPMessage::SESSIONID) //Type
			{
				i+=(static_cast<uint16_t>(me[i+2]))+4; //Length
			}
			else
			{
				return (static_cast<uint32_t>(me[i+4]));
			}
			if(i>=maxlen) scan=false;
		}
		return 0;
	}

	string CapwapPluginA::getWTPName(uint8_t *me, uint16_t maxlen)
	{
		bool scan=1;
		uint16_t i=0;
		string bufwtpname;
		while(scan)
		{
			if(static_cast<uint16_t>(me[i])!=CAPWAPMessage::WTPNAME) //Type
			{
				i+=(static_cast<uint16_t>(me[i+2]))+4; //Length
			}
			else
			{
				for (int a=0;a<(static_cast<uint16_t>(me[i+2]));a++)
				{
					bufwtpname.insert(bufwtpname.end(),static_cast<uint16_t>(me[2+2+i+a]));
				}
				return (bufwtpname);
			}
			if(i>=maxlen) scan=false;
		}
		return NULL;
	}

	string CapwapPluginA::getLocation(uint8_t *me, uint16_t maxlen)
	{
		bool scan=1;
		uint16_t i=0;
		string buflocation;
		while(scan)
		{
			if(static_cast<uint16_t>(me[i])!=CAPWAPMessage::LOCATIONDATA) //Type
			{
				i+=(static_cast<uint16_t>(me[i+2]))+4; //Length
			}
			else
			{
				for (int a=0;a<(static_cast<uint16_t>(me[i+2]));a++)
				{
					buflocation.insert(buflocation.end(),static_cast<uint16_t>(me[2+2+i+a]));
				}
				return (buflocation);
			}
			if(i>=maxlen) scan=false;
		}
		return NULL;
	}

	bool CapwapPluginA::AddJoinRequestInformation(IPv4Address newip,
	EthernetMACAddress newmac,
	uint32_t newsid)
	{
		if(pwtpti.AddJoinRequestInformation(newip,newmac,newsid))
 			return true; 
		return false;
	}


	bool CapwapPluginA::processPacket (EthernetFrame *ethf, EthernetFrame* v[], PacketSense sense[])
	{
		CAPWAPMessage::MessageTypes type = check (ethf);
		IPPacket *ipk = &(ethf->IPPayload());
		UDPDatagram *udpk;
		CAPWAPMessage *m;
		IPv4Address WTPIP=(ipk->source);
		//logger()<<"[CAPWAP] CapwapPluginA::processPacket!!!" << endl;
		switch ( type ) {
				
			case (CAPWAPMessage::DISCOVERYREQUEST):
				// send Discovery Response
				logger()<<"[CAPWAP] Discovery Request from "<< (WTPIP) << endl;
				//modify the array v with packets that the Dispatcher will send
				setDISCOVERYRESPONSE(ethf,v[0]);
				sense[0] = SEND_BACK;
				//sense[0] = SEND_SAME;
				sense[1] = END_OF_PACKETS;
				logger()<<"[CAPWAP] Send DISCOVERY RESPONSE"<< endl;
				return true;

			case (CAPWAPMessage::JOINREQUEST):
				// Verify: Max_WTP, save essential information of WTP, update status of WTP (Configure), send Join Response
				udpk = &(ipk->UDPPayload());
				m = reinterpret_cast<CAPWAPMessage*>(udpk->payload);
				logger()<<"[CAPWAP] Join Request from "<< (WTPIP) << endl;

				if(((pwtpti.GetNumberWTP())>(pwtpti.GetMaxNumberWTP())))
				{
					logger()<<"[CAPWAP] Join Request reject, too much WTP associated."<<endl;
					setJOINRESPONSE(ethf,v[0], 3, getSID(m->MessageElement,((m->MsgElementLength)-7))); // code=3 FAILURE
					sense[0] = SEND_BACK;
					//sense[0] = SEND_SAME;
					sense[1] = END_OF_PACKETS;
					logger()<<"[CAPWAP] Send  JOINRESPONSE (FAILURE)"<<endl;
					return true;
				}
				
				
				//logger()<<"[CAPWAP] Provo pwtpti.AddJoinRequestInformation"<<endl;
				
				if( pwtpti.AddJoinRequestInformation(
				WTPIP,
				ethf->sender,
				getSID(m->MessageElement,((m->MsgElementLength)-7))) )
				{
					setJOINRESPONSE(ethf,v[0], 0, getSID(m->MessageElement,((m->MsgElementLength)-7))); // code=0 SUCCESS
					sense[0] = SEND_BACK;
					//sense[0] = SEND_SAME;
					sense[1] = END_OF_PACKETS;
					//logger()<<"[CAPWAP] Provo pwtpti.SetStatus"<<endl;
					pwtpti.SetStatus(WTPIP, WTPTableInterface::CONFIGURE);
					logger()<<"[CAPWAP] Send JOINRESPONSE (SUCCESS)"<<endl;
				}
				else
				{
					setJOINRESPONSE(ethf,v[0], 3, getSID(m->MessageElement,((m->MsgElementLength)-7))); // code=3 FAILURE
					sense[0] = SEND_BACK;
					sense[1] = END_OF_PACKETS;
					logger()<<"[CAPWAP] Send  JOINRESPONSE (FAILURE)"<<endl;
				}
				return true;
				
			case (CAPWAPMessage::NONE):
				return false;

			default:
				break;
		}
		sense[0] = END_OF_PACKETS;
		return true;
	}
}


