/*##############################################################################
   
   Developed by NetMob Group, University of Trento, Italy

   University of Trento
   DIT - Department of Informatics and Telecommunications 
   Computer Networks and Mobility Research Program
   http://netmob.unitn.it/ 
   
   This file is part of the WilmaGate system, created as part
   of the WILMA Project, sponsored by the Autonomous
   Province of Trento and participated by ITC-irst, the University
   of Trento and Alpikom Spa. More info at

     http://www.wilmaproject.org/		

   To download the project:
      - in tarball format
        http://netmob.unitn.it/wilmagate.html
      - connect to CVS repository
	:pserver:cvs@rtm.unitn.it:/home/cvsroot
   In both previous cases, the download is password protected. To obtain
   the password you must send a mail to 

      wilmagate@dit.unitn.it

################################################################################

   The WilmaGate Licence of Use


   Copyright (c) 2003-2005
                - Universita` di Trento,
                  Dipartimento di Informatica e Telecomunicazioni (DIT)
                  Computer Networks and Mobility Research Program


   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (hereinafter
   the "WilmaGate") directly from the copyright owner, to use the WilmaGate
   without restriction of purpose or time, including without limitation the
   rights to use, copy, modify, and merge the WilmaGate, subject to the
   following conditions:

   The above copyright notice and this permission notice shall be included
   in all copies or substantial portions of the WilmaGate. 

   The WilmaGate shall not be sub-licenced either commercially or free of
   charge, as is or in part or as a part of any other package. 

   The licensee agrees to properly credit WilmaGate and its copyright 
   owners in any technical, scientific or divulgative publication, 
   whatever publishing means are used, that is based entirely or in part 
   on WilmaGate use, as well as in any booklet, web-site or brochure that 
   advertise services based on WilmaGate use. 

   If the WilmaGate is modified, the copyright owner shall be notified.

   THE WilmaGate IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

##############################################################################*/

//General Struct of Message Element:
//	-------------------------
//	| Type | Lenght | Value |
//	-------------------------
//	|  16  |   16   |   n   |
//	-------------------------
//
#include "../Common/utilities.h"
#include "capwap_plugin_B.h"


using namespace std;
using namespace Common;

namespace Gatekeeper {

	CapwapPluginB::CapwapPluginB(PluginStatusTableInterface &pluginStatusTableInterfaceObject, 
					WTPTableInterface &pluginWTPTableInterfaceObject,
					IPv4Address gateIpAddress,
					IPv4Address wayIpAddress,
					const char *domainName):
			gkIP(gateIpAddress),
	gwIP(wayIpAddress),
	domain(domainName),
	psti(pluginStatusTableInterfaceObject),
	pwtpti(pluginWTPTableInterfaceObject)
	{
		configurationupdatecount=0;
		rtl=NULL;
		inudp=pwtpti.Getinudpport();
		outudp=pwtpti.Getoutudpport();
		
		//psti.RegisterCAPWAPService (this);
	}

	CapwapPluginB::~CapwapPluginB (void)
	{
	}
	
	void CapwapPluginB::registerCAPWAPCONFService(RelayListener *pointerToDispatcher)
	{
		rtl=pointerToDispatcher;
	}

	CAPWAPMessage::MessageTypes CapwapPluginB::check(EthernetFrame *ethf)
	{
		CAPWAPMessage::MessageTypes type = CAPWAPMessage::NONE;
		CAPWAPMessage::MessageElementType MEtype = CAPWAPMessage::ME_NONE;
		bool scan=true;
		uint16_t p=0;
		bitset<4> cme; //Check message element
		if ( (ethf->isProtocol (EthernetFrame::IP_PROTOCOL))
				   && (ethf->IPPayload().protocol == IPPacket::UDP_PROTOCOL) )
				   //&& ethf->IPPayload().UDPPayload().destinationPort == htons(0x0043) ) //UDP Port!!!
		{
		
			CAPWAPMessage *m = reinterpret_cast<CAPWAPMessage*>(ethf->IPPayload().UDPPayload().payload);

		//Version Verify
			if(m->getVersion()!=0)
			{
				//logger()<<"[CAPWAP] CAPWAP Error Version"<<endl;
				return type;
			}

		//Only Configuration Status Request, Echo Request and configuration Update Response
			switch ( m->MessageType ) {
				case CAPWAPMessage::CONFIGURATIONSTATUSREQUEST:
					while ( p < ((m->MsgElementLength)-7) && scan) 
					{
						scan=false;
						MEtype = static_cast<CAPWAPMessage::MessageElementType>(m->MessageElement[p]);
						if (MEtype==CAPWAPMessage::WTPNAME){
							cme[0]=1; 
							p+=2;
							p+=static_cast<uint16_t>(m->MessageElement[p])+2;
							scan=true;
						}
						if (MEtype==CAPWAPMessage::LOCATIONDATA){
							cme[1]=1; 
							p+=2;
							p+=static_cast<uint16_t>(m->MessageElement[p])+2;
							scan=true;
						}
						if (MEtype==CAPWAPMessage::WTPSTATICIPADDRESSINFORMATION){
							cme[2]=1;
							p+=2;
							p+=static_cast<uint16_t>(m->MessageElement[p])+2;
							scan=true;
						}
						if (MEtype==CAPWAPMessage::WTPRADIOINFORMATION){
							cme[3]=1;
							p+=2;
							p+=static_cast<uint16_t>(m->MessageElement[p])+2;
							scan=true;
						}
						if( (cme[0]) && (cme[1]) && (cme[2]) && (cme[3]) ){
							type = CAPWAPMessage::CONFIGURATIONSTATUSREQUEST;
						}
					}
					break;
					
				case CAPWAPMessage::CONFIGURATIONUPDATERESPONSE:
					while ( p < ((m->MsgElementLength)-7) && scan) 
					{
						scan=false;
						MEtype = static_cast<CAPWAPMessage::MessageElementType>(m->MessageElement[p]);
						if (MEtype==CAPWAPMessage::RESULTCODE){
							cme[0]=1;
							p+=2;
							p+=static_cast<uint16_t>(m->MessageElement[p])+2;
							scan=true;
						}
						if( (cme[0]))
						{
							type = CAPWAPMessage::CONFIGURATIONUPDATERESPONSE;
						}
					}
					break;

				case CAPWAPMessage::ECHOREQUEST:
					type = CAPWAPMessage::ECHOREQUEST;
					break;

				default: 
					break;
			}
		}
	return type;
	}

	uint32_t CapwapPluginB::getResultCodeME(uint8_t *me, uint16_t maxlen)
	{
		CAPWAPMessage::MessageElementType MEtype = CAPWAPMessage::ME_NONE;
		uint16_t p=0;
		while (p < maxlen) 
		{
			MEtype = static_cast<CAPWAPMessage::MessageElementType>(me[p]);
			if(MEtype==CAPWAPMessage::RESULTCODE) //Type
			{				
				return((static_cast<uint32_t>(me[p+1])));
			}
			else
			{
				p+=2;
				p+=static_cast<uint16_t>(me[p+2]);
			}
		}
		return 3;
	}
	
	string CapwapPluginB::GetWTPNameME(uint8_t* me, uint16_t maxlen)
	{
	bool scan=1;
	uint16_t i=0;
	string bufwtpname;
	while(scan)
	{
		if(static_cast<uint16_t>(me[i])!=CAPWAPMessage::WTPNAME) //Type
		{
			i+=(static_cast<uint16_t>(me[i+2]))+4; //Length
		}
		else
		{
			for (int a=0;a<(static_cast<uint16_t>(me[i+2]));a++)
			{
				bufwtpname.insert(bufwtpname.end(),static_cast<uint16_t>(me[2+2+i+a]));
			}
			return (bufwtpname);
		}
		if(i>=maxlen) scan=false;
	}
	return NULL;
	}
	
	string CapwapPluginB::GetWTPLocationME(uint8_t* me, uint16_t maxlen)
	{
	bool scan=1;
	uint16_t i=0;
	string buflocation;
	while(scan)
	{
		if(static_cast<uint16_t>(me[i])!=CAPWAPMessage::LOCATIONDATA) //Type
		{
			i+=(static_cast<uint16_t>(me[i+2]))+4; //Length
		}
		else
		{
			for (int a=0;a<(static_cast<uint16_t>(me[i+2]));a++)
			{
				buflocation.insert(buflocation.end(),static_cast<uint16_t>(me[2+2+i+a]));
			}
			return (buflocation);
		}
		if(i>=maxlen) scan=false;
	}
	return NULL;
	}
	
	IPv4Address CapwapPluginB::GetWTPIPME(uint8_t* me, uint16_t maxlen)
	{
	bool scan=1;
	uint16_t i=0;
	while(scan)
	{
		if((static_cast<uint16_t>(me[i]))!=CAPWAPMessage::WTPSTATICIPADDRESSINFORMATION) //Type
		{
			i+=((static_cast<uint16_t>(me[i+2])))+4; //Length
		}
		else
		{
			return (IPv4Address(static_cast<uint16_t>(me[i+4])));
		}
		if(i>=maxlen) scan=false;
	}
	return IPv4Address();
	}

	IPv4Address CapwapPluginB::GetNetmaskME(uint8_t* me, uint16_t maxlen)
	{
	bool scan=1;
	uint16_t i=0;
	while(scan)
	{
		if((static_cast<uint16_t>(me[i]))!=CAPWAPMessage::WTPSTATICIPADDRESSINFORMATION) //Type
		{
			i+=((static_cast<uint16_t>(me[i+2])))+4; //Length
		}
		else
		{
			return (IPv4Address(static_cast<uint16_t>(me[i+4+4])));
		}
		if(i>=maxlen) scan=false;
	}
	return IPv4Address();
	}
	
	IPv4Address CapwapPluginB::GetGatewayME(uint8_t* me, uint16_t maxlen)
	{
	bool scan=1;
	uint16_t i=0;
	while(scan)
	{
		if((static_cast<uint16_t>(me[i]))!=CAPWAPMessage::WTPSTATICIPADDRESSINFORMATION) //Type
		{
			i+=((static_cast<uint16_t>(me[i+2])))+4; //Length
		}
		else
		{
			return (IPv4Address(static_cast<uint16_t>(me[i+4+4+4])));
		}
		if(i>=maxlen) scan=false;
	}
	return IPv4Address();
	}
	
	uint8_t CapwapPluginB::GetRadioID(uint8_t* me, uint16_t maxlen)
	{
	bool scan=1;
	uint16_t i=0;
	while(scan)
	{
		if((static_cast<uint16_t>(me[i]))!=CAPWAPMessage::WTPRADIOINFORMATION) //Type
		{
			i+=((static_cast<uint16_t>(me[i+2])))+4; //Length
		}
		else
		{
			return (static_cast<uint16_t>(me[i+4]));
		}
		if(i>=maxlen) scan=false;
	}
	return 0;
	}
	
	uint8_t CapwapPluginB::GetChannelME(uint8_t* me, uint16_t maxlen)
	{
	bool scan=1;
	uint16_t i=0;
	while(scan)
	{
		if((static_cast<uint16_t>(me[i]))!=CAPWAPMessage::WTPRADIOINFORMATION) //Type
		{
			i+=((static_cast<uint16_t>(me[i+2])))+4; //Length
		}
		else
		{
			return (static_cast<uint16_t>(me[i+4+1]));
		}
		if(i>=maxlen) scan=false;
	}
	return 0;
	}
	
	uint32_t CapwapPluginB::GetTxPowerME(uint8_t* me, uint16_t maxlen)
	{
	bool scan=1;
	uint16_t i=0;
	while(scan)
	{
		if((static_cast<uint16_t>(me[i]))!=CAPWAPMessage::WTPRADIOINFORMATION) //Type
		{
			i+=((static_cast<uint16_t>(me[i+2])))+4; //Length
		}
		else
		{
			return (static_cast<uint16_t>(me[i+4+1+1]));
		}
		if(i>=maxlen) scan=false;
	}
	return 0;
	}
	
	string CapwapPluginB::GetESSIDME(uint8_t* me, uint16_t maxlen)
	{
	bool scan=1;
	uint16_t i=0;
	string bufessid;
	while(scan)
	{
		if(static_cast<uint16_t>(me[i])!=CAPWAPMessage::WTPRADIOINFORMATION) //Type
		{
			i+=(static_cast<uint16_t>(me[i+2]))+4; //Length
		}
		else
		{
			for (int a=0;a<(static_cast<uint16_t>(me[i+2]));a++)
			{
				bufessid.insert(bufessid.end(),static_cast<uint16_t>(me[i+4+1+1+4]));
			}
			return (bufessid);
		}
		if(i>=maxlen) scan=false;
	}
	return NULL;
	}

	void CapwapPluginB::setCAPWAPTimers(uint8_t** data, int* len)
	{
		int p;
		uint8_t* bufpck;
		if(*len!=0){
			p=*len;
			bufpck=new uint8_t[p + 6]; //Type+Length+Value
			memmove(bufpck,*data,p); //copia message element precedenti
			delete [] *data;

		//LEN TOT
			*len=p+6;
		}
		else{
			p=0;
			bufpck=new uint8_t[6];
			*len=6;
		}

	//Type
		*reinterpret_cast<uint16_t*>(bufpck+p) = CAPWAPMessage::CAPWAPTIMERS; 
		p += 2;
	
	//Lenght
		*reinterpret_cast<uint16_t*>(bufpck+p) = 2; 
		p += 2;

	//Discovery
		*reinterpret_cast<uint8_t*>(bufpck+p) = 5; 
		//*reinterpret_cast<uint8_t*>(bufpck+p) = 1; 
		p += 1;

	//Echo Request
		*reinterpret_cast<uint8_t*>(bufpck+p) = 30; 
		//*reinterpret_cast<uint8_t*>(bufpck+p) = 1;
		p += 1;
		
		*data=bufpck;
	}

	void CapwapPluginB::setStatisticsTimer (uint8_t** data, int* len, uint16_t ST)
	{
		int p;
		uint8_t* bufpck;
		if(*len!=0){
			p=*len;
			bufpck=new uint8_t[p + 6]; //Type+Length+Value
			memmove(bufpck,*data,p); //copia message element precedenti
			delete [] *data;

		//LEN TOT
			*len= p+6; 
		}
		else{
			p=0;
			bufpck=new uint8_t[6];

		//LEN TOT
			*len=6;
		}

	//Type
		*reinterpret_cast<uint16_t*>(bufpck+p) = CAPWAPMessage::STATISTICSTIMER; 
		p += 2;
	
	//Lenght
		*reinterpret_cast<uint16_t*>(bufpck+p) = 2; 
		p += 2;

	//Statistics Timer
		*reinterpret_cast<uint16_t*>(bufpck+p) = 120; 
		p += 2;

		*data=bufpck;
	}
	void CapwapPluginB::setWTPName (uint8_t** data, int* len, string WTPName)
	{
		int p;
		uint8_t* bufpck;
		uint16_t StringLen=strlen(WTPName.c_str());
		if(*len!=0){
			p=*len;
			bufpck=new uint8_t[p + 4 + StringLen]; //Type+Length+Value
			memmove(bufpck,*data,p); //copia message element precedenti
			delete [] *data;
		//LEN TOT
			*len = p + 4 + StringLen;
		}
		else{
			p=0;
			bufpck=new uint8_t[4 + StringLen];

		//LEN TOT
			*len = (4 + StringLen); 
		}

	//Type
		*reinterpret_cast<uint16_t*>(bufpck+p) = CAPWAPMessage::WTPNAME; 
		p += 2;
	
	//Lenght
		*reinterpret_cast<uint16_t*>(bufpck+p) = StringLen; 
		p += 2;

	//WTPName
		memmove (reinterpret_cast<char*>(bufpck+p), WTPName.c_str(),StringLen);
		p += StringLen;

		*data=bufpck;
	}

	void CapwapPluginB::setLocationData (uint8_t** data, int* len, string Location)
	{
		int p;
		uint8_t* bufpck;
		uint16_t StringLen=strlen(Location.c_str());
		if(*len!=0){
			p=*len;
			bufpck=new uint8_t[p + 4 + StringLen]; //Type+Length+Value
			memmove(bufpck,*data,p); //copia message element precedenti
			delete [] *data;
		//LEN TOT
			*len = p + 4 + StringLen;
		}
		else{
			p=0;
			bufpck=new uint8_t[4 + StringLen];

		//LEN TOT
			*len = (4 + StringLen);
		}

	//Type
		*reinterpret_cast<uint16_t*>(bufpck+p) = CAPWAPMessage::LOCATIONDATA; 
		p += 2;
	
	//Lenght
		*reinterpret_cast<uint16_t*>(bufpck+p) = StringLen; 
		p += 2;

	//Location
		memmove (reinterpret_cast<char*>(bufpck+p), Location.c_str(),StringLen);
		p += StringLen;

		*data=bufpck;
	}

	void CapwapPluginB::SetWTPStaticIPAddress(uint8_t** data, int* len,IPv4Address newwtpip, IPv4Address newnetmask, IPv4Address newgateway)
	{
	int p;
	uint8_t* bufpck;
	
	if(*len!=0)
	{
		p=*len;
		bufpck=new uint8_t[p + 2+2+13]; //Type+Length+Value
		memmove(bufpck,*data,p); //copia message element precedenti
		delete [] *data;
	//LEN TOT
		*len = p+2+2+13; 
	}
	else
	{
		p=0;
		bufpck=new uint8_t[2+2+13];

	//LEN TOT
		*len=2+2+13; 
		
	}

	//Type
	*reinterpret_cast<uint16_t*>(bufpck+p) = CAPWAPMessage::WTPSTATICIPADDRESSINFORMATION; 
	p += 2;

	//Lenght
	*reinterpret_cast<uint16_t*>(bufpck+p) = 13; 
	p += 2;

	//WTP IPv4 Address
	*reinterpret_cast<uint32_t*>(bufpck+p) = newwtpip.asInteger();
	p += 4;
	
	//Netmask
	*reinterpret_cast<IPv4Address*>(bufpck+p) = newnetmask.asInteger(); 
	p += 4;
	
	//Gateway
	*reinterpret_cast<IPv4Address*>(bufpck+p) = newgateway.asInteger(); 
	p += 4;
	
	//Static
	*reinterpret_cast<uint8_t*>(bufpck+p) = 1; 
	p += 1;
	
	*data=bufpck;
	}
	
	void CapwapPluginB::SetWTPRadioInformation(uint8_t** data, int* len, uint8_t newrid, uint8_t newchannel, int32_t newtxpower,string newessid)
	{
		int p;
		uint8_t* bufpck;
		uint16_t StringLen=strlen(newessid.c_str());
		
		/*
		This message element don't respect CAPWAP specification
		*/
		
		if(*len!=0){
			p=*len;
			bufpck=new uint8_t[p + 2+2 + (1+1+4+StringLen)]; //Type+Length+Value + LEN TOT
			memmove(bufpck,*data,p); //copy message element before
			delete [] *data;
		//LEN TOT
			*len = p+2+2 + (1+1+4+StringLen); 
		}
		else{
			p=0;
			bufpck=new uint8_t[2+2 + (1+1+4+StringLen)];
	
		//LEN TOT
			*len = 2+2 + (1+1+4+StringLen);
		}
	
		//Type
		*reinterpret_cast<uint16_t*>(bufpck+p) = (uint16_t)CAPWAPMessage::WTPRADIOINFORMATION; 
		p += 2;
	
		//Lenght
		*reinterpret_cast<uint16_t*>(bufpck+p) = (1+1+4+StringLen); 
		p += 2;
		
		//Radio in use
		//*reinterpret_cast<uint8_t*>(bufpck+p) = newrid;
		*reinterpret_cast<uint8_t*>(bufpck+p) = 1; // !!!
		p += 1;
	
		//Channel
		*reinterpret_cast<uint8_t*>(bufpck+p) = newchannel;
		p += 1;
	
		//Tx Power
		*reinterpret_cast<uint32_t*>(bufpck+p) = newtxpower;
		p += 4;
		
		//Essid
		memmove (reinterpret_cast<char*>(bufpck+p), newessid.c_str(), StringLen );
		p += StringLen;
	
		*data=bufpck;
	}
//-------------------------------------------------------------------------------------------------END Message Elemnt
//-------------------------------------------------------------------------------------------------START CAPWAP CONTROL PCK
	void CapwapPluginB::setCONFIGURATIONSTATUSRESPONSE(EthernetFrame* ethf, EthernetFrame* e, uint16_t NewST)
	{
		memmove (e, ethf, sizeof(EthernetFrame)+sizeof(IPPacket)+sizeof(UDPDatagram)+CAPWAP_LEN+(ethf->IPPayload().UDPPayload().CAPWAPPayload().MsgElementLength)-7); 
		//memmove (e, ethf, sizeof(EthernetFrame)+sizeof(IPPacket)+sizeof(UDPDatagram)+CAPWAP_LEN+1); 
		
		e->setReplyHeader (ethf);
		IPPacket *ipk = &(e->IPPayload());
		ipk->setReplyHeader (&(ethf->IPPayload()));
		ipk->source = gkIP;
		UDPDatagram *udpk = &(ipk->UDPPayload());
		udpk->setReplyHeader (&(ethf->IPPayload().UDPPayload()));
		udpk->destinationPort=htons(outudp);
		udpk->sourcePort=htons(inudp);
		CAPWAPMessage *m = reinterpret_cast<CAPWAPMessage*>(udpk->payload);

	//Set CAPWAP HEADER
		setCAPWAPHdr(m);

	//Set CAPWAP CONTROL HEADER
		(m->MessageType) = CAPWAPMessage::CONFIGURATIONSTATUSRESPONSE;
		(m->Flags) = (0);
		(m->TimeStamp) = (0);

	//Set Message Element
		uint8_t* bufme;
		int totlen=0;
		setCAPWAPTimers(&bufme,&totlen);
		setStatisticsTimer(&bufme,&totlen, NewST);

		(m->MsgElementLength) = 7+totlen;
		//m->MessageElement=new uint8_t[totlen];
		memmove(m->MessageElement,bufme,totlen);

		ipk->length = htons ( CAPWAP_LEN + totlen+ sizeof(IPPacket) + sizeof(UDPDatagram));
		ipk->calculateHeaderChecksum ();

		udpk->length = htons ( CAPWAP_LEN + totlen + sizeof(UDPDatagram));
		//udpk->calculateChecksum (ipk->source, ipk->destination, CAPWAP_LEN + totlen + sizeof(UDPDatagram));
		udpk->checksum=0;
		logger()<<"[CAPWAP] Send CONFIGURATION STATUS RESPONSE"<<endl;
		delete [] bufme;
		//delete [] ethf;
		
	}

	void CapwapPluginB::setCONFIGURATIONUPDATEREQUEST(EthernetFrame* ethf, EthernetFrame* e, 
	string newwtpname, string newwtplocation,string newessid, 
	IPv4Address newwtpip, IPv4Address newnetmask,IPv4Address newgateway,
	uint8_t newrid, uint8_t newchannel,  uint32_t newtxpower)
	{
		//memmove (e, ethf, sizeof(EthernetFrame)+sizeof(IPPacket)+sizeof(UDPDatagram)+CAPWAP_LEN+(ethf->IPPayload().UDPPayload().CAPWAPPayload().MsgElementLength)-7); 
		memmove (e, ethf, sizeof(EthernetFrame)+sizeof(IPPacket)+sizeof(UDPDatagram)+CAPWAP_LEN+1); 

		e->setReplyHeader (ethf);
		IPPacket *ipk = &(e->IPPayload());
		ipk->setReplyHeader (&(ethf->IPPayload()));
		ipk->source = gkIP;
		UDPDatagram *udpk = &(ipk->UDPPayload());
		udpk->setReplyHeader (&(ethf->IPPayload().UDPPayload()));
		udpk->destinationPort=htons(outudp);
		udpk->sourcePort=htons(inudp);
		CAPWAPMessage *m = reinterpret_cast<CAPWAPMessage*>(udpk->payload);

	//Set CAPWAP HEADER
		setCAPWAPHdr(m);

	//Set CAPWAP CONTROL HEADER
		(m->MessageType) = CAPWAPMessage::CONFIGURATIONUPDATEREQUEST;
		(m->Flags) = (0);
		(m->TimeStamp) = (0);
		configurationupdatecount++;
		(m->SeqNum)=configurationupdatecount;
		
	//Set Message Element
		uint8_t* bufme;
		int totlen=0;
		setWTPName(&bufme,&totlen,newwtpname);
		setLocationData(&bufme,&totlen,newwtplocation);
		SetWTPStaticIPAddress(&bufme,&totlen,newwtpip,newnetmask,newgateway);
		SetWTPRadioInformation(&bufme,&totlen,newrid,newchannel,newtxpower,newessid);

		(m->MsgElementLength) = 7+totlen;
		//m->MessageElement=new uint8_t[totlen];
		memmove(m->MessageElement,bufme,totlen);

		ipk->length = htons ( CAPWAP_LEN + totlen + sizeof(IPPacket) + sizeof(UDPDatagram));
		ipk->calculateHeaderChecksum ();

		udpk->length = htons ( CAPWAP_LEN + totlen + sizeof(UDPDatagram));
		//udpk->calculateChecksum (ipk->source, ipk->destination, CAPWAP_LEN + totlen + sizeof(UDPDatagram));
		udpk->checksum=0;
		logger()<<"[CAPWAP] Send CONFIGURATION UPDATE REQUEST"<<endl;
		
		/*
		logger()<<"[CAPWAP] udpk->length (79): "<<ntohs(udpk->length)<<endl;
		logger()<<"[CAPWAP] checksum: "<<udpk->checksum<<endl;
		
		for(int y=0;y<ntohs(udpk->length)-8;y++)
		{
			cerr<<static_cast<char>(udpk->payload[y])<<"-";
		}
		cerr<<endl;
		*/
		delete [] bufme;
		//delete [] ethf;
	}

	void CapwapPluginB::setECHORESPONSE(EthernetFrame* ethf, EthernetFrame* e)
	{
		memmove (e, ethf, sizeof(EthernetFrame)+sizeof(IPPacket)+sizeof(UDPDatagram)+sizeof(CAPWAPMessage)+(ethf->IPPayload().UDPPayload().CAPWAPPayload().MsgElementLength)-7); 

		e->setReplyHeader (ethf);
		IPPacket *ipk = &(e->IPPayload());
		ipk->setReplyHeader (&(ethf->IPPayload()));
		ipk->source = gkIP;
		UDPDatagram *udpk = &(ipk->UDPPayload());
		udpk->setReplyHeader (&(ethf->IPPayload().UDPPayload()));
		udpk->destinationPort=htons(outudp);
		udpk->sourcePort=htons(inudp);
		CAPWAPMessage *m = reinterpret_cast<CAPWAPMessage*>(udpk->payload);

	//Set CAPWAP HEADER
		setCAPWAPHdr(m);

	//Set CAPWAP CONTROL HEADER
		(m->MessageType) = CAPWAPMessage::ECHORESPONSE;
		(m->Flags) = (0);
		(m->TimeStamp) = (0);

		(m->MsgElementLength) = 7;
		//m->MessageElement=NULL;

		ipk->length = htons ( CAPWAP_LEN + sizeof(IPPacket) + sizeof(UDPDatagram));
		ipk->calculateHeaderChecksum ();

		udpk->length = htons ( CAPWAP_LEN + sizeof(UDPDatagram));
		//udpk->calculateChecksum (ipk->source, ipk->destination, CAPWAP_LEN + sizeof(UDPDatagram));
		udpk->checksum=0;
		logger()<<"[CAPWAP] Send ECHO RESPONSE"<<endl;
		//delete [] bufme;
		//delete [] ethf;
		
	}
//-------------------------------------------------------------------------------------------------END CAPWAP CONTROL PCK

	void CapwapPluginB::setCAPWAPHdr(CAPWAPMessage *m)
	{
		uint32_t zero=0;
		m->F1|=zero;
		m->F2|=zero;
		setHLEN(m,CAPWAP_LEN);
	}

	void CapwapPluginB::setVersion(CAPWAPMessage *m, uint8_t Version)
	{
		m->F1|=(Version&0x0f)<<28;
	}

	void CapwapPluginB::setRID(CAPWAPMessage *m, uint8_t RID)
	{
		m->F1|=(RID&0x1f)<<23;
	}

	void CapwapPluginB::setHLEN(CAPWAPMessage *m, uint8_t HLEN)
	{
		m->F1|=(HLEN&0x1f)<<18;
	}

	void CapwapPluginB::setWBID(CAPWAPMessage *m, uint8_t WBID)
	{
		m->F1|=(WBID&0x1f)<<13;
	}

	void CapwapPluginB::setT(CAPWAPMessage *m, uint8_t T)
	{
		m->F1|=(T&0x01)<<12;
	}

	void CapwapPluginB::setF(CAPWAPMessage *m, uint8_t F)
	{
		m->F1|=(F&0x01)<<11;
	}

	void CapwapPluginB::setL(CAPWAPMessage *m, uint8_t L)
	{
		m->F1|=(L&0x01)<<10;
	}

	void CapwapPluginB::setW(CAPWAPMessage *m, uint8_t W)
	{
		m->F1|=(W&0x01)<<9;
	}

	void CapwapPluginB::setM(CAPWAPMessage *m, uint8_t M)
	{
		m->F1|=(M&0x01)<<8;
	}

	void CapwapPluginB::setFlags(CAPWAPMessage *m, uint8_t Flags)
	{
		m->F1|=(Flags&0xff)<<0;
	}

	void CapwapPluginB::setFragmentId(CAPWAPMessage *m, uint16_t FragmentId)
	{
		m->F2|=(FragmentId&0xffff)<<16;
	}

	void CapwapPluginB::setFragmentOffset(CAPWAPMessage *m, uint16_t FragmentOffset)
	{
		m->F2|=(FragmentOffset&0x1fff)<<3;
	}

	void CapwapPluginB::setRsvd(CAPWAPMessage *m, uint8_t Rsvd)
	{
		m->F2|=(Rsvd&0x7)<<0;
	}

	bool CapwapPluginB::AddConfigurationStatusRequestInformation(
	IPv4Address newip,
	string newwtpname,
	string newwtplocation,
	uint8_t newrid,
	uint8_t newchannel,
	uint32_t newtxpower,
	string newessid)
	{
		if(pwtpti.AddConfigurationStatusRequestInformation(
			newip,
			newwtpname,
			newwtplocation,
			newrid,
			newchannel,
			newtxpower,
			newessid))
			return true;
		return false;
	}
	
	bool CapwapPluginB::RefreshWTP(IPv4Address newip)
	{
		if( pwtpti.RefreshWTP(newip) )
			return true;
		return false;
	}
	
	bool CapwapPluginB::SendCONFIGURATIONUPDATEREQUEST(IPv4Address newwtp)
	{
		EthernetFrame *ethout;
		
		IPPacket *ipout;
	
		UDPDatagram *udpout;
		
		CAPWAPMessage *capwapout;
		
		uint8_t* bufme;
		int totlen=0;
		
		ethout = new (ETHERMAXSIZE) EthernetFrame;
		ipout = &(ethout->IPPayload());
		udpout = &(ipout->UDPPayload());
		capwapout = reinterpret_cast<CAPWAPMessage*>(udpout->payload);
		
		logger()<<"[CAPWAP] CAPWAP Plugin B must update configuration of WTP: "<<newwtp<< endl;
		
		ethout->sender = "00:00:00:00:00:00"; // !!!
		ethout->receiver = pwtpti.GetMAC(newwtp);
		ethout->protocol = htons(EthernetFrame::IP_PROTOCOL);

		//The type of ip packet is IPv4.
		ipout->version= IPv4;

		//Setting Differentiated Services to 0.
		ipout->diffserv=0;
			
		//Setting IP Identification Field to 0.
		ipout->id=0;

		//Setting IP fragment offset to 0.
		ipout->offset=0;

		//Setting IP field time to live to 0.
		//The same value set in server responses relayed by ISC DHCP Relay.
		ipout->ttl = TTL;
		
		//Setting IP field protocol type.
		ipout->protocol=IPPacket::UDP_PROTOCOL;

		ipout->source = gkIP;

		ipout->destination = newwtp;
				
		//Port from which the message is arrived.
		//myudp->sourcePort= htons(senderPort);
		udpout->sourcePort= htons(inudp);//htons(getServerPort());
		
		//The server answer is directed to bootpc port (68).
		udpout->destinationPort= htons(outudp);//htons(getClientPort());
		
		//Set CAPWAP HEADER
		setCAPWAPHdr(capwapout);

	//Set CAPWAP CONTROL HEADER
		(capwapout->MessageType) = CAPWAPMessage::CONFIGURATIONUPDATEREQUEST;
		(capwapout->Flags) = (0);
		(capwapout->TimeStamp) = (0);
		configurationupdatecount++;
		(capwapout->SeqNum)=configurationupdatecount;
		
	//Set Message Element		
		setWTPName(&bufme,&totlen,pwtpti.GetWTPName(newwtp));
		setLocationData(&bufme,&totlen,pwtpti.GetLocation(newwtp));
		SetWTPStaticIPAddress(&bufme,&totlen,newwtp,pwtpti.GetWTPNetmask(),pwtpti.GetWTPGateway());
		SetWTPRadioInformation(&bufme,&totlen,pwtpti.GetRadioID(newwtp),pwtpti.GetChannel(newwtp),pwtpti.GetTxPower(newwtp),pwtpti.GetWTPESSID());

		(capwapout->MsgElementLength) = 7+totlen;
		//capwapout->MessageElement=new uint8_t[totlen];
		memmove(capwapout->MessageElement,bufme,totlen);

		ipout->length = htons ( CAPWAP_LEN + totlen + sizeof(IPPacket) + sizeof(UDPDatagram));
		ipout->calculateHeaderChecksum ();

		udpout->length = htons ( CAPWAP_LEN + totlen + sizeof(UDPDatagram));
		//udpout->calculateChecksum (ipout->source, ipout->destination, CAPWAP_LEN + totlen + sizeof(UDPDatagram));
		udpout->checksum=0;
		
		if(rtl)
		{			
			//A dispatcher method is invoked
			rtl->sendResponseFrame(ethout, CAPWAP_LEN + totlen + sizeof(UDPDatagram)+sizeof(IPPacket)+sizeof(EthernetFrame));
			logger()<<"[CAPWAP] CAPWAP Plugin B sends CONFIGURATION UPDATE REQUEST"<< endl;
		}
		return true;
	}
	
	bool CapwapPluginB::processPacket (EthernetFrame *ethf, EthernetFrame* v[], PacketSense sense[])
	{
		CAPWAPMessage::MessageTypes type = check (ethf);
		IPPacket *ipk = &(ethf->IPPayload());
		UDPDatagram *udpk;
		CAPWAPMessage *m;
		IPv4Address WTPIP;
		udpk = &(ipk->UDPPayload());
		m = reinterpret_cast<CAPWAPMessage*>(udpk->payload);
		WTPIP=ipk->source;
		switch ( type ) {
		
			case CAPWAPMessage::CONFIGURATIONUPDATERESPONSE:
			
				logger()<<"[CAPWAP] Configuration Update Response from "<< WTPIP << endl;

				if( (getResultCodeME(m->MessageElement,((m->MsgElementLength)-7)))==0 )
				{
					logger()<<"[CAPWAP] Preconfiguration of "<<WTPIP<<" success" << endl;
					if(!(pwtpti.SetPreConfigure(WTPIP,WTPTableInterface::ACTIVEPRECONFIGURATION)))
						logger()<<"[CAPWAP] pwtpti.SetPreConfigure Error"<<endl;
						
					//RefreshWTP(WTPIP);
					//pwtpti.SetStatus(WTPIP,WTPTableInterface::RUN);
					
				}
				else
				{
					logger()<<"[CAPWAP] Preconfiguration of "<<WTPIP<<" failed" << endl;
					pwtpti.SetPreConfigure(WTPIP,WTPTableInterface::ERRORPRECONFIGURATION);
					//RefreshWTP(WTPIP);
					//pwtpti.SetStatus(WTPIP,WTPTableInterface::RUN);
				}
				sense[0] =  END_OF_PACKETS;
				
				return true;
				break;
			
			case CAPWAPMessage::CONFIGURATIONSTATUSREQUEST:
			
				logger()<<"[CAPWAP] Configuration Status Request from "<< (ipk->source) << endl;
				
				WTPTableInterface::StatusConfiguration soc;
				soc=pwtpti.GetPreConfigure(WTPIP);
				
				switch (soc)
				{
					case WTPTableInterface::NOPRECONFIGURATION :
					
						logger()<<"[CAPWAP] WTP: "<<WTPIP<<" has not the pre-configuration"<<endl;
						if(pwtpti.AddConfigurationStatusRequestInformation(
						WTPIP,
						GetWTPNameME(m->MessageElement,((m->MsgElementLength)-7)),
						GetWTPLocationME(m->MessageElement,((m->MsgElementLength)-7)),
						GetRadioID(m->MessageElement,((m->MsgElementLength)-7)),
						GetChannelME(m->MessageElement,((m->MsgElementLength)-7)),
						GetTxPowerME(m->MessageElement,((m->MsgElementLength)-7)),
						GetESSIDME(m->MessageElement,((m->MsgElementLength)-7))) )
						{
							setCONFIGURATIONSTATUSRESPONSE(ethf, v[0],120);
							RefreshWTP(WTPIP);
							pwtpti.SetStatus(WTPIP, WTPTableInterface::RUN);
						}
						else
						{
							logger()<<"[CAPWAP] pwtpti.AddCSRI Error"<<endl;
							sense[0] = END_OF_PACKETS;
							return true;
						}
						sense[0] = SEND_BACK;
						sense[1] = END_OF_PACKETS;
						return true;
					break;
					
					case WTPTableInterface::NOACTIVEPRECONFIGURATION :
					
						logger()<<"[CAPWAP] WTP: "<<WTPIP<<" has the pre-configuration"<<endl;
					
						setCONFIGURATIONUPDATEREQUEST(ethf, v[0],
						pwtpti.GetWTPName(WTPIP), 
						pwtpti.GetLocation(WTPIP),
						pwtpti.GetWTPESSID(), 
						WTPIP, 
						pwtpti.GetWTPNetmask(),
						pwtpti.GetWTPGateway(), 
						pwtpti.GetRadioID(WTPIP),
						pwtpti.GetChannel(WTPIP),
						pwtpti.GetTxPower(WTPIP) );
						
						sense[0] = SEND_BACK;
						sense[1] = END_OF_PACKETS;
						return true;
					break;
					
					case WTPTableInterface::ACTIVEPRECONFIGURATION :
						logger()<<"[CAPWAP] WTP: "<<WTPIP<<" support pre-configuration"<<endl;
						if(pwtpti.AddConfigurationStatusRequestInformation(
						WTPIP,
						GetWTPNameME(m->MessageElement,((m->MsgElementLength)-7)),
						GetWTPLocationME(m->MessageElement,((m->MsgElementLength)-7)),
						GetRadioID(m->MessageElement,((m->MsgElementLength)-7)),
						GetChannelME(m->MessageElement,((m->MsgElementLength)-7)),
						GetTxPowerME(m->MessageElement,((m->MsgElementLength)-7)),
						GetESSIDME(m->MessageElement,((m->MsgElementLength)-7))) )
						{
							setCONFIGURATIONSTATUSRESPONSE(ethf, v[0],120);
							RefreshWTP(WTPIP);
							pwtpti.SetStatus(WTPIP, WTPTableInterface::RUN);
						}
						else
						{
							logger()<<"[CAPWAP] pwtpti.AddCSRI Error"<<endl;
							sense[0] = END_OF_PACKETS;
							return true;
						}
						sense[0] = SEND_BACK;
						sense[1] = END_OF_PACKETS;
						return true;

					break;
					case WTPTableInterface::ERRORPRECONFIGURATION :
						logger()<<"[CAPWAP] WTP: "<<WTPIP<<" not support the pre-configuration, its configuration is accepted"<<endl;
						if(pwtpti.AddConfigurationStatusRequestInformation(
						WTPIP,
						GetWTPNameME(m->MessageElement,((m->MsgElementLength)-7)),
						GetWTPLocationME(m->MessageElement,((m->MsgElementLength)-7)),
						GetRadioID(m->MessageElement,((m->MsgElementLength)-7)),
						GetChannelME(m->MessageElement,((m->MsgElementLength)-7)),
						GetTxPowerME(m->MessageElement,((m->MsgElementLength)-7)),
						GetESSIDME(m->MessageElement,((m->MsgElementLength)-7))) )
						{
							setCONFIGURATIONSTATUSRESPONSE(ethf, v[0],120);
							RefreshWTP(WTPIP);
							pwtpti.SetStatus(WTPIP, WTPTableInterface::RUN);
						}
						else
						{
						logger()<<"[CAPWAP] pwtpti.AddCSRI Error"<<endl;
						sense[0] = END_OF_PACKETS;
						return true;
						}
						sense[0] = SEND_BACK;
						sense[1] = END_OF_PACKETS;
						return true;
					break;
					
					case WTPTableInterface::PENDINGPRECONFIGURATION :
						logger()<<"[CAPWAP] WTP: "<<WTPIP<<" has a pending pre-configuration, configuration update response packet may be lost"<<endl;
						sense[0] = END_OF_PACKETS;
						return true;
					break;
				}
			case CAPWAPMessage::ECHOREQUEST:
		// Verify that WTP is in RUN state and send ECHORESPONSE
				ipk = &(ethf->IPPayload());
				if(pwtpti.GetStatus(WTPIP)!=WTPTableInterface::RUN)
				{
					logger()<<"[CAPWAP] Echo Request from: "<<WTPIP<<" but it is not in RUN state"<< endl;
					sense[0] = END_OF_PACKETS;
					return true;
				}
				logger()<<"[CAPWAP] Echo Request from "<< (WTPIP)<< endl;
				RefreshWTP(WTPIP);
				
				setECHORESPONSE(ethf,v[0]);

				sense[0] = SEND_BACK;
				sense[1] = END_OF_PACKETS;
				return true;
				break;

			case CAPWAPMessage::NONE:
				return false;
				break;
			default:
				break;
		}

		sense[0] = END_OF_PACKETS;
		return true;
	}
}

