/*##############################################################################
   
   Developed by NetMob Group, University of Trento, Italy

   University of Trento
   DIT - Department of Informatics and Telecommunications 
   Computer Networks and Mobility Research Program
   http://netmob.unitn.it/ 
   
   This file is part of the WilmaGate system, created as part
   of the WILMA Project, sponsored by the Autonomous
   Province of Trento and participated by ITC-irst, the University
   of Trento and Alpikom Spa. More info at

     http://www.wilmaproject.org/		

   To download the project:
      - in tarball format
        http://netmob.unitn.it/wilmagate.html
      - connect to CVS repository
        :pserver:cvs@rtm.unitn.it:/home/cvsroot
   In both previous cases, the download is password protected. To obtain
   the password you must send a mail to 

      wilmagate@dit.unitn.it

################################################################################

   The WilmaGate Licence of Use


   Copyright (c) 2003-2005
                - Universita` di Trento,
                  Dipartimento di Informatica e Telecomunicazioni (DIT)
                  Computer Networks and Mobility Research Program


   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (hereinafter
   the "WilmaGate") directly from the copyright owner, to use the WilmaGate
   without restriction of purpose or time, including without limitation the
   rights to use, copy, modify, and merge the WilmaGate, subject to the
   following conditions:

   The above copyright notice and this permission notice shall be included
   in all copies or substantial portions of the WilmaGate. 

   The WilmaGate shall not be sub-licenced either commercially or free of
   charge, as is or in part or as a part of any other package. 

   The licensee agrees to properly credit WilmaGate and its copyright 
   owners in any technical, scientific or divulgative publication, 
   whatever publishing means are used, that is based entirely or in part 
   on WilmaGate use, as well as in any booklet, web-site or brochure that 
   advertise services based on WilmaGate use. 

   If the WilmaGate is modified, the copyright owner shall be notified.

   THE WilmaGate IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

##############################################################################*/

#include "../Common/utilities.h"
#include "dhcp_plugin.h"

using namespace std;
using namespace Common;

namespace Gatekeeper {

DhcpPlugin::DhcpPlugin(PluginStatusTableInterface &p, 
					IPv4Address gIp,
					IPv4Address wIp,
					IPv4Address DNSIPAddress,
					IPv4Address clientNetmask,
					const char *domainName,
					const char *wfile,
					int sl,
					int ll):
	shortLease (sl),
	longLease (ll),
	psti(p),
	gkIP(gIp),
	gwIP(wIp),
	dnsIP (DNSIPAddress),
	netmask (clientNetmask),
	domain (domainName),
	wpad_file (wfile),
	wpad_exists (wfile && *wfile > ' ')
{
}

DhcpPlugin::~DhcpPlugin (void)
{
}

bool DhcpPlugin::processPacket (EthernetFrame *ethf, EthernetFrame* v[], PacketSense sense[])
{
	IPv4Address ip;

	DHCPMessage::OptionMessageType type = check (ethf, ip);

	switch ( type ) {

	case DHCPMessage::DISCOVER:

		// if the frame is a DHCP Discover packet
		// a new IP is required by the user and the array of Etehrent Frame
		// must be set with the proper packets to send
		// Inquire the StatusTable to abtain an IP address

		if( (ip = psti.getFreeIP (ethf->sender)) != static_cast<uint32_t>(0) ) {

			StatusTableInterface::statusType status = psti.getStatus (ip);
			time_t lease = static_cast<time_t>((status == StatusTableInterface::AUTHORIZED) ? longLease : shortLease);

			logger() << "Offering IP " << ip << " to " << ethf->sender << endl;

			//modify the array v with packets that the Dispatcher will send
			setOffer (ethf, ip, v[0], lease);
			v[1]->protocol = 0;

			if ( status < StatusTableInterface::AUTHORIZED )
				psti.setStatus(ip, StatusTableInterface::FIRST_SEEN, true);
			psti.setExpirationIP(ip, lease);
			sense[0] = SEND_BACK;
			sense[1] = END_OF_PACKETS;
		}
		else
			sense[0] = END_OF_PACKETS;

		return true;

	case DHCPMessage::INFORM:

		// if the frame is a DHCP Inform packet
		// inquire the StatusTable to confirm the IP address
		if ( psti.checkIP(ethf->sender, ip) ) {
			setInfoAck(ethf, ip, v[0]);
			sense[0] = SEND_BACK;
			sense[1] = END_OF_PACKETS;
		}
		else
			sense[0] = END_OF_PACKETS;
		return true;

	case DHCPMessage::REQUEST:

		// if the frame is a DHCP Request packet
		// inquire the StatusTable to confirm the IP address
		if ( psti.checkIP(ethf->sender, ip) ) {
			
			StatusTableInterface::statusType status = psti.getStatus (ip);
			
			//DHCP albino here
			psti.setIPMode(ip,true);
			
			if ( status < StatusTableInterface::AUTHORIZED ) { //albino here
			
				if(psti.isPreAuthenticated(ethf->sender,ip))  //albino here
					psti.setStatus(ip ,  StatusTableInterface::AUTHORIZED); 
				else 
				{ 
					psti.setStatus(ip ,  StatusTableInterface::TEMPORARY_LEASE,false);
				}
			}
			
			status = psti.getStatus (ip);
			time_t lease = static_cast<time_t>((status == StatusTableInterface::AUTHORIZED) ? longLease : shortLease);
			psti.setExpirationIP(ip, lease);
			logger() << "Acknowledging IP " << ip << " requested by " << ethf->sender << endl;
			
			
              
			//modify the array v with packets that the Dispatcher will send
			setAck(ethf, ip, v[0], lease);

			/*
			if ( status < StatusTableInterface::AUTHORIZED ) 
			{	
				psti.setStatus(ethf->IPPayload().source, StatusTableInterface::TEMPORARY_LEASE);
			}
			*/


			
			
		}
		else
			setNack (ethf, ip, v[0]);

		if ( type == DHCPMessage::INFORM )
			v[0]->IPPayload().destination = ethf->IPPayload().source;
		sense[0] = SEND_BACK;
		sense[1] = END_OF_PACKETS;
		return true;


	case DHCPMessage::NONE:

		return false;

	default:
		break;

	}

	sense[0] = END_OF_PACKETS;
	return true;

}

DHCPMessage::OptionMessageType DhcpPlugin::check (EthernetFrame *ethf, IPv4Address &reqIP)
{
	DHCPMessage::OptionMessageType type = DHCPMessage::NONE;

	if ( ethf->isProtocol (EthernetFrame::IP_PROTOCOL)
			&& ethf->IPPayload().protocol == IPPacket::UDP_PROTOCOL
			&& ethf->IPPayload().UDPPayload().destinationPort == htons(0x0043) ) {

		DHCPMessage *m = reinterpret_cast<DHCPMessage*>(ethf->IPPayload().UDPPayload().payload);

		if ( m->op != DHCPMessage::QUERY )
			return DHCPMessage::NONE;

		reqIP = m->ciaddr;
		int p;
		p = 0;
		while ( p < 1000 && m->options[p] != DHCPMessage::END_OPTIONS ) {

			switch ( m->options[p] ) {

			case DHCPMessage::MESSAGE_TYPE:

				type = static_cast<DHCPMessage::OptionMessageType>(m->options[p+2]);
				// We are just interested in DISCOVER, INFORM and REQUEST packets.
				// Only the REQUEST packet deserves further exploration.
				if ( type != DHCPMessage::REQUEST && type != DHCPMessage::INFORM )
					return type;
				break;

			case DHCPMessage::REQUESTED_ADDRESS:

				if ( !reqIP.asInteger() )
					reqIP = *reinterpret_cast<IPv4Address*>(m->options + p + 2);
				break;

			}
			p += 2 + m->options[p+1];
		}
	}
	return type;
}



void DhcpPlugin::setOffer(EthernetFrame* ethf, IPv4Address ip, EthernetFrame* e, time_t lease)
{
	memcpy (e, ethf, sizeof(EthernetFrame)+sizeof(IPPacket)+sizeof(UDPDatagram)+sizeof(DHCPMessage));

	e->setReplyHeader (ethf);
	IPPacket *ipk = &(e->IPPayload());
	ipk->setReplyHeader (&(ethf->IPPayload()));
	ipk->source = gkIP;
	ipk->destination = 0xFFFFFFFFUL;
	UDPDatagram *udpk = &(ipk->UDPPayload());
	udpk->setReplyHeader (&(ethf->IPPayload().UDPPayload()));
	DHCPMessage *m = reinterpret_cast<DHCPMessage*>(udpk->payload);

	m->op = DHCPMessage::RESPONSE;
	m->flags = 0x0080;
	m->yiaddr = ip;
	m->siaddr = gkIP;
	int p = 0;

	m->options[p++] = DHCPMessage::MESSAGE_TYPE;
	m->options[p++] = 1;
	m->options[p++] = DHCPMessage::OFFER;

	m->options[p++] = DHCPMessage::SERVER_ID;
	m->options[p++] = 4;
	*reinterpret_cast<IPv4Address*>(m->options+p) = gkIP;
	p += 4;

	m->options[p++] = DHCPMessage::LEASE_TIME;
	m->options[p++] = 4;
	*reinterpret_cast<uint32_t*>(m->options+p) = htonl(lease);
	p += 4;
	
	m->options[p++] = DHCPMessage::RENEWAL_TIME;
	m->options[p++] = 4;
	*reinterpret_cast<uint32_t*>(m->options+p) = htonl(lease/2);
	p += 4;
	
	m->options[p++] = DHCPMessage::SUBNET_MASK;
	m->options[p++] = 4;
	*reinterpret_cast<IPv4Address*>(m->options+p) = netmask;
	p += 4;

	m->options[p++] = DHCPMessage::ROUTER;
	m->options[p++] = 4;
	*reinterpret_cast<IPv4Address*>(m->options+p) = gwIP;
	p += 4;

	m->options[p++] = DHCPMessage::DNS;
	m->options[p++] = 4;
	*reinterpret_cast<IPv4Address*>(m->options+p) = dnsIP;
	p += 4;

	m->options[p++] = DHCPMessage::DOMAIN_NAME;
	m->options[p++] = domain.size();
	strcpy (reinterpret_cast<char*>(m->options+p), domain.c_str());
	p += m->options[p-1];

	if ( wpad_exists ) {
		m->options[p++] = DHCPMessage::WPAD_URL;
		m->options[p++] = wpad_file.length()+1;
		strcpy ((char*)(m->options+p), wpad_file.c_str());
		*reinterpret_cast<char*>(m->options+p+wpad_file.length()) = 0x0a;
		p += m->options[p-1];
	}

	m->options[p++] = DHCPMessage::END_OPTIONS;

	int l = sizeof(DHCPMessage) + p;
	memset (reinterpret_cast<char*>(m)+l, 0, DHCPMessage::PACKET_LENGTH - l);

	ipk->length = htons (DHCPMessage::PACKET_LENGTH + sizeof(IPPacket) + sizeof(UDPDatagram));
	ipk->calculateHeaderChecksum ();

	udpk->length = htons (DHCPMessage::PACKET_LENGTH + sizeof(UDPDatagram));
	udpk->calculateChecksum (ipk->source, ipk->destination, DHCPMessage::PACKET_LENGTH + sizeof(UDPDatagram));
}


void DhcpPlugin::setAck(EthernetFrame* ethf, IPv4Address ip, EthernetFrame* e, time_t lease)
{
	memcpy (e, ethf, sizeof(EthernetFrame)+sizeof(IPPacket)+sizeof(UDPDatagram)+sizeof(DHCPMessage));

	e->setReplyHeader (ethf);
	IPPacket *ipk = &(e->IPPayload());
	ipk->setReplyHeader (&(ethf->IPPayload()));
	ipk->source = gkIP;
	ipk->destination = 0xFFFFFFFFUL;
	UDPDatagram *udpk = &(ipk->UDPPayload());
	udpk->setReplyHeader (&(ethf->IPPayload().UDPPayload()));
	DHCPMessage *m = reinterpret_cast<DHCPMessage*>(udpk->payload);

	m->op = DHCPMessage::RESPONSE;
	m->flags = 0x0080;
	m->yiaddr = ip;
	m->siaddr = gkIP;
	int p = 0;

	m->options[p++] = DHCPMessage::MESSAGE_TYPE;
	m->options[p++] = 1;
	m->options[p++] = DHCPMessage::ACK;

	m->options[p++] = DHCPMessage::SERVER_ID;
	m->options[p++] = 4;
	*reinterpret_cast<IPv4Address*>(m->options+p) = gkIP;
	p += 4;

	m->options[p++] = DHCPMessage::LEASE_TIME;
	m->options[p++] = 4;
	*reinterpret_cast<uint32_t*>(m->options+p) = htonl(lease);
	p += 4;
	
	m->options[p++] = DHCPMessage::RENEWAL_TIME;
	m->options[p++] = 4;
	*reinterpret_cast<uint32_t*>(m->options+p) = htonl(lease/2);
	p += 4;
	
	m->options[p++] = DHCPMessage::SUBNET_MASK;
	m->options[p++] = 4;
	*reinterpret_cast<IPv4Address*>(m->options+p) = netmask;
	p += 4;

	m->options[p++] = DHCPMessage::ROUTER;
	m->options[p++] = 4;
	*reinterpret_cast<IPv4Address*>(m->options+p) = gwIP;
	p += 4;

	m->options[p++] = DHCPMessage::DNS;
	m->options[p++] = 4;
	*reinterpret_cast<IPv4Address*>(m->options+p) = dnsIP;
	p += 4;

	m->options[p++] = DHCPMessage::DOMAIN_NAME;
	m->options[p++] = domain.size();
	strcpy (reinterpret_cast<char*>(m->options+p), domain.c_str());
	p += m->options[p-1];

	if ( wpad_exists ) {
		m->options[p++] = DHCPMessage::WPAD_URL;
		m->options[p++] = wpad_file.length()+1;
		strcpy ((char*)(m->options+p), wpad_file.c_str());
		*reinterpret_cast<char*>(m->options+p+wpad_file.length()) = 0x0a;
		p += m->options[p-1];
	}

	m->options[p++] = DHCPMessage::END_OPTIONS;

	int l = sizeof(DHCPMessage) + p;
	memset (reinterpret_cast<char*>(m)+l, 0, DHCPMessage::PACKET_LENGTH - l);

	ipk->length = htons (DHCPMessage::PACKET_LENGTH + sizeof(IPPacket) + sizeof(UDPDatagram));
	ipk->calculateHeaderChecksum ();

	udpk->length = htons (DHCPMessage::PACKET_LENGTH + sizeof(UDPDatagram));
	udpk->calculateChecksum (ipk->source, ipk->destination, DHCPMessage::PACKET_LENGTH + sizeof(UDPDatagram));
}




void DhcpPlugin::setInfoAck(EthernetFrame* ethf, IPv4Address ip, EthernetFrame* e)
{
	memcpy (e, ethf, sizeof(EthernetFrame)+sizeof(IPPacket)+sizeof(UDPDatagram)+sizeof(DHCPMessage));

	e->setReplyHeader (ethf);
	IPPacket *ipk = &(e->IPPayload());
	ipk->setReplyHeader (&(ethf->IPPayload()));
	ipk->source = gkIP;
	UDPDatagram *udpk = &(ipk->UDPPayload());
	udpk->setReplyHeader (&(ethf->IPPayload().UDPPayload()));

	DHCPMessage *m = reinterpret_cast<DHCPMessage*>(udpk->payload);

	m->op = DHCPMessage::RESPONSE;
	m->flags = 0;
	//m->yiaddr = ip;
	m->siaddr = gkIP;
	int p = 0;

	m->options[p++] = DHCPMessage::MESSAGE_TYPE;
	m->options[p++] = 1;
	m->options[p++] = DHCPMessage::ACK;

	m->options[p++] = DHCPMessage::SERVER_ID;
	m->options[p++] = 4;
	*reinterpret_cast<IPv4Address*>(m->options+p) = gkIP;
	p += 4;

	m->options[p++] = DHCPMessage::SUBNET_MASK;
	m->options[p++] = 4;
	*reinterpret_cast<IPv4Address*>(m->options+p) = netmask;
	p += 4;

	m->options[p++] = DHCPMessage::ROUTER;
	m->options[p++] = 4;
	*reinterpret_cast<IPv4Address*>(m->options+p) = gwIP;
	p += 4;

	m->options[p++] = DHCPMessage::DNS;
	m->options[p++] = 4;
	*reinterpret_cast<IPv4Address*>(m->options+p) = dnsIP;
	p += 4;

	m->options[p++] = DHCPMessage::DOMAIN_NAME;
	m->options[p++] = domain.size();
	strcpy (reinterpret_cast<char*>(m->options+p), domain.c_str());
	p += m->options[p-1];

	if ( wpad_exists ) {
		m->options[p++] = DHCPMessage::WPAD_URL;
		m->options[p++] = wpad_file.length()+1;
		strcpy ((char*)(m->options+p), wpad_file.c_str());
		*reinterpret_cast<char*>(m->options+p+wpad_file.length()) = 0x0a;
		p += m->options[p-1];
	}

	m->options[p++] = DHCPMessage::END_OPTIONS;

	int l = sizeof(DHCPMessage) + p;
	memset (reinterpret_cast<char*>(m)+l, 0, DHCPMessage::PACKET_LENGTH - l);

	ipk->length = htons (DHCPMessage::PACKET_LENGTH + sizeof(IPPacket) + sizeof(UDPDatagram));
	ipk->calculateHeaderChecksum ();

	udpk->length = htons (DHCPMessage::PACKET_LENGTH + sizeof(UDPDatagram));
	udpk->calculateChecksum (ipk->source, ipk->destination, DHCPMessage::PACKET_LENGTH + sizeof(UDPDatagram));
}




void DhcpPlugin::setNack(EthernetFrame* ethf, IPv4Address ip, EthernetFrame* e)
{
	memcpy (e, ethf, sizeof(EthernetFrame)+sizeof(IPPacket)+sizeof(UDPDatagram)+sizeof(DHCPMessage));

	e->setReplyHeader (ethf);
	IPPacket *ipk = &(e->IPPayload());
	ipk->setReplyHeader (&(ethf->IPPayload()));
	ipk->source = gkIP;
	ipk->destination = 0xFFFFFFFFUL;
	UDPDatagram *udpk = &(ipk->UDPPayload());
	udpk->setReplyHeader (&(ethf->IPPayload().UDPPayload()));
	DHCPMessage *m = reinterpret_cast<DHCPMessage*>(udpk->payload);

	m->op = DHCPMessage::RESPONSE;
	m->flags = 0x0080;
	m->yiaddr = ip;
	m->siaddr = gkIP;
	int p = 0;

	m->options[p++] = DHCPMessage::MESSAGE_TYPE;
	m->options[p++] = 1;
	m->options[p++] = DHCPMessage::NACK;

	m->options[p++] = DHCPMessage::SERVER_ID;
	m->options[p++] = 4;
	*reinterpret_cast<IPv4Address*>(m->options+p) = gkIP;
	p += 4;

	m->options[p++] = DHCPMessage::LEASE_TIME;
	m->options[p++] = 4;
	*reinterpret_cast<uint32_t*>(m->options+p) = htonl(shortLease);
	p += 4;
	
	m->options[p++] = DHCPMessage::RENEWAL_TIME;
	m->options[p++] = 4;
	*reinterpret_cast<uint32_t*>(m->options+p) = htonl(shortLease/2);
	p += 4;
	
	m->options[p++] = DHCPMessage::SUBNET_MASK;
	m->options[p++] = 4;
	*reinterpret_cast<IPv4Address*>(m->options+p) = netmask;
	p += 4;

	m->options[p++] = DHCPMessage::ROUTER;
	m->options[p++] = 4;
	*reinterpret_cast<IPv4Address*>(m->options+p) = gwIP;
	p += 4;

	m->options[p++] = DHCPMessage::DNS;
	m->options[p++] = 4;
	*reinterpret_cast<IPv4Address*>(m->options+p) = dnsIP;
	p += 4;

	m->options[p++] = DHCPMessage::DOMAIN_NAME;
	m->options[p++] = domain.size();
	strcpy (reinterpret_cast<char*>(m->options+p), domain.c_str());
	p += m->options[p-1];

	if ( wpad_exists ) {
		m->options[p++] = DHCPMessage::WPAD_URL;
		m->options[p++] = wpad_file.length()+1;
		strcpy ((char*)(m->options+p), wpad_file.c_str());
		*reinterpret_cast<char*>(m->options+p+wpad_file.length()) = 0x0a;
		p += m->options[p-1];
	}

	m->options[p++] = DHCPMessage::END_OPTIONS;

	int l = sizeof(DHCPMessage) + p;
	memset (reinterpret_cast<char*>(m)+l, 0, DHCPMessage::PACKET_LENGTH - l);

	ipk->length = htons (DHCPMessage::PACKET_LENGTH + sizeof(IPPacket) + sizeof(UDPDatagram));
	ipk->calculateHeaderChecksum ();

	udpk->length = htons (DHCPMessage::PACKET_LENGTH + sizeof(UDPDatagram));
	udpk->calculateChecksum (ipk->source, ipk->destination, DHCPMessage::PACKET_LENGTH + sizeof(UDPDatagram));
}

}
