/*##############################################################################
   
   Developed by NetMob Group, University of Trento, Italy

   University of Trento
   DIT - Department of Informatics and Telecommunications 
   Computer Networks and Mobility Research Program
   http://netmob.unitn.it/ 
   
   This file is part of the WilmaGate system, created as part
   of the WILMA Project, sponsored by the Autonomous
   Province of Trento and participated by ITC-irst, the University
   of Trento and Alpikom Spa. More info at

     http://www.wilmaproject.org/		

   To download the project:
      - in tarball format
        http://netmob.unitn.it/wilmagate.html
      - connect to CVS repository
        :pserver:cvs@rtm.unitn.it:/home/cvsroot
   In both previous cases, the download is password protected. To obtain
   the password you must send a mail to 

      wilmagate@dit.unitn.it

################################################################################

   The WilmaGate Licence of Use


   Copyright (c) 2003-2005
                - Universita` di Trento,
                  Dipartimento di Informatica e Telecomunicazioni (DIT)
                  Computer Networks and Mobility Research Program


   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (hereinafter
   the "WilmaGate") directly from the copyright owner, to use the WilmaGate
   without restriction of purpose or time, including without limitation the
   rights to use, copy, modify, and merge the WilmaGate, subject to the
   following conditions:

   The above copyright notice and this permission notice shall be included
   in all copies or substantial portions of the WilmaGate. 

   The WilmaGate shall not be sub-licenced either commercially or free of
   charge, as is or in part or as a part of any other package. 

   The licensee agrees to properly credit WilmaGate and its copyright 
   owners in any technical, scientific or divulgative publication, 
   whatever publishing means are used, that is based entirely or in part 
   on WilmaGate use, as well as in any booklet, web-site or brochure that 
   advertise services based on WilmaGate use. 

   If the WilmaGate is modified, the copyright owner shall be notified.

   THE WilmaGate IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

##############################################################################*/

#include "../Common/utilities.h"
#include "dispatcher.h"

namespace Gatekeeper {

Dispatcher::Dispatcher(
RelayPlugin* pointerToRelay, //For DHCP relay plugin
IPv4Address listenIP, uint16_t listenPort,
IPv4Address destIP, uint16_t destPort): pToRelay(pointerToRelay) //For DHCP relay plugin
,l(0)
{

    if(pToRelay)     //For DHCP relay plugin
	pToRelay->registerDHCPService (this);
	
	// Inizialize the vector of EthernetFrame
	for( int i = 0; i < MAXFRAMES ; i++ )
		frames[i]=new (1<<16) EthernetFrame;
		// Each frame has a dimension equal to 2^16=65536 == 1 shifted of 16 positions

	//Inizialize the socket
	skudp.setReceiveBuffer (1UL<<20);
	skudp.bind(listenIP, listenPort);
	skudp.registerListener(*this, 1);
	skudp.setDestination(destIP, destPort);

	//
	//MB: moved to a separate method.
	//this->skudp.startReceiving();

}

Dispatcher::Dispatcher(
     RelayPlugin* pointerToRelay, //For DHCP relay plugin
	CapwapPluginB* pointercpb, //For CAPWAP Plugin B     
	 IPv4Address listenIP, uint16_t listenPort, 
	 IPv4Address destIP, uint16_t destPort): 
	 pToRelay(pointerToRelay), //For DHCP relay plugin
	 cpb(pointercpb),
	 l(0)
{

    if(pToRelay)     //For DHCP relay plugin
	pToRelay->registerDHCPService (this);
	
    if(cpb)     //For DHCP relay plugin
	cpb->registerCAPWAPCONFService (this);
	
	// Inizialize the vector of EthernetFrame
	for( int i = 0; i < MAXFRAMES ; i++ )
		frames[i]=new (1<<16) EthernetFrame;
		// Each frame has a dimension equal to 2^16=65536 == 1 shifted of 16 positions

	//Inizialize the socket
	skudp.setReceiveBuffer (1UL<<20);
	skudp.bind(listenIP, listenPort);
	skudp.registerListener(*this, 1);
	skudp.setDestination(destIP, destPort);

	//
	//MB: moved to a separate method.
	//this->skudp.startReceiving();

}

Dispatcher::~Dispatcher()
{
	for ( int i = 0; i < MAXFRAMES; i++ )
		delete frames[i];
	delete frames;
}


void Dispatcher::insertPlugin(Plugin* p)
{
	element[l++]=p;		// The private variable l contains the number of objects in the array
						// of Plugin elements.
	p->registerDispatcher (this);
}

void Dispatcher::sendPacket(EthernetFrame *e, int intf)
{
	for ( int i = 0; i < MAXFRAMES && sense[i] != END_OF_PACKETS; i++ ) {
		EthernetFrame *ef = (sense[i] == SEND_SAME) ? e : frames[i];
		setSense_GK (ef, (sense[i] == SEND_BACK) ? SEND_BACK : FORWARD);
		setInterface_GK (ef, intf);
		skudp.send (ef, ntohs(ef->IPPayload().length) + sizeof(EthernetFrame));
	}
}

SocketListener::SuggestedAction Dispatcher::receivedPacket (uint8_t *message, int size, int socketID,
								IPv4Address senderIP, uint16_t senderPort)
{
	for ( int i = 0; i < l; i++)
		if ( element[i]->processPacket(reinterpret_cast< EthernetFrame*>(message), frames, sense) ) {
			sendPacket (reinterpret_cast< EthernetFrame*>(message),
						getInterface_GK(reinterpret_cast< EthernetFrame*>(message)));
			break;
		}

	return CONTINUE;
}


SocketListener::SuggestedAction Dispatcher::failed (int socketID)
{
	return CONTINUE; //(socketID == 1) ? STOP : DELETE;
}

SocketListener::SuggestedAction Dispatcher::sendResponseFrame(EthernetFrame* ethf, int size)
{

                memcpy (frames[0], ethf, size);
                frames[1]->protocol = 0;
                sense[0] = FORWARD;
                sense[1] = END_OF_PACKETS;
                sendPacket (ethf, getInterface_GK(ethf) );
                return CONTINUE;

}

}
