/*##############################################################################
   
   Developed by NetMob Group, University of Trento, Italy

   University of Trento
   DIT - Department of Informatics and Telecommunications 
   Computer Networks and Mobility Research Program
   http://netmob.unitn.it/ 
   
   This file is part of the WilmaGate system, created as part
   of the WILMA Project, sponsored by the Autonomous
   Province of Trento and participated by ITC-irst, the University
   of Trento and Alpikom Spa. More info at

     http://www.wilmaproject.org/		

   To download the project:
      - in tarball format
        http://netmob.unitn.it/wilmagate.html
      - connect to CVS repository
        :pserver:cvs@rtm.unitn.it:/home/cvsroot
   In both previous cases, the download is password protected. To obtain
   the password you must send a mail to 

      wilmagate@dit.unitn.it

################################################################################

   The WilmaGate Licence of Use


   Copyright (c) 2003-2005
                - Universita` di Trento,
                  Dipartimento di Informatica e Telecomunicazioni (DIT)
                  Computer Networks and Mobility Research Program


   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (hereinafter
   the "WilmaGate") directly from the copyright owner, to use the WilmaGate
   without restriction of purpose or time, including without limitation the
   rights to use, copy, modify, and merge the WilmaGate, subject to the
   following conditions:

   The above copyright notice and this permission notice shall be included
   in all copies or substantial portions of the WilmaGate. 

   The WilmaGate shall not be sub-licenced either commercially or free of
   charge, as is or in part or as a part of any other package. 

   The licensee agrees to properly credit WilmaGate and its copyright 
   owners in any technical, scientific or divulgative publication, 
   whatever publishing means are used, that is based entirely or in part 
   on WilmaGate use, as well as in any booklet, web-site or brochure that 
   advertise services based on WilmaGate use. 

   If the WilmaGate is modified, the copyright owner shall be notified.

   THE WilmaGate IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

##############################################################################*/

#include "../Common/utilities.h"
#include <iostream>
extern "C" {
	#include <unistd.h>
	#include <syslog.h>
}

#include "../Common/addresses.h"
#include "../Common/packets.h"
#include "../Common/socket.h"
#include "../Common/exceptions.h"
#include "../Common/parameters.h"

#include "plugin_statustable_interface.h"
#include "auth_statustable_interface.h"
#include "status_table.h"
#include "statustable_interface.h"
#include "token.h"
#include "dispatcher.h"
#include "http_plugin.h"
#include "dhcp_plugin.h"
#include "plugin.h"
#include "provider_list.h"
#include "providerlist_interface.h"
#include "report.h"
#include "proxy_plugin.h"
//#include "relay_plugin.h"
#include "authentication.h"

//For DHCP relay plugin
//Including relay class
#include "relay_plugin.h"
//SIP stuff
#include "sip_plugin.h"
#include "sip_providers.h"
//For CAPWAP plugin
#include "wtptable_interface.h"
#include "wtp_table.h"
#include "capwap_plugin_A.h"
#include "capwap_plugin_B.h"
#include "capwap_wtpconfiguration.h"

using namespace std;
using namespace Gatekeeper;
using namespace Common;

int main (int argc, char *argv[])
{

    RelayPlugin * rel = NULL;
    DhcpPlugin  * dhc = NULL;
    SipPlugin   * sip = NULL;
    CapwapPluginA *capwapA=NULL;
    CapwapPluginB *capwapB=NULL;
    WTPConfigurationListener *wtpcl=NULL;
    WTPTable *wtpt=NULL;

	try {

		IPv4Address authIP;
		int authPort;
		IPv4Address authRelayIP;
		IPv4Address reportIP;
		int reportPort;
		IPv4Address dispatcherIP;
		int dispatcherPort;
		IPv4Address gatewayTunnelIP;
		int gatewayTunnelPort;
		IPv4Address gatewayCommandIP;
		int gatewayCommandPort;
		IPv4Address ipBASE;
		IPv4Address firstIP;
		IPv4Address lastIP;
		IPv4Address dnsIP;
		IPv4Address rtrIP;
		IPv4Address netmask;
		string domain;
		IPv4Address fakeDHCPServer;
		int shortLease;
		int longLease;
		string redirectURL;
		string redirectURL_NOPROXY;
		string wpadFile;
		int renewalTime;
		int logformat;
		int runmode;
		string statusFile;
		string loggerFile;
		string PIDFile;

		/////////////////////////////////////////////////
		//For DHCP relay plugin
        //relay Parameters
		////////////////////////////////////////////////
		int RelayActivated;
		IPv4Address DHCPServerIP;
		IPv4Address OuterInterfaceIP;
		IPv4Address NatPublicIP;
		IPv4Address InnerInterfaceIP;
		int 	DHCPServerPort;
		int 	DHCPClientPort;
		
		/////////////////////////////////////////////////
		//For CAPWAP plugin
		int enablecapwap;
		string pmtcapwap;
		UDPSocket conf_sk;
		IPv4Address conf_ip;
		int conf_udp;
		/////////////////////////////////////////////////
		
		string InnerInterfaceName;
		//EthernetMACAddress InnerInterfaceMAC;


		Parameters pmt (argc, argv);

		pmt
			>> authIP
			>> authPort
			>> authRelayIP
			>> reportIP
			>> reportPort
			>> dispatcherIP
			>> dispatcherPort
			>> gatewayTunnelIP
			>> gatewayTunnelPort
			>> gatewayCommandIP
			>> gatewayCommandPort
			>> ipBASE
			>> firstIP
			>> lastIP
			>> dnsIP;
		/*
		if ( dnsIP == 0 )
			pmt
				
		else
		*/
			pmt
				>> rtrIP
				>> netmask
				>> domain
				>> fakeDHCPServer;
		pmt
			>> shortLease
			>> longLease
			>> redirectURL
			>> redirectURL_NOPROXY
			>> wpadFile
			>> renewalTime			
			>> logformat
			>> runmode
			>> statusFile
			>> loggerFile
			>> PIDFile;
		
			//Common::epoch=logformat;
						
		if ( runmode ) {
			detachFromConsole (loggerFile.c_str());
			cerr << "\n=\n===========================================\n=\n";
		}

		if ( PIDFile.length() ) {
			ofstream pidf (PIDFile.c_str());
			pidf << getpid() << endl;
			pidf.close ();
		}
		
        //For DHCP relay plugin
		//Capturing relay parameters
			 pmt 	>> RelayActivated
			 	>> DHCPServerIP
				>> OuterInterfaceIP
				>> NatPublicIP
				>> InnerInterfaceIP
				>> DHCPServerPort
				>> DHCPClientPort
			 	//>> InnerInterfaceMAC
	         	 	>> InnerInterfaceName
			 ;
		logger() << "Inner interface name (eth1): " << InnerInterfaceName << "\n";
		openlog ("GK", 0, LOG_USER);
		
		logger() << "Gatekeeper program started.\n";

		logger() << "Creating provider list\n";
		ProviderList pl;
		for ( ; ; ) {
			string name, url, prefix, domain;
			IPv4Address ip;
			pmt >> name;
			if ( name.length() == 0 )
				break;
			pmt >> url >> prefix >> domain >> ip;
			pl.insertProvider (name.c_str(), (url.length() == 0) ? NULL : url.c_str(), prefix.c_str(), domain.c_str(), ip);
		}
		
		SipProviders sp;
		int enableSip;
		int sip_debugging;
		pmt >> enableSip;
		pmt >> sip_debugging;
		
		if(enableSip){
			if (!sip_debugging)
				logger() << "SIP plugin enabled - NORMAL LOG mode - \n";
			else
				logger() << "SIP plugin enabled - VERBOSE LOG mode -\n";
			logger() << "Creating SIP provider list\n";
		}else{
			logger() << "SIP plugin disabled\n";
		}
		//logger() << "SIP servers...\n";
		for ( ; ; ) {
			string name;
			IPv4Address ip;
			int auth_mode;
			pmt >> name;
			if ( name.length() == 0 )
				break;
			pmt >> ip >>  auth_mode;
			if(enableSip){
				sp.insertProvider(name.c_str(), ip, auth_mode);
			}
		}
		
		logger() << "Reading CAPWAP parameters...\n";
		pmt >> enablecapwap >> pmtcapwap >> conf_ip >> conf_udp;
		
		
		logger() << "Creating status table\n";
		StatusTable st(ipBASE, firstIP, lastIP, statusFile.c_str(),shortLease);
		for ( ; ; ) {
			string name, email;
			EthernetMACAddress mac;
			IPv4Address ip;
			pmt >> name;
			if ( name.length() == 0 )
				break;
			pmt >> email >> mac >> ip;
			if ( ! st.addAuthorization (name.c_str(), email.c_str(), ip, mac) ) {
				logger() << "Error in adding persistent authorization item for " << name << '\n';
				exit (1);
			}
		}
		

		if(RelayActivated == 1) 
		logger() << "Relay selected.\n";	 
		else
		if(RelayActivated == 0) 
		logger() << "Fake DHCP selected.\n";	
		
		//RelayPlugin * rel= NULL;
		//DhcpPlugin * dhc = NULL;
		
		logger() << "Creating plugins\n";
		/*	Mauro code
        //	Plugin *dhcp = NULL;

		//if ( domain == "" )
			dhcp = new DhcpPlugin (st, fakeDHCPServer, rtrIP, dnsIP, netmask, domain.c_str(), wpadFile.c_str(), shortLease, longLease);
		//else
		//	dhcp = new RelayPlugin (st, fakeDHCPServer, 67, 67, InnerInterfaceIP, InnerInterfaceName,
		//			InnerInterfaceMAC, OuterInterfaceIP, NatPublicIP, netmask, shortLease, longLease, dnsIP, wpadFile.c_str());
		*/
			if(RelayActivated==0) {	
			
			dhc= new DhcpPlugin (st, fakeDHCPServer, rtrIP, dnsIP, netmask, domain.c_str(), wpadFile.c_str(), shortLease, longLease);
			//logger() <<&dhcp<< "  " <<dhc<<"dhc\n";
		
		}
		
		else {	
			
			rel= new RelayPlugin (st, DHCPServerIP, DHCPServerPort,DHCPClientPort,  InnerInterfaceIP , 				InnerInterfaceName,
			//InnerInterfaceMAC, 
			OuterInterfaceIP,NatPublicIP, 
			netmask,shortLease,longLease,dnsIP, wpadFile.c_str());		
			//logger() <<&relay<< "  " <<rel<<"rel\n";
		}
	

    
		HttpPlugin http (st, pl, reportIP, reportPort, authRelayIP, authPort, redirectURL_NOPROXY.c_str());
		ProxyPlugin proxy (st, pl, reportIP, reportPort, authRelayIP, authPort, redirectURL.c_str());
		
		if (enableSip){
			sip= new SipPlugin(st, sp, sip_debugging);
		}
		
		if(enablecapwap)
		{			
			logger() << "Creating WTP table\n";
			wtpt = new WTPTable (pmtcapwap);
			wtpcl = new WTPConfigurationListener (*wtpt);
			conf_sk.bind (conf_ip, conf_udp);
			conf_sk.registerListener (*wtpcl, 0);
			conf_sk.startReceiving ();
		
			logger() << "Creating Capwap Plugin A and Capwap Plugin B\n";
			capwapA = new CapwapPluginA  (st, *wtpt, fakeDHCPServer, rtrIP, domain.c_str());
			capwapB = new CapwapPluginB  (st, *wtpt, fakeDHCPServer, rtrIP, domain.c_str());
			wtpt->Register_capwap_plugin_B(capwapB);
			
		}else{
			logger() << "CAPWAP plugin disabled\n";
		}
		
		logger() << "All plugins created: closing status table subscriptions\n";
		st.createInfo ();

		logger() << "Creating dispatcher\n";
		Dispatcher d (rel, //For DHCP relay plugin
        dispatcherIP, dispatcherPort, gatewayTunnelIP, gatewayTunnelPort);
        //Mauro code
		//d.insertPlugin(dhcp); 
		//For DHCP relay plugin
        //conditional inclusion
        	if(dhc!=NULL)
			d.insertPlugin(dhc);
		else if(rel!=NULL)
			d.insertPlugin(rel);
			
        	d.insertPlugin(&http);
		d.insertPlugin(&proxy);
				
		if(sip!=NULL){
			d.insertPlugin(sip);
		}
		
 
 		logger() << "Starting status table maintenance schedule\n";
 		st.start ();
 	
 		if(rel!=NULL) 
 		{
		 	rel->start();
 			logger() << "Starting relay module\n";
 		}
	
 		logger() << "Starting dispatcher interface\n";
 		d.start ();

		if(enablecapwap)
		{
			d.insertPlugin(capwapA);
			d.insertPlugin(capwapB);
			capwapB->registerCAPWAPCONFService(&d);
			logger() << "Starting wtp table maintenance schedule\n";
			wtpt->start ();
			
		}
		
    	//inserting interceptor
    	//d.insertPlugin(&interceptor);
	
		logger() << "Creating report interface\n";
		Report r(st, pl, reportIP, reportPort, gatewayCommandIP, gatewayCommandPort);
		r.start ();
	
		logger() << "Creating authentication module\n";
		Authentication auth (st, pl, authIP, authPort, renewalTime);
		auth.start ();
	
		logger() << "Hooking execution till dispatcher completion" << endl;
		d.wait ();

	}
	catch ( WilmaGateException &e ) {
		logger() << e << '\n';
	}
	catch ( ... ) {
		logger() << "Unexpected exception!\n";
	}
	
	//For DHCP relay plugin


//if(rel !=NULL)
//delete rel;
//else
//if(dhc !=NULL)
//delete dhc;
	
	closelog ();
	return 0;
}
