/*##############################################################################
   
   Developed by NetMob Group, University of Trento, Italy

   University of Trento
   DIT - Department of Informatics and Telecommunications 
   Computer Networks and Mobility Research Program
   http://netmob.unitn.it/ 
   
   This file is part of the WilmaGate system, created as part
   of the WILMA Project, sponsored by the Autonomous
   Province of Trento and participated by ITC-irst, the University
   of Trento and Alpikom Spa. More info at

     http://www.wilmaproject.org/		

   To download the project:
      - in tarball format
        http://netmob.unitn.it/wilmagate.html
      - connect to CVS repository
        :pserver:cvs@rtm.unitn.it:/home/cvsroot
   In both previous cases, the download is password protected. To obtain
   the password you must send a mail to 

      wilmagate@dit.unitn.it

################################################################################

   The WilmaGate Licence of Use


   Copyright (c) 2003-2005
                - Universita` di Trento,
                  Dipartimento di Informatica e Telecomunicazioni (DIT)
                  Computer Networks and Mobility Research Program


   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (hereinafter
   the "WilmaGate") directly from the copyright owner, to use the WilmaGate
   without restriction of purpose or time, including without limitation the
   rights to use, copy, modify, and merge the WilmaGate, subject to the
   following conditions:

   The above copyright notice and this permission notice shall be included
   in all copies or substantial portions of the WilmaGate. 

   The WilmaGate shall not be sub-licenced either commercially or free of
   charge, as is or in part or as a part of any other package. 

   The licensee agrees to properly credit WilmaGate and its copyright 
   owners in any technical, scientific or divulgative publication, 
   whatever publishing means are used, that is based entirely or in part 
   on WilmaGate use, as well as in any booklet, web-site or brochure that 
   advertise services based on WilmaGate use. 

   If the WilmaGate is modified, the copyright owner shall be notified.

   THE WilmaGate IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

##############################################################################*/

#include "../Common/utilities.h"
#include "interceptor_plugin.h"

using namespace std;
using namespace Common;

namespace Gatekeeper {

///Interceptor Plugin
///This plugin don't create any response packet.
///It simply checks mac address and IP of any packet that is not part DHCP protocol
///If the client which sends the packet is not in status table this plugin adds client
///in status table with the status of TEMPORARY LEASE if it is not a pre-authenticated one.
///If the client is a pre-authenticated one it is added in status table with the status of AUTHORIZED
///so this one perpetually persists in the table form this moment 

interceptorPlugin::interceptorPlugin(PluginStatusTableInterface &p,  IPv4Address innerInterfaceIP,
IPv4Address  netm, int sl):
psti(p) ,
inIP(innerInterfaceIP),
netmask (netm),
shortLease(sl) {
}

interceptorPlugin::~interceptorPlugin (void) {
}

bool interceptorPlugin::processPacket (EthernetFrame *ethf, EthernetFrame* v[], PacketSense sense[])
{
   IPv4Address ip;

   IPv4Address subnet, testedSubnet;

   EthernetMACAddress  mac;

   int protocol;
	
   mac=   ethf->sender;
		
   ip =   ethf->IPPayload().source;

   subnet= inIP & netmask;
   
   testedSubnet= ip & netmask;

   //Getting Transport type protocol 
   protocol= ethf->IPPayload().protocol;

   StatusTableInterface::statusType status = psti.getStatus (ip);

//   if(subnet == testedSubnet)
//  	 logger () << "Packet Intercepted from: "<< ip << " on "<< mac << "\n"; 
   
   
//1.1
   if ( ((protocol == IPPacket::TCP_PROTOCOL || protocol == IPPacket::UDP_PROTOCOL)  ) && status < 	StatusTableInterface::FIRST_SEEN && subnet == testedSubnet )  {

	if ( 
		!( (protocol == IPPacket::UDP_PROTOCOL) 
		&& ( ethf->IPPayload().UDPPayload().sourcePort == htons(0x0044))
		) 
	){
	      
	      	//psti.associateMACtoIP (mac ,ip);

              	//manual albino here
              	//psti.setIPMode(ip,false) ;
              
              	if(psti.isPreAuthenticated(mac,ip))   {     //albino here

                 	//manual albino here
              		psti.setIPMode(ip,false) ;
              
			psti.setStatus(ip ,  StatusTableInterface::AUTHORIZED); //albino here
			logger() << "Adding manually configured IP: " << ip << " on: " << ethf->sender  <<" status is AUTHORIZED \n";
              	}            
              	else {
              		//logger() << "Giving a time window to IP: " << ip << " requested by: " << ethf->sender << endl;

			if(status == StatusTableInterface::FREE)
			{
				psti.associateMACtoIP (mac ,ip);

                		psti.setExpirationIP(ip,shortLease);
                
				//manual albino here
              			psti.setIPMode(ip,false) ;
              
				psti.setStatus(ip ,  StatusTableInterface::TEMPORARY_LEASE,true);
				logger() << "Adding manually configured IP: " << ip << " on: " << ethf->sender  <<" status is TEMPORARY_LEASE \n";
              		} else {
				logger() << "MAC " << mac <<" is trying to use the FORBIDDEN IP "<<ip<< "\n";
			}
		}
    	}
    	
     } 
	   return false;

}



}
