///
/// \file plugin_statustable_interface.h
///	\author Danilo Severina
///
/// The file contains the definitions of functions that are used by the
///	Plugin sub-classes to access the Status Table to read and/or write 
///	data. Not all of functions that read and write informations in Status
/// Table must be used by plugins, so this class has the function of interface:
///	only the methods that are described here can be used byb plugins.
/// There are functions that can be used by the plugins but they are not 
///	defined in this file: they are defined virtual in StatusTableInterface and
///	the interface beetwen tha plugins and the Status Table derive from the
/// previous class.
///
///	$Id: plugin_statustable_interface.h,v 1.4 2006/03/30 13:35:23 cvs Exp $
///

/*##############################################################################
   
   Developed by NetMob Group, University of Trento, Italy

   University of Trento
   DIT - Department of Informatics and Telecommunications 
   Computer Networks and Mobility Research Program
   http://netmob.unitn.it/ 
   
   This file is part of the WilmaGate system, created as part
   of the WILMA Project, sponsored by the Autonomous
   Province of Trento and participated by ITC-irst, the University
   of Trento and Alpikom Spa. More info at

     http://www.wilmaproject.org/		

   To download the project:
      - in tarball format
        http://netmob.unitn.it/wilmagate.html
      - connect to CVS repository
        :pserver:cvs@rtm.unitn.it:/home/cvsroot
   In both previous cases, the download is password protected. To obtain
   the password you must send a mail to 

      wilmagate@dit.unitn.it

################################################################################

   The WilmaGate Licence of Use


   Copyright (c) 2003-2005
                - Universita` di Trento,
                  Dipartimento di Informatica e Telecomunicazioni (DIT)
                  Computer Networks and Mobility Research Program


   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (hereinafter
   the "WilmaGate") directly from the copyright owner, to use the WilmaGate
   without restriction of purpose or time, including without limitation the
   rights to use, copy, modify, and merge the WilmaGate, subject to the
   following conditions:

   The above copyright notice and this permission notice shall be included
   in all copies or substantial portions of the WilmaGate. 

   The WilmaGate shall not be sub-licenced either commercially or free of
   charge, as is or in part or as a part of any other package. 

   The licensee agrees to properly credit WilmaGate and its copyright 
   owners in any technical, scientific or divulgative publication, 
   whatever publishing means are used, that is based entirely or in part 
   on WilmaGate use, as well as in any booklet, web-site or brochure that 
   advertise services based on WilmaGate use. 

   If the WilmaGate is modified, the copyright owner shall be notified.

   THE WilmaGate IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

##############################################################################*/

#ifndef _PLUGIN_STATUSTABLE_INTERFACE_
#define	_PLUGIN_STATUSTABLE_INTERFACE_

#include <iostream>
#include "statustable_interface.h"
#include "statustable_listener.h"
//#include "token.h"
//#include "addresses.h"
//#include "mac_address.h"

using namespace std;

namespace Gatekeeper {

class PluginInfo;

/// 
/// The class contains the definitions of functions that are used by the
///	%Plugin classes to access the Status Table to read and/or write 
///	data. Not all of functions that read and write informations in Status
/// Table must be used by %Plugins, so this class has function of interface:
///	only the methods that are described here can be used by %Plugins.
///
///	Beyond the functions that are defined here, the %Plugins can use also the function
///	that are defined in the StatusTableInterface: the %PluginStatusTableInterface (as
///	all the other interface between an object and the %StatusTabel) derives from this
///	class. The inherit is virtual.
///
/// Detailed descriptionof the functions follows and can be also found in the StatusTable class.
///
class PluginStatusTableInterface : virtual public StatusTableInterface{

private:

public:

	//
	// The function sets the URL that the user requires during the first HTTP request
	//	when it is not authorizated to access the web yet. The required URL is 
	//	stored in the Status Table and it will be used after the authentication
	//	procedure: when the user is authorized to access the web, the previous required 
	// URL are shown to him by the system.
	//
	//virtual Token* setURL(EthernetMACAddress macUser, IPv4Address ip, const char *urlReqired) = 0;

	//
	//	The function is used in the DHCP offer transistion: the DHCP plugin requires a new
	//	IP address for the user identified by the mac address. The status table looks for a free
	//	IP address and offers it to the user (in this case tha status of IP address change from
	//	FREE to FIRST_SEEN) (see StatusTableInterface).
	//	The function returns the IP offered to the user.
	//
	//virtual IPv4Address requireIP (EthernetMACAddress macUser) = 0;
 
	//
	//	The function is used in the DHCP acknowledgement transistion: the DHCP 
	//	confirms that the IP address offered in the previous section (DHCP offer) by the 
	//	system is accepted by the user.
	//	The status table changes the status of IP address from
	//	FIRST_SEEN to TEMPORAY_LEASE (see StatusTableInterface).
	//	The IP address has a short lease time, but it is known also by the Status Table.
	//	\return: \e true if association MAC and IP address and the status of the IP address
	//	are right; \e false otherwise. FOrse e' meglio return TOken.
	//
	//virtual Token* acceptIP (EthernetMACAddress macUser, IPv4Address ip) = 0;

	///
	/// \brief Check an IP-MAC association
	///
	virtual bool checkIP (EthernetMACAddress &mac, IPv4Address ip) = 0;

	///
	///	\brief Notify the addition of a %Plugin in the system.
	///
	///	The function is invoked by the Plugin when it is initializes. In this way the Status Table
	///	can know how many Plugins are in the system and can create an array (userElement::*info)
	///	with right dimensions.
	///	The %StatusTable doesn't know the type of the new %Plugin.
	///	\return The index of the colum of the array reserved for the added %Plugin.
	///
	virtual int registerPlugin() = 0;

	///
	///	\brief Add a pointer to Plugin.
	///
	///	The function adds the pointer to a %PluginInfo object where the %Plugin storaged
	///	information about a user.
	///	\param IDPlugin The ID of the %Plugin.
	///	\param ip The IP address of the user.
	///	\param pi Pointer to the pluginInfo object to be added.
	///	\return \e true if the insertion was successful, \e false otherwise.
	///
	virtual bool addPluginInfo(int IDPlugin, IPv4Address ip, PluginInfo* pi) = 0;

	///
	///	\brief Get the pointer to %PluginInfo.
	///
	///	The functino gets the pointer to the %PluginInfo that is identified by 
	///	the ID of the %Plugin and the user's IP address. If the required PluginInfo is not jet
	///	created, it returns a NULL pointer.
	///	\param IDPlugin The ID of the %Plugin.
	///	\param ip The IP address of the user.
	///	\return The pointer to the PluginInfo object if present, NULL otherwise.
	///
	virtual PluginInfo* getPluginInfo(int IDPlugin, IPv4Address ip) = 0;

	/// \brief Register the DHCP service for calback on status variations
	///
	///     \param p Object being registered
	///
	   //For DHCP relay plugin
        virtual void registerDHCPService (StatusTableListener *p) = 0;

        //Used by DHCP relay plugin to write MAC address of a client in the status table.
        ///     \param macUser The MAC address of the user.
        ///     \param ipUser The IP address of the user.
        virtual void associateMACtoIP(EthernetMACAddress macUser, IPv4Address ipUser) =0;
        //0 manual 1 DHCP
        virtual void setIPMode(IPv4Address ipUser, bool ipMode) =0;
        //Time of next presence control
       // virtual void setPresenceCheck(IPv4Address ipUser,time_t nextCheck) =0;
        
	// inherited
	virtual EthernetMACAddress *getMAC (IPv4Address ip) = 0;

};

}

#endif

