/*##############################################################################
   
   Developed by NetMob Group, University of Trento, Italy

   University of Trento
   DIT - Department of Informatics and Telecommunications 
   Computer Networks and Mobility Research Program
   http://netmob.unitn.it/ 
   
   This file is part of the WilmaGate system, created as part
   of the WILMA Project, sponsored by the Autonomous
   Province of Trento and participated by ITC-irst, the University
   of Trento and Alpikom Spa. More info at

     http://www.wilmaproject.org/		

   To download the project:
      - in tarball format
        http://netmob.unitn.it/wilmagate.html
      - connect to CVS repository
        :pserver:cvs@rtm.unitn.it:/home/cvsroot
   In both previous cases, the download is password protected. To obtain
   the password you must send a mail to 

      wilmagate@dit.unitn.it

################################################################################

   The WilmaGate Licence of Use


   Copyright (c) 2003-2005
                - Universita` di Trento,
                  Dipartimento di Informatica e Telecomunicazioni (DIT)
                  Computer Networks and Mobility Research Program


   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (hereinafter
   the "WilmaGate") directly from the copyright owner, to use the WilmaGate
   without restriction of purpose or time, including without limitation the
   rights to use, copy, modify, and merge the WilmaGate, subject to the
   following conditions:

   The above copyright notice and this permission notice shall be included
   in all copies or substantial portions of the WilmaGate. 

   The WilmaGate shall not be sub-licenced either commercially or free of
   charge, as is or in part or as a part of any other package. 

   The licensee agrees to properly credit WilmaGate and its copyright 
   owners in any technical, scientific or divulgative publication, 
   whatever publishing means are used, that is based entirely or in part 
   on WilmaGate use, as well as in any booklet, web-site or brochure that 
   advertise services based on WilmaGate use. 

   If the WilmaGate is modified, the copyright owner shall be notified.

   THE WilmaGate IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

##############################################################################*/

#include "../Common/utilities.h"
#include "provider_list.h"

namespace Gatekeeper {

ProviderList::ProviderList(void) : l(0) 
{}
// The number of authenticatin provider is zero.

bool ProviderList::insertProvider(const char * nameP, const char* urlP, const char* prefixP, const char *domainP, IPv4Address ipP)
{
//	cout<<"insert provider"<<endl;
/*
	if(getIPFromName(nameP)==ipP && strcmp(getURLFromDomain(domainP), urlP)==0)
		return false;
*/
	// the provider was inserted, avoid duplication of a single provider
	// The provider is equal to another one if it has the same name, ip and url
	provider[l].name=new char[strlen(nameP)+1];
	strcpy(provider[l].name, nameP);
	if ( urlP ) {
		provider[l].url=new char[strlen(urlP)+1];
		strcpy(provider[l].url, urlP);
	}
	else
		provider[l].url = NULL;
	provider[l].prefix= new char[strlen(prefixP)+1];
	strcpy(provider[l].prefix, prefixP);
	provider[l].domain = new char[strlen(domainP)+1];
	strcpy(provider[l].domain, domainP);
	provider[l].ip = ipP;
	l++;
	return true;
}


const char *ProviderList::getDomainFromName(const char *name)
{
	int i=0;
	for( ; i<l && strcmp(provider[i].name, name)!=0 ; i++);
	return (i<l) ? provider[i].domain : NULL;
}


const char* ProviderList::getNameFromDomain(const char *domain)
{
	int i=0;
	for( ; i<l && strcmp (provider[i].domain, domain) ; i++);
	return (i<l) ? provider[i].name : 0;
}


const char* ProviderList::getPrefixFromDomain(const char *domain)
{
	int i=0;
	for( ; i<l && strcmp (provider[i].domain, domain) ; i++);
	return (i<l) ? provider[i].prefix : 0;
}


const char* ProviderList::getPrefixFromName(const char* name)
{
	int i=0;
	for( ; i<l && !strcmp(provider[i].name, name) ; i++)
		;
	return (i<l) ? provider[i].prefix : 0;
}


const char* ProviderList::getURLFromDomain(const char *domain)
{
	int i=0;
	for( ; i<l && strcmp (provider[i].domain, domain) ; i++);
	return (i<l) ? provider[i].url : 0;
}


const char* ProviderList::getURLFromName(const char* name)
{
	int i=0;
	for( ; i<l && !strcmp(provider[i].name, name) ; i++)
		;
	return (i<l) ? provider[i].url : 0;
	//null pointer if there is not the name in the provider list
}


bool ProviderList::isPrefixPresent(const char* url)
{
	int i, l1, l2, l3;
	l2 = strlen (url);
	char *url2 = NULL;
	for ( i = 6; url[i] && url[i] != ':'; i++ );
	if ( url[i] ) {
		int j;
		url2 = new char [l2];
		memcpy (url2, url, i);
		for ( j = i+1; url[j] >= '0' && url[j] <= '9'; j++ );
		strcpy (url2 + i, url + j);
		l3 = strlen (url2);
	}
	for ( i = 0; i < l; i++ ) {
		l1 = strlen (provider[i].prefix);
		if ( !strncmp(provider[i].prefix, url, (l1 < l2) ? l1 : l2)
				|| ( url2
				&& !strncmp(provider[i].prefix, url2, (l1 < l3) ? l1 : l3)) ) {
			delete url2;
			return true;
		}
		
	}
	logger()<<"Missing provider: "<<url<<'\n';
	delete url2;
	return false;
}


bool ProviderList::isDomainPresent (const char *domain)
{
	int i;
	for( i = 0; i < l; i++ )
		if ( !strcmp (provider[i].domain, domain) )
			return true;
	logger()<<"Missing secure provider: "<<domain<<'\n';
	return false;
}

bool ProviderList::isIPPresent (const IPv4Address IP) const
{
	int i;
	for( i = 0; i < l; i++ )
		if ( provider[i].ip == IP )
			return true;
	return false;
}

void ProviderList::listProviders (ostream &o)
{
	for ( int i = 0; i < l; i++ )
		if ( provider[i].url )
			o << provider[i].name << '\t' << provider[i].url << '\n';
}

IPv4Address ProviderList::getIPFromName(const char* name)
{
	int i=0;
	for( ; i<l && !strcmp(provider[i].name, name) ; i++)
		;
	return (i<l) ? provider[i].ip : IPv4Address("");
	//null pointer if there is not the name in the provider list
}



}


