/*##############################################################################
   
   Developed by NetMob Group, University of Trento, Italy

   University of Trento
   DIT - Department of Informatics and Telecommunications 
   Computer Networks and Mobility Research Program
   http://netmob.unitn.it/ 
   
   This file is part of the WilmaGate system, created as part
   of the WILMA Project, sponsored by the Autonomous
   Province of Trento and participated by ITC-irst, the University
   of Trento and Alpikom Spa. More info at

     http://www.wilmaproject.org/		

   To download the project:
      - in tarball format
        http://netmob.unitn.it/wilmagate.html
      - connect to CVS repository
        :pserver:cvs@rtm.unitn.it:/home/cvsroot
   In both previous cases, the download is password protected. To obtain
   the password you must send a mail to 

      wilmagate@dit.unitn.it

################################################################################

   The WilmaGate Licence of Use


   Copyright (c) 2003-2005
                - Universita` di Trento,
                  Dipartimento di Informatica e Telecomunicazioni (DIT)
                  Computer Networks and Mobility Research Program


   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (hereinafter
   the "WilmaGate") directly from the copyright owner, to use the WilmaGate
   without restriction of purpose or time, including without limitation the
   rights to use, copy, modify, and merge the WilmaGate, subject to the
   following conditions:

   The above copyright notice and this permission notice shall be included
   in all copies or substantial portions of the WilmaGate. 

   The WilmaGate shall not be sub-licenced either commercially or free of
   charge, as is or in part or as a part of any other package. 

   The licensee agrees to properly credit WilmaGate and its copyright 
   owners in any technical, scientific or divulgative publication, 
   whatever publishing means are used, that is based entirely or in part 
   on WilmaGate use, as well as in any booklet, web-site or brochure that 
   advertise services based on WilmaGate use. 

   If the WilmaGate is modified, the copyright owner shall be notified.

   THE WilmaGate IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

##############################################################################*/

#include "../Common/utilities.h"
#include <sstream>

#include "proxy_plugin.h"
#include "http_info.h"

using namespace std;

static char *replyHeader[] = {
	"HTTP/1.0 302 Temporarily Moved\r\n"
	"Location: ","?REP=","&AUTH=","&CLI=","&PORT=","&RPORT=","\r\n"
	"Content-Type: text/html\r\n"
	"Content-Length: ","\r\n"
	"Proxy-Connection: close\r\n"
	"\r\n"
};

static char *replyBody[] = {
	"<html>\r\n"
	"<body>\r\n"
	"Questa pagina dovrebbe rinviarvi subito verso il sito di autenticazione.<p>\r\n"
	"Se la ridirezione non avviene, <a href=\"","?REP=","&AUTH=","&CLI=","&PORT=","&RPORT=","\">fate clic qui</a>.\r\n"
	"</body>\r\n"
	"</html>\r\n"
	"\r\n"
};

enum RequestType {
	PROXY_NONE,
	PROXY_GET,
	PROXY_CONNECT
};

static RequestType parseRequest (char *req, int l, string &url)
{
	char rl[10000];

	req[l] = '\0';

	const char *p, *q;
	if ( strstr(req, "GET ") == req ) {
		p = strstr (req, "http://");
		if ( !p )
			return PROXY_NONE;
		q = strchr (p, ' ');
		if ( q ) {
			memcpy (rl, p, q-p);
			rl[q-p] = '\0';
		}
		else {
			int i;
			for ( i = 0; p[i] >= ' '; i++ )
				rl[i] = p[i];
			rl[i] = '\0';
		}
		url = rl;
		return PROXY_GET;
	}
	if ( strstr(req, "CONNECT ") == req ) {
		p = req + 8;
		for ( ; *p <= ' '; p++ );
		for ( q = p; *q && *q > ' ' && *q != ':'; q++ );
		if ( q != p ) {
			memcpy (rl, p, q-p);
			rl[q-p] = '\0';
			url = rl;
		}
		else
			url = "";
		return PROXY_CONNECT;
	}
	return PROXY_NONE;
}


namespace Gatekeeper {

ProxyPlugin::ProxyPlugin(PluginStatusTableInterface &p1, ProviderListInterface &p2,
			IPv4Address rip, uint16_t rport, IPv4Address aip, uint16_t aport, const char * red) :
	psti(p1),
	pli(p2),
	reportIP (rip),
	reportPort (rport),
	authIP (aip),
	authPort (aport),
	redirectURL (red)
{
	idPlugin = psti.registerPlugin();
}

ProxyPlugin::~ProxyPlugin (void)
{
}

bool ProxyPlugin::processPacket (EthernetFrame* ethf, EthernetFrame* vect[], PacketSense sense[])
{
	IPPacket *in = &(ethf->IPPayload());
	TCPDatagram *intcp = &(in->TCPPayload());

	if ( intcp->sourcePort != htons(3128) && intcp->destinationPort != htons(3128) )
		return false;

	EthernetMACAddress *m = psti.getMAC (in->destination);
	bool outgoing = !m;

	IPv4Address index = outgoing ? in->source : in->destination;
	HTTPInfo *hi = reinterpret_cast<HTTPInfo*>(psti.getPluginInfo (idPlugin, index));
	if ( !hi ) {
		hi = new HTTPInfo;
		hi->status = HTTPInfo::OPEN;
		psti.addPluginInfo (idPlugin, index, hi);
	}

	IPPacket *out = &(vect[0]->IPPayload());
	TCPDatagram *outtcp = &(out->TCPPayload());
	string url;

	if ( !outgoing ) {

		if ( hi->status == HTTPInfo::CLOSING && intcp->isFlag(TCPDatagram::FIN_FLAG) && intcp->isFlag(TCPDatagram::ACK_FLAG) ) {
			outtcp->setReplyHeader (intcp, ntohs(in->length) - intcp->getHeaderLength() + 1);
			outtcp->setOptionLength (0);
			outtcp->flags = TCPDatagram::ACK_FLAG;
			outtcp->calculateChecksum (in->source, in->destination, sizeof(TCPDatagram));
			out->setReplyHeader (in);
			out->length = htons (sizeof(IPPacket) + sizeof(TCPDatagram));
			out->calculateHeaderChecksum ();
			vect[0]->protocol = htons(EthernetFrame::IP_PROTOCOL);
			vect[0]->receiver = ethf->sender;
			sense[0] = SEND_BACK;
			sense[1] = END_OF_PACKETS;
			return true;
		}

		if ( hi->status == HTTPInfo::OPEN ) {
			ethf->receiver = *m; 
			sense[0] = SEND_SAME;
			sense[1] = END_OF_PACKETS;
			return true;
		}
		sense[0] = END_OF_PACKETS;
		return true;
	}

	if ( intcp->isFlag(TCPDatagram::SYN_FLAG) ) {
		hi->status = HTTPInfo::OPEN;
		sense[0] = SEND_SAME;
		sense[1] = END_OF_PACKETS;
		return true;
	}

	::RequestType rt = ::parseRequest (reinterpret_cast<char*>(intcp->getPayload()),
				ntohs (in->length) -  sizeof(IPPacket) - intcp->getHeaderLength(), url);

	if ( rt == ::PROXY_GET && !pli.isPrefixPresent(url.c_str()) ) {
		outtcp->setReplyHeader (intcp, ntohs(in->length) - in->getHeaderLength());
		outtcp->setOptionLength (0);
		ostringstream of;
		string rs;
		of << ::replyBody[0] << redirectURL << ::replyBody[1] << reportIP << ::replyBody[2] << authIP
			<< ::replyBody[3] << in->source << ::replyBody[4] << authPort << ::replyBody[5] << reportPort << ::replyBody[6];
		rs = of.str();
		of.str("");
		of << ::replyHeader[0] << redirectURL << ::replyHeader[1] << reportIP
			<< ::replyHeader[2] << authIP << ::replyHeader[3] << in->source
			<< ::replyHeader[4] << authPort << ::replyHeader[5] << reportPort
			<< ::replyHeader[6] << rs.length() << ::replyHeader[7] << rs;
		strcpy (reinterpret_cast<char*>(outtcp->payload), of.str().c_str());
		outtcp->flags = TCPDatagram::PSH_FLAG | TCPDatagram::ACK_FLAG | TCPDatagram::FIN_FLAG;
		outtcp->calculateChecksum (in->source, in->destination, sizeof(TCPDatagram) + of.str().length());
		out->setReplyHeader (in);
		out->length = htons (sizeof(IPPacket) + sizeof(TCPDatagram) + of.str().length());
		out->calculateHeaderChecksum ();
		vect[0]->receiver = ethf->sender;
		vect[0]->protocol = htons(EthernetFrame::IP_PROTOCOL);
		sense[0] = SEND_BACK;

		out = &(vect[1]->IPPayload());
		outtcp = &(out->TCPPayload());
		memcpy (out, in, sizeof(IPPacket) + sizeof(TCPDatagram));
		outtcp->setOptionLength (0);
		outtcp->flags = TCPDatagram::FIN_FLAG | TCPDatagram::ACK_FLAG;
		outtcp->calculateChecksum (in->source, in->destination, sizeof(TCPDatagram));
		out->length = htons (sizeof(IPPacket) + sizeof(TCPDatagram));
		out->calculateHeaderChecksum ();
		vect[1]->protocol = htons(EthernetFrame::IP_PROTOCOL);
		sense[1] = FORWARD;

		sense[2] = END_OF_PACKETS;

		hi->requestedURL = url;
		psti.setURL (in->source, url.c_str());
		hi->status = HTTPInfo::CLOSING;

		return true;
	}

	if ( rt == ::PROXY_CONNECT && !pli.isDomainPresent(url.c_str()) ) {
		sense[0] = END_OF_PACKETS;
		return true;
	}

	if ( hi->status == HTTPInfo::CLOSING ) {
		if ( intcp->isFlag(TCPDatagram::FIN_FLAG) && intcp->isFlag(TCPDatagram::ACK_FLAG) ) {
			outtcp->setReplyHeader (intcp, ntohs(in->length) - intcp->getHeaderLength() + 1);
			outtcp->setOptionLength (0);
			outtcp->flags = TCPDatagram::ACK_FLAG;
			outtcp->calculateChecksum (in->source, in->destination, sizeof(TCPDatagram));
			out->setReplyHeader (in);
			out->length = htons (sizeof(IPPacket) + sizeof(TCPDatagram));
			out->calculateHeaderChecksum ();
			vect[0]->protocol = htons(EthernetFrame::IP_PROTOCOL);
			vect[0]->receiver = ethf->sender;
			sense[0] = SEND_BACK;
			sense[1] = END_OF_PACKETS;
			return true;
		}
		sense[0] = END_OF_PACKETS;
		return true;
	}

	sense[0] = SEND_SAME;
	sense[1] = END_OF_PACKETS;
	return true;
}

}
