/// \file relay_plugin.cpp
/// \author Albino Scalet
///
///\brief implementation of DHCP relay agent features.
//// $Id: relay_plugin.cpp,v 1.5 2007/06/13 15:21:34 cvs Exp $

#include "relay_plugin.h"

using namespace Common;

namespace Gatekeeper {

	RelayPlugin::RelayPlugin (
						   PluginStatusTableInterface &p,
						   IPv4Address sip,
						   uint16_t sport,
						   uint16_t cport,
						   IPv4Address interface3IP,
						   string  interface3Name,
						   IPv4Address interface2IP,
						   IPv4Address NatIP,
						   IPv4Address 		      	mask,
						  int shortl,
						  int longl,
						  IPv4Address dns,
						  const char * wpadf
		)
		: psti(p)
		
		{

		wpad= new string (wpadf);
		
		//Inherithed from relay status table interface
		 rtl=NULL;
		
		psti.registerDHCPService (this);
		
   		shortLease=shortl;
		longLease=longl;
		dnsIP=dns;
		NatPublicIP=NatIP;

		//Pointers to packets of various protocols included in the set P= { ETHERNET, IP, UDP, DHCP}.
		ethin = new (ETHERMAXSIZE) EthernetFrame;
		ipin = &(ethin->IPPayload());
		udpin = &(ipin->UDPPayload());
		dhcpin = reinterpret_cast<DHCPMessage*>(udpin->payload);
		
		//Pointers to packets of various protocols included in the set P= { ETHERNET, IP, UDP, DHCP}.
		ethout = new (ETHERMAXSIZE) EthernetFrame;
		ipout = &(ethout->IPPayload());
		udpout = &(ipout->UDPPayload());
		dhcpout = reinterpret_cast<DHCPMessage*>(udpout->payload);
		interface[0]=NULL;
		
		//Binding the socket to the interface oriented towards the server.
	 	if(sip=="127.0.0.1") {
			relay.bind ("127.0.0.1", cport);
			logger() << "BINDING TO IP & Port :"<< "127.0.0.1"<<" "<< sport +1<< '\n';
		}
		else {  int res=0;
			res=relay.bind(interface2IP,cport);
			logger() << "BINDING TO IP & Port :"<< interface2IP <<" "<< cport<<" "<<res<< '\n';
		}	
		
		//Setting the destination of the client's message received.
		relay.setDestination (sip, sport);
		logger() << "SEND TO IP & Port :"<< sip <<" "<< sport << '\n';
		//Registering this router as a listener of socket.
		relay.registerListener (*this, 0);

		//Setting informations  that regards the interface oriented towards client subnet.
		toClientSubnet.InterfaceName=interface3Name;
		toClientSubnet.InterfaceIP=interface3IP;
		toClientSubnet.InterfaceMAC="00:00:00:00:00:00";
		toClientSubnet.netmask=mask;

		//Setting informations  that regards the interface oriented towards server subnet.
		toServerSubnet.InterfaceIP=interface2IP;

		//Setting informations  that regards the DHCP server.
		serverIP=sip;
		serverPort= sport;
		clientPort= cport;
	}

	RelayPlugin::~RelayPlugin (void) {
	}


	void RelayPlugin::sendFrame (EthernetFrame *eth, int l, PacketSense sense) {

		IPPacket* ip = &(eth->IPPayload());
		UDPDatagram* udp = &(ip->UDPPayload());
		DHCPMessage* dhcp = reinterpret_cast<DHCPMessage*>(udp->payload);
		//logger () <<"pkg received \n";
		
		//To control if packet received is a DHCP message and in particular a request.
		if ( 		       (eth->protocol == htons(EthernetFrame::IP_PROTOCOL) )
				&& (eth->IPPayload().protocol == IPPacket::UDP_PROTOCOL)
				&& (eth->IPPayload().UDPPayload().sourcePort == htons(0x0044)) 

				
	            		&& (dhcp->op==DHCPMessage::QUERY)
				//An intermediary relay intercepts a DHCP query in receivedPacket(socket listener).
		    		&&(dhcp->giaddr==static_cast<uint32_t>(0))
		    )
			{
		    	
			//#ifdef DEBUG
			//logger() << "SENDFRAME 1 Got packet from " << //eth->sender << " It's a DHCP request!!" << '\n';
			//#endif

			//Initialization of a variable that will contains the size of the final DHCP message built.
			int dhcplen=0;

			//A simple way to calculate the size of the static part of the final DHCP message built.
			dhcplen= (
				        (	 4 * 	sizeof(uint8_t) 				) 				+
				        (	 2 * 	sizeof(uint16_t) 				) 				+
				        (	 2 * 	sizeof(uint32_t) 				) 				+
				        (	 4 * 	sizeof(IPv4Address) 			) 				+
				        	       	sizeof(EthernetMACAddress) 					+
				        (      CHADDR -	sizeof(EthernetMACAddress)		)	+
				     	       SFILE 											+
				      	       SNAME
					);

			///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			//Static part of the DHCP Message will be treated from this point
			///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

			//logger()   << "DHCP Query catched " << '\n';
					
			//This field is used to say that message is a DHCP REQUEST.
			dhcpin->op = DHCPMessage::QUERY;

			//This field indicates the type of the hardware. For example it should be equal to Ethernet.
			dhcpin->htype = dhcp->htype;

			//This field indicates the length of the Hardware address Rapresentation.
			dhcpin->hlen = dhcp->hlen;

			//An hop is added because the client isn't in the same subnet of the DHCP Server.
			//In a real network topology may be present one or more Relay agents.
			dhcpin->hops= dhcp->hops +1 ;

			//This is an ID of the current transaction between Client and  Server.
			dhcpin->xid= dhcp->xid;

			//This field contains an indication of elapsed time.
			dhcpin->secs= dhcp->secs;

			//These are BOOTP flags.
			dhcpin->flags= dhcp->flags;

			//This is the client IP Address. Usually it is set to 0.0.0.0 when the DHCP Message is a Request.
			dhcpin->ciaddr= dhcp->ciaddr;

			//This is the IP Address offered to a client and it is usually set to 0.0.0.0  when the DHCP Message is a Request.
			dhcpin->yiaddr= dhcp->yiaddr;

			//This is the IP Address of a DHCP Server and is usually set to 0.0.0.0  when the DHCP Message is a Request.
			dhcpin->siaddr= dhcp->siaddr;

			//This is the relay agent IP address field. We will set it with the IP value of the interface that is oriented towards
			//the client Subnet.
			//mydhcp->giaddr= getClientInterfaceIP();
			//mydhcp->giaddr= getServerInterfaceIP();
			dhcpin->giaddr= getNatIP();
			
			//This field contains the client hardware address of the network interface on the host of the client that has sent the request.
			dhcpin->chaddr= dhcp->chaddr;

			//This is a padding field  for the chaddr field.
			strcpy(dhcpin->padding, dhcp->padding);

			//This field of 64 bytes may be filled with the name of the server.
			strcpy(dhcpin->sname, dhcp->sname);

			//This field of 128 bytes may be filled with the name of a remote configuration file.
			strcpy(dhcpin->file, dhcp->file);

			//The magic cookie signs the start of the option field.
			dhcpin->cookie= dhcp->cookie;

			////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			//Option field of a DHCPMESSAGE
			///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

			//This is an index used to flow into the option field of a DHCP message that relay has just received.
			int p= 0;

			//An index to flow into the options field of the DHCP Message that will be built and relayed.
			int optlen=0;

			DHCPMessage::OptionType otype = DHCPMessage::MESSAGE_TYPE;

			
			int i=0;
						
			#define WPAD
			
			#ifdef WPAD		
			DHCPMessage::OptionMessageType mtype = DHCPMessage::NONE;
			
			int queryType=0;
			#endif
			
			//We will looping into the DHCP options field until reaching the end delimiter
			//Adding a control on the position p+1 because some old DHCP clients represents an infinite lease time
			//with the same code of the end option (of course 0xff).
			while (	
			
			
					  //(
					  ( (dhcp->options[p]) !=DHCPMessage::END_OPTIONS)
					   //|| ((dhcp->options[p+1]) !=0))
					     && 
					     
					     //((p < (l- HEADERSIZE - DHCPSTATICSIZE - 1) ) && 
					     (p < DHCPOPTIONSMAXSIZE)
					     //)
					
				   )
				   
				   {
			//The options are untouched we will only append the relay agent information option before
			//the end delimiter.
				
				
				otype = static_cast<DHCPMessage::OptionType>(dhcp->options[p]);
				
				if(otype == DHCPMessage::END_OPTIONS)
					break;
				
				#ifdef WPAD		
				if ( otype == DHCPMessage::MESSAGE_TYPE ) {
					
			
					mtype=static_cast<DHCPMessage::OptionMessageType>(dhcp->options[p+2]);
					//logger()  << mtype << " is mtype value " << '\n';
				
					
					if (mtype  == DHCPMessage::INFORM) {
						queryType=1;
						//dhcpin->options[optlen++]=DHCPMessage::MESSAGE_TYPE;
						//dhcpin->options[optlen++]=1;
						//dhcpin->options[optlen++]=DHCPMessage::INFORM;
						//p+=3;
						//logger()  << mtype << " Offer catched " << '\n';
						
					}
					//dealloco il record della ST
					if (mtype  == DHCPMessage::RELEASE) {
						psti.setStatus(dhcpin->ciaddr, StatusTableInterface::FREE,false);
						//dhcpin->options[optlen++]=DHCPMessage::MESSAGE_TYPE;
						//dhcpin->options[optlen++]=1;
						//dhcpin->options[optlen++]=DHCPMessage::INFORM;
						//p+=3;
						logger()  << "DHCP RELEASE from: "<<dhcpin->ciaddr << "\n";
						
					}
				
					
				}
				
				#endif
				i=p;
				
				if(otype == DHCPMessage::HOST_NAME) {
					for(;p<i+dhcp->options[i+1]+2;p++) {
						if(dhcp->options[p]!=0x00)
							dhcpin->options[optlen++]=dhcp->options[p];
						
					}
						
				
				}
				else if(otype ==0x00)
				{
					dhcpin->options[optlen++]=dhcp->options[p];
					p++;	
				
				}
				else {
					for( ;p<i+dhcp->options[i+1]+2;p++) 
					dhcpin->options[optlen++]=dhcp->options[p];
				
					p=i+dhcp->options[i+1]+2;
				}

			}  //end while.

						
			#ifdef WPAD
			if(queryType==1) {
			
			logger() <<otype <<  " DHCP INFORM Packet" << '\n';
				
				/*
				dhcpin->op = DHCPMessage::RESPONSE;
				
				dhcpin->options[optlen++] = DHCPMessage::WPAD_URL;
				dhcpin->options[optlen++] = strlen(wpad->c_str());
		        	memcpy(&(dhcpin->options[optlen]),wpad->c_str(),strlen(wpad->c_str()));
				optlen+=strlen(wpad->c_str());
			
				//The construction of the DHCP message ends appending the end options.
				dhcpin->options[optlen]=DHCPMessage::END_OPTIONS;
				optlen++;

				//Now we adding the variable size of the option field to the fixed size of the DHCP message.
				dhcplen+=optlen;
				
				UDPSocket relay2;
				relay2.bind(getClientInterfaceIP(),getClientPort());
				relay2.setDestination (dhcpin->ciaddr, getClientPort()+1);
				relay2.send(dhcpin,dhcplen);
				return;
				*/
				
			}
			#endif
			
			
			//logger()  << optlen << "Adding RAI Option" << '\n';



		/////Subnet_SELECTION option
		IPv4Address subnet=getClientSubnetMask();

		//Code of SUBNET_SELECTION suboption.
		dhcpin->options[optlen]=DHCPMessage::SUBNET_SELECTION;
		optlen++;

		//The size of the payload for the suboption.
		dhcpin->options[optlen]= sizeof(IPv4Address);
		optlen++;
		
       	subnet= getClientInterfaceIP() & subnet;

		memcpy(&dhcpin->options[optlen] , &subnet,  sizeof(IPv4Address) ) ;

		optlen+=sizeof(IPv4Address);

	


			//RAI option (code number 82) in a separate procedure.
			raioBuild(dhcpin, optlen);

			//The construction of the DHCP message ends appending the end options.
			dhcpin->options[optlen]=DHCPMessage::END_OPTIONS;
			optlen++;

			//Variable size of the option field to the fixed size of the DHCP message.
			dhcplen+=optlen;


			//Sending the message to server.
			if(!(relay.send (dhcpin, dhcplen)))
			logger()  << "Error in forwarding message to server\n" <<  '\n';


			#ifdef DEBUG
			logger()  << "SER <<--------------| Forwarding a DHCP Request  |-------------- CLI\n" <<  '\n';
			# endif

		}  //end if.
		else {
			#ifdef DEBUG
			logger() << " another packet " << eth->sender<<  eth->receiver<<  '\n';
			# endif
		}

	}   //end sendFrame.

	void RelayPlugin::registerListener (PacketListener *t) {
		
		interface[0]= t;
	}

	RelayPlugin::SuggestedAction RelayPlugin::failed (int socketID) {
		
		return CONTINUE;
	}

	RelayPlugin::SuggestedAction RelayPlugin::receivedPacket (uint8_t *message, int size, int socketID, IPv4Address senderIP,uint16_t senderPort) {
						
		#ifdef DEBUG
		logger() << "Received packet from : "<<  senderIP<<"  on port  "<< senderPort << endl;
		#endif
		//logger() << "Received packet from : "<<  senderIP<<"  on port  "<< senderPort << endl;
		DHCPMessage* dhcp = reinterpret_cast<DHCPMessage*>(message);

		//True when a DHCP RESPONSE message is received.
		
		if( ( senderPort==getServerPort() )&& (senderIP==getServerIP())&&( (dhcp->op==DHCPMessage::RESPONSE)) ) {

			//size of the packet sent.
			int ethlen=0;

			//size of IP header + IP payload.
			int iplen=0;

			//size of UDP header + UDP payload.
			int udplen=0;

			//size of DHCP message (options included).
			int dhcplen=0;

			#ifdef DEBUG
			logger() << "It's a DHCP reply!!\n";
			logger() << "Received packet from : "<<  senderIP<<"  on port  "<< senderPort << endl;
			#endif

			//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			// Ethernet Header
			//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

			//MAC address of the source of the message will be the Ethernet address of the interface
			//oriented towards client subnet.
			ethout->sender = getClientInterfaceMAC();

			//MAC address of the destination of the message will be the Ethernet address of the interface of the client machine. This information is stored in chaddr field of the dhcp message.
			ethout->receiver = dhcp->chaddr;

			//Setting the type of the protocol.
			ethout->protocol = htons(EthernetFrame::IP_PROTOCOL);

			//Adding to the counter the size of the Ethernet Header.
			ethlen+= ( 2 * sizeof(EthernetMACAddress) ) + sizeof(uint16_t);

			//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			// IP Header
 			// Total size and checksum fields will be added  at the end of the costruction of the  UDP datagram.
			//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

			//The type of ip packet is IPv4.
			ipout->version= IPv4;

			//Setting Differentiated Services to 0.
			ipout->diffserv=0;

			//Setting IP Identification Field to 0.
			ipout->id=0;

			//Setting IP fragment offset to 0.
			ipout->offset=0;

			//Setting IP field time to live to 0.
			//The same value set in server responses relayed by ISC DHCP Relay.
			ipout->ttl= TTL;

			//Setting IP field protocol type.
			ipout->protocol=IPPacket::UDP_PROTOCOL;

			//This field contains IP address of relay's interface that is oriented towards client subnet.
			ipout->source = getClientInterfaceIP();

			//This field contains the client IP address equal to yiaddr field of DHCP message received.
			if(dhcp->ciaddr=="0.0.0.0")
				ipout->destination = 0xFFFFFFFFUL;
			else
				ipout->destination = dhcp->yiaddr; 
			//This is the size of the IP header.
			iplen=  ( 4 * sizeof(uint8_t) + 4 * sizeof(uint16_t) + 2 * sizeof(IPv4Address) );


			//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			// UDP Header.
			//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

			//Port from which the message is arrived.
			//myudp->sourcePort= htons(senderPort);
			udpout->sourcePort= htons(0x0043);//htons(getServerPort());
			
			//The server answer is directed to bootpc port (68).
			udpout->destinationPort= htons(0x0044);//htons(getClientPort());

			//size of the UDP header.
			udplen=  ( 4 * sizeof(uint16_t)  );

			//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			// DHCP message
			//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

			//message is a DHCP RESPONSE.
			dhcpout->op = dhcp->op;

			//Hardware type. For example it may be equal to Ethernet.
			dhcpout->htype = dhcp->htype;

			//Length of the Hardware address.
			dhcpout->hlen = dhcp->hlen;

			//An hop is added because the client isn't in the same subnet of the DHCP Server.
			//In a real network topology may be present one or more Relay agents.
			dhcpout->hops= dhcp->hops;

			//This is an ID of the current transaction between Client and  Server.
			dhcpout->xid= dhcp->xid;

			//Indication of elapsed time.
			dhcpout->secs= dhcp->secs;

			//These are BOOTP flags.
			dhcpout->flags= dhcp->flags;

			//This is the client IP Address.
			dhcpout->ciaddr= dhcp->ciaddr;

			//This is the IP Address offered to a client.
			dhcpout->yiaddr= dhcp->yiaddr;

			//This is the IP Address of a DHCP Server.
   			dhcpout->siaddr= dhcp->siaddr;

			//This is the relay agent IP address field. 
            //IP value of the interface that is oriented towards
			//the client Subnet.
			dhcpout->giaddr= dhcp->giaddr ;

			//This field contains the client hardware address of the network interface on the host of the client that has sent the 				request.
			dhcpout->chaddr= dhcp->chaddr;

			//This is a padding field  for the chaddr field.
			strcpy(dhcpout->padding, dhcp->padding);

			//This field of 64 bytes may be filled with the name of the server.
			strcpy(dhcpout->sname, dhcp->sname);

			//This field of 128 bytes may be filled with the name of a remote configuration file.
			strcpy(dhcpout->file, dhcp->file);

			//The magic cookie signs the start of the option field.
			dhcpout->cookie= dhcp->cookie;

			///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			//Option field of a DHCPMESSAGE
			///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

			//Index used to flow into the option field of a DHCP message.
			int p=0;

			//Index used to flow into the options field of the DHCP Message.
			int optlen=0;

			// 1 if response is an OFFER 2 if response is an ACK
			int responseType=0;
			DHCPMessage::OptionType otype = DHCPMessage::MESSAGE_TYPE;

			DHCPMessage::OptionMessageType mtype = DHCPMessage::NONE;

			//A variable used for keep track of lease time option
			//which will be overwrite. 
			int leaseTimeIndex=0;
			int i=0;
			int skip=0;
      
			//loop into the DHCP options field until the end delimiter.
			while ( 
			 p < DHCPOPTIONSMAXSIZE && (dhcp->options[p] != DHCPMessage::END_OPTIONS)
			)
			 {
         skip=0;
         
			//The options are untouched 
            //we will only append the relay agent information option before
			//the end delimiter.
				otype = static_cast<DHCPMessage::OptionType>(dhcp->options[p]);
				if(otype == DHCPMessage::AGENT_OPTION)
					break;
        //new code
        //else if (otype==DHCPMessage::SUBNET_SELECTION) {
           //skipping this option
          // skip=1;
        //}
        else
				if ( otype == DHCPMessage::MESSAGE_TYPE ) {
					
					mtype=static_cast<DHCPMessage::OptionMessageType>(dhcp->options[p+2]);
									
					if (mtype  == DHCPMessage::OFFER) {
						responseType=RESPONSEISOFFER;
						//logger()  << mtype << " Offer catched " << '\n';
					
					}
					else if (mtype == DHCPMessage::ACK) {
						responseType=RESPONSEISACK;
					}
				}else
				
				if ( otype == DHCPMessage::LEASE_TIME ) {
					leaseTimeIndex=p;
				}
				
				if(otype==0x00) {
					dhcpout->options[optlen++]=dhcp->options[p];
					p++;
				
				}
				else {
					
					i=p;
					for( ;p<i+dhcp->options[i+1]+2;p++) 
						dhcpout->options[optlen++]=dhcp->options[p];
					p=i+dhcp->options[i+1]+2;
				}
        

			}  //end while.

			//Appending Router and DNS information to response packet
		//	dhcpout->options[optlen++] = DHCPMessage::ROUTER;
		//	dhcpout->options[optlen++] = 4;
		//	*reinterpret_cast<IPv4Address*>(dhcpout->options+optlen) = getClientInterfaceIP();
		//	optlen += 4;

			//dhcpout->options[optlen++] = DHCPMessage::DNS;
			//dhcpout->options[optlen++] = 4;
			//*reinterpret_cast<IPv4Address*>(dhcpin->options+optlen) = getDnsIP();
			//optlen += 4;
						
		//	logger()  << optlen<< "  : " <<p << " Building WPAD option " << '\n';
/*
			if(responseType==RESPONSEISACK) {
			
				dhcpout->options[optlen++] = DHCPMessage::WPAD_URL;
				dhcpout->options[optlen++] = strlen(wpad->c_str())+1;
   	            memcpy(&(dhcpout->options[optlen]),wpad->c_str(),strlen(wpad->c_str())+1);
				optlen+=strlen(wpad->c_str())+1;
        dhcpout->options[optlen-1] = 0x0a;
        
			}
*/			       
			
			
			//Appending the end options.
			dhcpout->options[optlen]=DHCPMessage::END_OPTIONS;
			optlen++;

			//A small amount of padding is added. 10 bytes + sizeof RAIO bytes.
			int j;

			for( j =0; j < PADDINGSIZE; j++) {
				dhcpout->options[optlen]=0;
				optlen++;
			}

			//A simple way to calculate the size of the static part of the final DHCP Message Built.
			dhcplen=(
				  (	 4 * 	sizeof(uint8_t) ) 				+
				  (	 2 * 	sizeof(uint16_t) ) 				+
				  (	 2 * 	sizeof(uint32_t) ) 				+
				  (	 4 * 	sizeof(IPv4Address) ) 			+
				               	sizeof(EthernetMACAddress) 	+
				  (  CHADDR  -	sizeof(EthernetMACAddress))    	+
				     SFILE +
				     SNAME
			);

			//Adding the size of the options field.
			dhcplen+=optlen;

			///////////////////////////////////////////////////////////////////////////////
			//Calculating the size of UDP datagram
			///////////////////////////////////////////////////////////////////////////////

			//The size of UDP Datagram.
			//Adding the length of UDP Payload of course  dhcplen.
			udplen+=dhcplen;

			//Setting UDP size (header + payload) into UDP  packet.
			udpout->length=htons(udplen);

			//////////////////////////////////////////////////////////////////////////////
			//Calculating the size and the checksum of the IP message
			///////////////////////////////////////////////////////////////////////////////

			//Adding the length of IP Payload of course udplen.
			iplen+=udplen;

			//Setting IP size (header + payload) into IP  packet.
			ipout->length =htons(iplen);

			
			/////////////////////////////////////////////////////////////////////////////
			//Finally we will send the Ethernet Frame
			///////////////////////////////////////////////////////////////////////////////

			//Sending the ethernet frame.
			#ifdef DEBUG
			showETHmessage( ethout);
			showIPmessage( ipout);
			showUDPmessage( udpout);
			showDHCPmessage( dhcpout);
			#endif

			///////////////////////////////////////////////////////////////////////////////
			//Refresh status table information
			///////////////////////////////////////////////////////////////////////////////
			
			StatusTableInterface::statusType status;


		if(responseType==RESPONSEISOFFER) {
			
			psti.associateMACtoIP (dhcpout->chaddr ,dhcpout->yiaddr);
			
            
            status = psti.getStatus (dhcpout->yiaddr);

            psti.setIPMode(dhcpout->yiaddr,true);
             
      
			if ( status < StatusTableInterface::AUTHORIZED )
			{ 	


                            psti.setStatus(dhcpout->yiaddr, StatusTableInterface::FIRST_SEEN);	
			}
			status = psti.getStatus (dhcpout->yiaddr);
			time_t lease = static_cast<time_t>((status == StatusTableInterface::AUTHORIZED) ? longLease : 					shortLease);
	
			psti.setExpirationIP(dhcpout->yiaddr, lease);
			
			int leaset =htonl(lease);
			memcpy(&(dhcpout->options[leaseTimeIndex+2]),&leaset,sizeof(lease));
      
      logger()  <<"Offering IP  "<<dhcpout->yiaddr<<" to  "<< dhcpout->chaddr  << '\n';
			
		}
		else if(responseType==RESPONSEISACK) {
	   		//true means obtained with DHCP
	   		
			psti.associateMACtoIP (dhcpout->chaddr ,dhcpout->yiaddr);
		  //DHCP albino here
           psti.setIPMode(dhcpout->yiaddr,true);
           
           status = psti.getStatus (dhcpout->yiaddr);

			if ( status < StatusTableInterface::AUTHORIZED )
			{
				if(psti.isPreAuthenticated(dhcpout->chaddr,dhcpout->yiaddr)) {        //albino here
                  psti.setStatus(dhcpout->yiaddr ,  StatusTableInterface::AUTHORIZED); //albino here
        }
        else	{
      
                          if(psti.getToken(dhcpout->yiaddr)==NULL) {
                                      psti.setStatus(dhcpout->yiaddr ,  StatusTableInterface::TEMPORARY_LEASE,true);
          }
          else {
                              psti.setStatus(dhcpout->yiaddr ,  StatusTableInterface::TEMPORARY_LEASE);
          }
          
        }
			}
      
				
			status = psti.getStatus (dhcpout->yiaddr);
			time_t lease = static_cast<time_t>((status == StatusTableInterface::AUTHORIZED) ? longLease : 					shortLease);

			int leaset =htonl(lease);
			memcpy(&(dhcpout->options[leaseTimeIndex+2]),&leaset,sizeof(lease));
		
			psti.setExpirationIP(dhcpout->yiaddr , lease);
					
			logger()  <<"IP  "<<dhcpout->yiaddr<<"  leased  to  "<< dhcpout->chaddr  << '\n';
			
		
		}
		
			//Initializing the IP checksum field.
			//This field will be fill calling the calculateHeaderChecksum after the
			//calculus of Total Size field.
			ipout->checksum = 0;

			//Calculating the IP checksum field.
			ipout->calculateHeaderChecksum();
			
			//Adding IP size (header + payload) to the total size of message.
			ethlen+=iplen;

			//And now the UDP checksum is calculated taking each parameter
			//from IP layer.
			//udpout->calculateChecksum (ipout->source, ipout->destination, htons( udpout->length));
			//MB - 20070613
			udpout->checksum = 0;
		       // udpout->checksum=udp->checksum;
			if(rtl) {			
			//A dispatcher method is invoked
			rtl->sendResponseFrame(ethout, ethlen);
			
			
			}
			
		} //end if DHCPmsg is RESPONSE.
		else {
			//#ifdef DEBUG
			logger()  << " NAK or another response "  <<  '\n';
			//	#endif
		}

			#ifdef STATS
			showStats();
			#endif


		return CONTINUE;
	}   //end receivedPacket.


	///////////////////////////////////////////////////////////////////////////////////////////////
	//Building RELAY AGENT OPTIONS
	///////////////////////////////////////////////////////////////////////////////////////////////

	void RelayPlugin::raioBuild(DHCPMessage* & myd, int & optlen)  {
		//The Relay Agent Information Option consists of a suboption called
		//circuitID that contains the name of the interface from which
		//the packet was received.

		//Used to keep track of the index for option 82 length field

		//The RAIO option starts with its own identification code that is equal to 82.
		myd->options[optlen]= DHCPMessage::AGENT_OPTION;
		optlen++;

		//With this operation we will fill the field that contains  size of the  payload for the option 82
		//The payload is composed from 2 bytes that are used for suboption encoding and
		//for saving the size of suboption payload.
		myd->options[optlen]=2+ (getClientInterfaceName()).length() ;
		optlen++;

		//Field that contains the code of CIRCUIT_ID suboption.
		myd->options[optlen]=DHCPMessage::CIRCUIT_ID;
		optlen++;

		//Size of the payload for the suboption.
		myd->options[optlen]= getClientInterfaceName().length();
		optlen++;

		//Name of the interface that is oriented on the client subnet.
		string inf= getClientInterfaceName();

		unsigned i;

		//Writing the name of the interface into the circuitID suboption payload field.
		for(i=0;  i < (getClientInterfaceName()).length();  i++)

			myd->options[optlen+i]=inf[i];

		//Refreshing the dinamic size of the options fields.
		optlen+=(getClientInterfaceName()).length();

		return;
	}	//end RAIObuild.


	//A simple method to get access to the name of the interface oriented towards the client.
	string 	RelayPlugin::getClientInterfaceName() {

		return toClientSubnet.InterfaceName;
	}

	//A simple method to get access to the IP of the interface oriented towards the client.
	IPv4Address  RelayPlugin::getClientInterfaceIP() {

		return toClientSubnet.InterfaceIP;
	}

	//A simple method to get access to the MAC of the interface oriented towards the client.
	EthernetMACAddress  	RelayPlugin::getClientInterfaceMAC() {

		return toClientSubnet.InterfaceMAC;
	}

	//A simple method to get the netmask of  the client subnet
	IPv4Address  	RelayPlugin::getClientSubnetMask() {

		return toClientSubnet.netmask;
	}


	//A simple method to get the IP of DHCP server.
	IPv4Address  RelayPlugin::getServerIP() {

		return serverIP;
	}

	//A simple method to get the port of DHCP server.
	uint16_t  RelayPlugin::getServerPort() {

		return serverPort;
	}
	
	
	
	//A simple method to get the port of DHCP server.
	uint16_t  RelayPlugin::getClientPort() {

		return clientPort;
	}

	//A simple method to get access to the IP of the interface oriented towards the server.
	IPv4Address  RelayPlugin::getServerInterfaceIP() {

		return toServerSubnet.InterfaceIP;
	}
	
	
	
	//A simple method to get the IP of dns server.
	IPv4Address  RelayPlugin::getDnsIP() {
		return dnsIP;
	}

	IPv4Address RelayPlugin::getNatIP(){
		return NatPublicIP;
	
	}
	
	

	bool RelayPlugin::processPacket (EthernetFrame *pointerToPacket, EthernetFrame* instructionVector[],
	PacketSense sense[]) {

		EthernetFrame *ethf=pointerToPacket;
		IPPacket* ip = &(ethf->IPPayload());
		UDPDatagram* udp = &(ip->UDPPayload());
		DHCPMessage* dhcp = reinterpret_cast<DHCPMessage*>(udp->payload);

		if ( ethf->isProtocol (EthernetFrame::IP_PROTOCOL)
		      && ethf->IPPayload().protocol == IPPacket::UDP_PROTOCOL
		      && (ethf->IPPayload().UDPPayload().destinationPort == htons(getClientPort() ))
		      && (dhcp->op==DHCPMessage::QUERY)
		) {

			this->sendFrame(ethf, ip->length+ETHHEADERSIZE ,FORWARD);

			sense[0]=END_OF_PACKETS;
			return true;
		}

	  return false;
	} //processPacket


	//Build a DHCP release message for external server when a lease is expired
	
	void RelayPlugin::sendRelease (IPv4Address ip, EthernetMACAddress &mac)
	{
		 //Initialization of a variable that will contains the size of the final DHCP message built.
		int dhcplen=0;
		
		/// Pointer to a memory area devoted to a DHCP RELEASE message.
		///
		DHCPMessage *dhcprelease = new DHCPMessage;
		///
		

		//A simple way to calculate the size of the static part of the final DHCP message built.
		dhcplen= (
				(	 4 * 	sizeof(uint8_t) 				) 				+
				(	 2 * 	sizeof(uint16_t) 				) 				+
				(	 2 * 	sizeof(uint32_t) 				) 				+
				(	 4 * 	sizeof(IPv4Address) 			) 				+
						sizeof(EthernetMACAddress) 					+
				(      CHADDR -	sizeof(EthernetMACAddress)		)	+
					SFILE 											+
					SNAME
				);

		  
		dhcprelease->op = DHCPMessage::QUERY;

		//This field indicates the type of the hardware. For example it should be equal to Ethernet.
		dhcprelease->htype = 1;

		//This field indicates the length of the Hardware address Rapresentation.
		dhcprelease->hlen = 6;

		//An hop is added because the client isn't in the same subnet of the DHCP Server.
		//In a real network topology may be present one or more Relay agents.
		dhcprelease->hops= 1 ;

		//This is an ID of the current transaction between Client and  Server.
		dhcprelease->xid= 0;

		//This field contains an indication of elapsed time.
		dhcprelease->secs= 0;

		//These are BOOTP flags.
		dhcprelease->flags= 0;

		//This is the client IP Address. Usually it is set to 0.0.0.0 when the DHCP Message is a Request.
		dhcprelease->ciaddr= ip;

		//This is the IP Address offered to a client and it is usually set to 0.0.0.0  when the DHCP Message is a Request.
		dhcprelease->yiaddr= "0.0.0.0";

		//This is the IP Address of a DHCP Server and is usually set to 0.0.0.0  when the DHCP Message is a Request.
		dhcprelease->siaddr="0.0.0.0";

		//This is the relay agent IP address field. We will set it with the IP value of the interface that is oriented towards
		//the client Subnet.
		//dhcpin->giaddr= getClientInterfaceIP();
		
		dhcprelease->giaddr=getNatIP();

		//dhcpin->giaddr= getServerInterfaceIP();

		//This field contains the client hardware address of the network interface on the host of the client that has sent the request.
		dhcprelease->chaddr= mac;
		
		//This is a padding field  for the chaddr field.
		int cnt;
        for(cnt=sizeof(EthernetMACAddress);cnt<CHADDR;cnt++)
                dhcprelease->padding[cnt]=0;
  
        //strcpy(dhcprelease->padding, dhcpin->padding);
		
		//This field of 64 bytes may be filled with the name of the server.
		//strcpy(dhcprelease->sname, dhcpin->sname);
		
    	for(cnt=0;cnt<SNAME;cnt++)
                dhcprelease->sname[cnt]=0;
  
		
		//This field of 128 bytes may be filled with the name of a remote configuration file.
		//strcpy(dhcprelease->file, dhcpin->file);
		for(cnt=0;cnt<SFILE;cnt++)
                dhcprelease->sname[cnt]=0;
  

		//The magic cookie signs the start of the option field.
		dhcprelease->cookie= dhcpin->cookie;

		///////////////////////////////////////////////////////////////////////////////////////
		//Now we will build the option field of a DHCPMESSAGE
		///////////////////////////////////////////////////////////////////////////////////////

		//This is an index used to flow into the option field of a DHCP message that relay has just received.
		int p= 0;

		//An index to flow into the options field of the DHCP Message that will be built and relayed.
		int optlen=0;

		//We will looping into the DHCP options field until reaching the end delimiter
		//Adding a control on the position p+1 because some old DHCP clients represents an infinite lease time
		//with the same code of the end option (of course 0xff).
		dhcprelease->options[p]=DHCPMessage::MESSAGE_TYPE;
		optlen++;
		p++;
		
		dhcprelease->options[p]=1;
		optlen++;
		p++;
		
		dhcprelease->options[p]=DHCPMessage::RELEASE;
		optlen++;
		p++;
		
		//Adding SUBNET_SELECTION OPT
		IPv4Address subnet = getClientSubnetMask();
		dhcprelease->options[optlen]=DHCPMessage::SUBNET_SELECTION;
		optlen++;
		dhcprelease->options[optlen]=sizeof(IPv4Address);
		optlen++;
		subnet=getClientInterfaceIP() & subnet;
		memcpy(&dhcprelease->options[optlen],&subnet,sizeof(IPv4Address));
		optlen+=sizeof(IPv4Address);
		
		#ifdef DEBUG
		logger()  << optlen << "Adding RAI Option" << "\n";
		# endif



		//We will build the RAI option (code number 82) in a separate procedure.
		raioBuild(dhcprelease, optlen);

		//The construction of the DHCP message ends appending the end options.
		dhcprelease->options[optlen]=DHCPMessage::END_OPTIONS;
		optlen++;

		//Now we adding the variable size of the option field to the fixed size of the DHCP message.
		dhcplen+=optlen;

		logger() << " Sending DHCP release for "<< ip <<" to server" << '\n';
		//Sending the message to server.
		if(! (relay.send (dhcprelease, dhcplen)) )
		logger() << " Error in Sending DHCP release \n" ;
		

		delete dhcprelease;




	}
	
	
	
	
	//Invoked from status table when a short lease expires
	
	void RelayPlugin::statusChanged (IPv4Address ip, EthernetMACAddress &mac, StatusTableInterface::statusType oldStatus,
					StatusTableInterface::statusType newStatus)
	{
		
	
	}
	
	
	
	void RelayPlugin::registerDHCPService(RelayListener* pointerToDispatcher)
	{
		rtl=pointerToDispatcher;
	
	}
	
	
	
	
	
	
	
} // namespace Gatekeeper

