/*##############################################################################
   
   Developed by NetMob Group, University of Trento, Italy

   University of Trento
   DIT - Department of Informatics and Telecommunications 
   Computer Networks and Mobility Research Program
   http://netmob.unitn.it/ 
   
   This file is part of the WilmaGate system, created as part
   of the WILMA Project, sponsored by the Autonomous
   Province of Trento and participated by ITC-irst, the University
   of Trento and Alpikom Spa. More info at

     http://www.wilmaproject.org/		

   To download the project:
      - in tarball format
        http://netmob.unitn.it/wilmagate.html
      - connect to CVS repository
        :pserver:cvs@rtm.unitn.it:/home/cvsroot
   In both previous cases, the download is password protected. To obtain
   the password you must send a mail to 

      wilmagate@dit.unitn.it

################################################################################

   The WilmaGate Licence of Use


   Copyright (c) 2003-2005
                - Universita` di Trento,
                  Dipartimento di Informatica e Telecomunicazioni (DIT)
                  Computer Networks and Mobility Research Program


   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (hereinafter
   the "WilmaGate") directly from the copyright owner, to use the WilmaGate
   without restriction of purpose or time, including without limitation the
   rights to use, copy, modify, and merge the WilmaGate, subject to the
   following conditions:

   The above copyright notice and this permission notice shall be included
   in all copies or substantial portions of the WilmaGate. 

   The WilmaGate shall not be sub-licenced either commercially or free of
   charge, as is or in part or as a part of any other package. 

   The licensee agrees to properly credit WilmaGate and its copyright 
   owners in any technical, scientific or divulgative publication, 
   whatever publishing means are used, that is based entirely or in part 
   on WilmaGate use, as well as in any booklet, web-site or brochure that 
   advertise services based on WilmaGate use. 

   If the WilmaGate is modified, the copyright owner shall be notified.

   THE WilmaGate IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

##############################################################################*/

#include "../Common/utilities.h"
#include "report.h"
#include <cstring>
#include <sstream>

namespace Gatekeeper {

Report::Report(ReportStatusTableInterface &rstio,
		ProviderListInterface &plio,
		IPv4Address listenIP, uint16_t listenPort,
		IPv4Address gwIP, uint16_t gwPort)
			throw (WilmaGateException):
	rsti (rstio),
	pli (plio),
	gatewayIP (gwIP),
	gatewayPort (gwPort)
{
		
	rsti.registerListener (this);
	
	for ( int i = 0; i < 256; tcpconn[i++] = NULL );
	if ( ! skTCPin.bind(listenIP, listenPort) )
		throw WilmaGateException ("Reporter not bound", 446633);
	skTCPin.registerConnectionListener(*this, 1000);

	
}

Report::~Report() {}

void Report::statusChanged (IPv4Address ip, EthernetMACAddress &mac, StatusTableInterface::statusType oldStatus,
					StatusTableInterface::statusType newStatus)
{
	if ( (newStatus == StatusTableInterface::AUTHORIZED && oldStatus < StatusTableInterface::AUTHORIZED)
		|| (newStatus < StatusTableInterface::AUTHORIZED && oldStatus == StatusTableInterface::AUTHORIZED) ) {
		TCPSocket *s = new TCPSocket;
		if ( ! s->setDestination (gatewayIP, gatewayPort) ) {
			logger() << "Gateway unreachable, could not commit changes.\n";
			delete s;
			return;
		}
		ostringstream os;
		if ( newStatus == StatusTableInterface::AUTHORIZED )
			os << "enable " << ip << ' ' << mac << endl;
		else
			os << "disable " << ip << endl;
		if ( ! s->send (os.str().c_str(), os.str().length()) ) {
			logger() << "Gateway was connected, but error on send: changes not committed.\n";
			delete s;
			return;
		}
		incomingConnection (s, 0);
	}
}

SocketListener::SuggestedAction Report::receivedPacket (uint8_t *message, int size, int socketID,
								IPv4Address senderIP, uint16_t senderPort)
{
	if ( socketID >= 256 )
		return DELETE;
	message[size] = '\0';
	string ins = reinterpret_cast<char*>(message);
	istringstream in (ins);
	string command;
	in >> command;
	if ( command == "providers" ) {
		IPv4Address IP;
		in >> IP;
		ostringstream of;
		Token *t = rsti.getToken (IP);
		if ( t )
			of << *t << '\n';
		else
			of << "0\n";
		pli.listProviders (of);
		of << endl;
		tcpconn[socketID]->send (of.str().c_str(), of.str().length());
	}
	else if ( command == "list" ) {
		int level = 0;
		in >> level;
		ostringstream of;
		rsti.listUsers (of, level ? static_cast<StatusTableInterface::statusType>(level)
					: StatusTableInterface::AUTHORIZED);
		of << endl;
		tcpconn[socketID]->send (of.str().c_str(), of.str().length());
	}
	else if ( command == "OK" ) {
		IPv4Address IP;
		EthernetMACAddress e;
		bool a;
		uint64_t intrf, outtrf;
		in >> IP >> e >> a >> intrf >> outtrf;
		if ( !a )
			logger() << "Traffic by " << IP << ": " << intrf << " in, " << outtrf << " out." << endl;
		tcpconn[socketID] = NULL;
		return DELETE;
	}
	else if ( command == "detail" ) {
		IPv4Address IP;
		in >> IP;
		ostringstream of;
		char name[200];
		char email[200];
		time_t expireIP, expireAuth;
		StatusTableInterface::statusType status;
		bool ok = rsti.getUserData (IP, name, email, &expireIP, &expireAuth, &status, 200);
		if ( *name == '\0' )
			strcpy (name, "none");
		if ( *email == '\0' )
			strcpy (email, "none");
		if ( ok ) {
			of << "OK\t" << time(NULL) << '\t' << status << '\t' << name << '\t' << email << '\t' << expireIP << '\t'
				<< expireAuth << endl;
		}
		else
			of << "Error: user " << IP << " not in table." << endl;
		tcpconn[socketID]->send (of.str().c_str(), of.str().length());
	}
	else {
		tcpconn[socketID] = NULL;
		return DELETE;
	}
	return CONTINUE;
}


SocketListener::SuggestedAction Report::incomingConnection (TCPSocket *s, int socketID)
{
	int i;
	for ( i = 0; i < 256 && tcpconn[i]; i++ );
	if ( i == 256 )
		return DELETE;
	tcpconn[i] = s;
	s->registerListener (*this, i);
	s->startReceiving ();
	return CONTINUE;
}

SocketListener::SuggestedAction Report::failed (int socketID)
{
	return (socketID == 1000) ? STOP : DELETE;
}

//For DHCP relay plugin
	void Report::sendRelease (IPv4Address ip, EthernetMACAddress &mac) {
	}

}
