///
/// \file report_statustable_interface.h
///	\author Danilo Severina
///
/// The file contains the definitions of functions that are used
///	to access the Status Table to read and/or write 
///	data. Not all of functions that read and write informations in Status
/// Table must be used by Report, so this class has the function of interface:
///	only the methods that are described here can be used by Report.
///	The class derives from the StatusTableInterface and so it acan use all the methods 
///	included in that class.
/// The file contains also pure virtual function and their implementation are in the StatusTable 
///	class.
///
///	$Id: report_statustable_interface.h,v 1.3 2005/06/16 13:22:22 cvs Exp $
///

/*##############################################################################
   
   Developed by NetMob Group, University of Trento, Italy

   University of Trento
   DIT - Department of Informatics and Telecommunications 
   Computer Networks and Mobility Research Program
   http://netmob.unitn.it/ 
   
   This file is part of the WilmaGate system, created as part
   of the WILMA Project, sponsored by the Autonomous
   Province of Trento and participated by ITC-irst, the University
   of Trento and Alpikom Spa. More info at

     http://www.wilmaproject.org/		

   To download the project:
      - in tarball format
        http://netmob.unitn.it/wilmagate.html
      - connect to CVS repository
        :pserver:cvs@rtm.unitn.it:/home/cvsroot
   In both previous cases, the download is password protected. To obtain
   the password you must send a mail to 

      wilmagate@dit.unitn.it

################################################################################

   The WilmaGate Licence of Use


   Copyright (c) 2003-2005
                - Universita` di Trento,
                  Dipartimento di Informatica e Telecomunicazioni (DIT)
                  Computer Networks and Mobility Research Program


   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (hereinafter
   the "WilmaGate") directly from the copyright owner, to use the WilmaGate
   without restriction of purpose or time, including without limitation the
   rights to use, copy, modify, and merge the WilmaGate, subject to the
   following conditions:

   The above copyright notice and this permission notice shall be included
   in all copies or substantial portions of the WilmaGate. 

   The WilmaGate shall not be sub-licenced either commercially or free of
   charge, as is or in part or as a part of any other package. 

   The licensee agrees to properly credit WilmaGate and its copyright 
   owners in any technical, scientific or divulgative publication, 
   whatever publishing means are used, that is based entirely or in part 
   on WilmaGate use, as well as in any booklet, web-site or brochure that 
   advertise services based on WilmaGate use. 

   If the WilmaGate is modified, the copyright owner shall be notified.

   THE WilmaGate IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

##############################################################################*/

#ifndef _REPORT_STATUSTABLE_INTERFACE_
#define	_REPORT_STATUSTABLE_INTERFACE_

#include <iostream>
#include "statustable_interface.h"
//#include "token.h"
//#include "addresses.h"
#include "statustable_listener.h"

using namespace std;


namespace Gatekeeper {

///
/// The file contains the definitions of functions that are used
///	to access the Status Table to read and/or write 
///	data. Not all of functions that read and write informations in Status
/// Table must be used by Report, so this class has the function of interface:
///	only the methods that are described here can be used by Report.
///	The class derives from the StatusTableInterface and so it acan use all the methods 
///	included in that class.
/// The file contains also pure virtual function and their implementation are in the StatusTable 
///	class.
///

class ReportStatusTableInterface : virtual public StatusTableInterface{

private:

public:

	///
	///	\brief Restire a listener for the %Report.
	///
	///	The function is used by the StatusTable to notify changes in the Status Table to the Report.
	///	When there is a change of the status of an IP address the Report must be informed, because it
	///	must send the new informations to the Gateway wich updates its own list of users.
	///	The main task of the function is to send to the StatusTabel a pointer to the 
	///	StatusTableListener. Really parameter is a the pointer to the Report object created in 
	///	main function.
	///	The function is implemented in the StatusTable class.
	///	\param pointerToReport is a pointer to Report object, but it is defined as a pointer to
	///	the interface between StatusTable anda Report.
	///	
	virtual void registerListener(StatusTableListener *pointerToReport) = 0;

	///
	///	\brief Get the number of users. The dimension of the Status Table
	///
	virtual int getNumberUser() = 0;

	///
	///	\brief
	///
	virtual IPv4Address getFirstIP() = 0;

	///
	/// \brief List all users above a given authorization status on an output stream.
	///
	/// \param o Output stream.
	/// \param l Minimum authorization status.
	///
	virtual void listUsers (ostream &o, statusType l) = 0;

	///
	/// \brief Get status of a user.
	///
	virtual bool getUserData (IPv4Address ip, char *name, char *email, time_t *expIP, time_t *expAuth, statusType *s, int bufsize) = 0;
};

}

#endif

