///
/// \file http_info.h
///	\author Danilo Severina
///
///	The file contains the definition of function to read and write informations about
///	a user for the HTTPPlugin.
///
///	$Id: http_info.h,v 1.2 2005/06/16 13:22:22 cvs Exp $
///	

/*##############################################################################
   
   Developed by NetMob Group, University of Trento, Italy

   University of Trento
   DIT - Department of Informatics and Telecommunications 
   Computer Networks and Mobility Research Program
   http://netmob.unitn.it/ 
   
   This file is part of the WilmaGate system, created as part
   of the WILMA Project, sponsored by the Autonomous
   Province of Trento and participated by ITC-irst, the University
   of Trento and Alpikom Spa. More info at

     http://www.wilmaproject.org/		

   To download the project:
      - in tarball format
        http://netmob.unitn.it/wilmagate.html
      - connect to CVS repository
        :pserver:cvs@rtm.unitn.it:/home/cvsroot
   In both previous cases, the download is password protected. To obtain
   the password you must send a mail to 

      wilmagate@dit.unitn.it

################################################################################

   The WilmaGate Licence of Use


   Copyright (c) 2003-2005
                - Universita` di Trento,
                  Dipartimento di Informatica e Telecomunicazioni (DIT)
                  Computer Networks and Mobility Research Program


   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (hereinafter
   the "WilmaGate") directly from the copyright owner, to use the WilmaGate
   without restriction of purpose or time, including without limitation the
   rights to use, copy, modify, and merge the WilmaGate, subject to the
   following conditions:

   The above copyright notice and this permission notice shall be included
   in all copies or substantial portions of the WilmaGate. 

   The WilmaGate shall not be sub-licenced either commercially or free of
   charge, as is or in part or as a part of any other package. 

   The licensee agrees to properly credit WilmaGate and its copyright 
   owners in any technical, scientific or divulgative publication, 
   whatever publishing means are used, that is based entirely or in part 
   on WilmaGate use, as well as in any booklet, web-site or brochure that 
   advertise services based on WilmaGate use. 

   If the WilmaGate is modified, the copyright owner shall be notified.

   THE WilmaGate IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

##############################################################################*/

#ifndef SIP_INFO_H_
#define SIP_INFO_H_

#include "plugin_info.h"
#include <inttypes.h>

namespace Gatekeeper {
	
const int BRANCH_NUM = 10;

///
///	\brief Contains additional SIP information about a user.
///
///	The class contains the definition of function to read and write informations
///	about a user for the SIPlugin.
///
class SIPInfo : public PluginInfo{

public:

	struct branch_t{
		///
		/// \brief SIP transaction identifier.
		///
		char *branch;
		///
		/// \brief Expiration time
		///
		int expires;
		//struct branch_t *next;
		time_t time;	
		
	} sbranch[BRANCH_NUM];
	
	//char *branch;	
	///
	/// \brief Number of SIP registration renew
	///
	//int expires;

	///
	/// \brief Number of SIP registration renew
	///
	int renew_num;
	
	///false
	/// \brief Virtual constructor.
	///
	SIPInfo();

	///
	///	\brief Virtual Destructor.
	///
	virtual ~SIPInfo();
	
	/*
	void add_branch(char* branch, int expires){
		
		int i;
		for(i = 0; i < BRANCH_NUM; i++){
			if(!sbranch[i].branch && sbranch[i].time < (time(NULL) + 10)){
				cout << "add_branch " << i << ' ';
				sbranch[i].time = time(NULL);
				sbranch[i].branch = branch;
				sbranch[i].expires = expires;
				break;
			}
		}
	}
	*/
	
	/*
	struct branch_t* get_branch(char* branch){
		
		int i;
		for(i = 0; i < BRANCH_NUM; i++){
			if(sbranch[i].branch && !strcmp(sbranch[i].branch, branch))
				return &sbranch[i];
		}
		return NULL;
	}
	*/

	//
	// Inherited from PluginInfo
	//
	virtual void read(istream &is);
	virtual void write (ostream &os);

};

}

#endif



