/*##############################################################################
   
   Developed by NetMob Group, University of Trento, Italy

   University of Trento
   DIT - Department of Informatics and Telecommunications 
   Computer Networks and Mobility Research Program
   http://netmob.unitn.it/ 
   
   This file is part of the WilmaGate system, created as part
   of the WILMA Project, sponsored by the Autonomous
   Province of Trento and participated by ITC-irst, the University
   of Trento and Alpikom Spa. More info at

   http://www.wilmaproject.org/		

   To download the project:
      - in tarball format
        http://netmob.unitn.it/wilmagate.html
      - connect to CVS repository
        :pserver:cvs@rtm.unitn.it:/home/cvsroot
   In both previous cases, the download is password protected. To obtain
   the password you must send a mail to 

      wilmagate@dit.unitn.it

################################################################################

   The WilmaGate Licence of Use


   Copyright (c) 2003-2005
                - Universita` di Trento,
                  Dipartimento di Informatica e Telecomunicazioni (DIT)
                  Computer Networks and Mobility Research Program


   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (hereinafter
   the "WilmaGate") directly from the copyright owner, to use the WilmaGate
   without restriction of purpose or time, including without limitation the
   rights to use, copy, modify, and merge the WilmaGate, subject to the
   following conditions:

   The above copyright notice and this permission notice shall be included
   in all copies or substantial portions of the WilmaGate. 

   The WilmaGate shall not be sub-licenced either commercially or free of
   charge, as is or in part or as a part of any other package. 

   The licensee agrees to properly credit WilmaGate and its copyright 
   owners in any technical, scientific or divulgative publication, 
   whatever publishing means are used, that is based entirely or in part 
   on WilmaGate use, as well as in any booklet, web-site or brochure that 
   advertise services based on WilmaGate use. 

   If the WilmaGate is modified, the copyright owner shall be notified.

   THE WilmaGate IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

##############################################################################*/

#include "../Common/utilities.h"
#include <sstream>

#include "sip_plugin.h"
#include "sip_info.h"

using namespace std;

namespace Gatekeeper {
	

SipPlugin::SipPlugin(PluginStatusTableInterface &p1, SipProviders &p2, int d): psti(p1),  sp(p2), debug(d){
	idPlugin = psti.registerPlugin();
}
	
SipPlugin::~SipPlugin (void){	}

bool SipPlugin::paramscomp(char *line, char *header, char **params, char **buf){
	
	int i,j;
	char *tmp;
	//the string starts with the header
	if(!strncasecmp(header, line, strlen(header))){
		//read line
		if(!params){
			j = 0;
			tmp = line + strlen(header) + 1;
			while(tmp[j] != '\r')
				if(!j && tmp[j] == ' ') tmp++;
				else ++j;
			buf[0] = new char[j + 1];
			memcpy(buf[0], tmp, j);
			buf[0][j] = '\0';//add the standard string terminator  
		}
		else{		
			//the last params must be equal to 0
			for(i = 0; params[i]; ++i){
				j = 0;
				tmp = strstr(line, params[i]);//find the position of param into the string
				if(tmp){
					tmp += strlen(params[i]) + 1;//skip the header's name. Ex: 'branch='
					while(tmp[j] != ',' && tmp[j] != ';' && tmp[j] != '\r') ++j;
					buf[i] = new char[j + 1];
					memcpy(buf[i], tmp, j);
					buf[i][j] = '\0';//add string terminator
					}
					else buf[i] = 0;
				}
			}
			return true;
		}
	return false;
}

bool SipPlugin::getUserInfo(char *line, char *header, char** destination, char chrstart, char chrend, int* retPosition){
	
	int j=0, start=0, end=0, init=0;
	char *tmp;
	char *buf=NULL;
	bool found=false;
	//the string starts with the header
	if(!strncasecmp(header, line, strlen(header))){
		if((*retPosition) !=0)//to remember if display-name has been found
			init=(*retPosition);
		//read line
    	   	tmp = line + strlen(header) + 1;
		while(tmp[j] != '\r')
			if(!j && tmp[j] == ' ') tmp++;
			else ++j;
		buf = new char[j + 1];
		if (buf == NULL) {
 		  // unable to allocate block of memory !
 		  logger () << "[Sip_Plugin] Unable to alloc memory in function getUserInfo" <<endl; 
 		  return false;
		}
		memcpy(buf, tmp, j);
		buf[j] = '\0';//add the standard string terminator
		for(int ind=init;ind<j && !found;ind++){
			if(buf[ind]==chrstart){//is the initial delimiter present?
				start=ind;
				found=true;
			}
		}	
		if(found){
			start++;//Point to the first character of the searched character
			found=false;
			for(int ind=j-1;ind>=start && !found;ind--){
				if(buf[ind]==chrend){//Point to the end delimiter
					end=ind;
					found=true;
				}
			}	
		}
		if(end--){
			*destination = new char[end-start+3];//plus the null terminator
			if (*destination == NULL) {
 		  		// unable to allocate block of memory !
 		  		logger () << "[Sip_Plugin] Unable to alloc memory in function getUserInfo" <<endl;
 		  		if (buf != NULL) {//to avoid memory leaks!
 		  			delete [] buf;
 		  		} 
 		  		return false;
			}
			for(int t=start, index=0;t<end+1;t++,index++){
			  	(*destination)[index]=buf[t];
			}//then copy the found string into the destination variable
			(*destination)[end-start+1]='\0'; 
			(*retPosition)=end+2 ;
			delete [] buf;//to avoid memory leaks!
	 		return true;
		}
	}
	return false;
}

char* SipPlugin::readline(char* payload, int length, int *offset, char delim){
	int i, j;
/* //Very verbose...
if (debug){cerr << '\t'; }*/
	for(i = *offset, j = 0; i < length && payload[i] != delim; i++, j++){
/*
if (debug){cerr << payload[i];}*/
	}
	if(!j) return NULL;
/* //Very verbose...
if (debug){ cerr << endl; }*/
	*offset += j + 2;
	return payload + (*offset - j - 2);
}


	
bool SipPlugin::processPacket(EthernetFrame* ethf, EthernetFrame* vect[], PacketSense sense[]){
	/*
	* Extract the IP packet from the Ethernet frame
	*/
	IPPacket *in = &(ethf->IPPayload());
	/*
	 * Allow only UDP traffic
	 */
	if(ethf->IPPayload().protocol != IPPacket::UDP_PROTOCOL)
		return false;
	/*
	 * Extract the UDP datagram from the IP packet
	 */ 
	UDPDatagram *inudp = &(in->UDPPayload());
	/*
	 * SIP Server listen for requests on the default SIP ports
	 * (5060 for TCP and UDP, 5061 for TLS over TCP)
	 * on the interfaces.
	 */
	if(inudp->sourcePort != htons(5060) && inudp->destinationPort != htons(5060))
		return false;
	/*
	 * In order to verify if the packet goes from the WLAN interface to the
	 * Internet of viceverse we try to get the MAC address associated to the
	 * destination IP address
	 */
	EthernetMACAddress *mac = psti.getMAC(in->destination);
	/*
	 * The MAC address equal to NULL says us that the
	 * packet goes from the WLAN interface to the
	 * Internet.
	 */	
	bool outgoing = !mac;
	
	Gatekeeper::SipPlugin::method_t method = SipPlugin::DUMMY;

	if(outgoing && !sp.isIPPresent(in->destination)){
		
		logger() << "[Sip_Plugin] Discard SIP message: client " << in->source << " (" << ethf->sender << ')'
				 << " to un-trusted SIP Provider " << in->destination << endl;
		return false;
	}
	else if(!outgoing && !sp.isIPPresent(in->source)){
		
		logger() << "[Sip_Plugin] Discard SIP message: un-trusted SIP Provider " << in->source << 
		" to  client " << in->destination << " (" << *mac << ')' << endl;
		return false;
	}
	
	/*
	 *	Get the SIP information about the user 
	 */	
	IPv4Address index = outgoing ? in->source : in->destination;
	SIPInfo *sipinf = reinterpret_cast<SIPInfo*>(psti.getPluginInfo(idPlugin, index));	
	if(!sipinf){
		sipinf = new SIPInfo;
		psti.addPluginInfo(idPlugin, index, sipinf);
	}
	/*
	 * Extracts the UDP payload
	 */
	char *payload = reinterpret_cast<char*>(inudp->payload),
		 *line = 0,
		 *branch = 0,
		 *expires = 0,
		 *cseq = 0,
		 *buf[1] = {0},
		 *from_name= 0,
		 *from_account=0,
		 *via_params[2] = {"branch", 0},
		 *contact_params[2] = {"expires", 0};
		 
	 
	uint16_t udp_len = ntohs(inudp->length);
	int	length = reinterpret_cast<unsigned short int>(udp_len) - 8,
		offset = 0,
		code = 0,
		i = 0;
		
	/*
	 * The first line contains:
	 * - or SIP method;
	 * - or server response;
	 */
/* //Very verbose...
if (debug){
		cerr << "\n[SipPlugin]: src: " << in->source << ":" << htons(inudp->sourcePort) << " (" << ethf->sender << "), dst:" 
			 << in->destination << ":" << htons(inudp->destinationPort) << " (" << ethf->receiver << ") length: " << length << endl;
}
*/
	if((line = readline(payload, length, &offset, '\r'))){

		if(!strncmp("REGISTER", line, 8))
			method = SipPlugin::REGISTER;
		else if(!strncmp("SIP/2.0", line, 7)){
			method = SipPlugin::SIP_2_0;
			if(!strncmp("100", line+8, 3))//Trying
				code = 100;
			else if(!strncmp("200", line+8, 3))//Ok
				code = 200;
			else if(!strncmp("401", line+8, 3))//Unauthorized
				code = 401;
		}
	}
	
	/*
	 * Parse all headers
	 */
	int quot_ptr=0;
	while((line = readline(payload, length, &offset, '\r'))){
		if(!from_name){ //From header: search display-name
			getUserInfo(line, "From", &from_name, '\"','\"', &quot_ptr); 
		}
		if(!from_account){//From header: search for account
			getUserInfo(line, "From", &from_account, '<','>',&quot_ptr); 
		}

		if(paramscomp(line, "Via", via_params, buf)){//Via header
			/*
			 * Via header can be include more the one time:
			 * 
			 * Via: .....
			 * Via: .....
			 * 
			 * Take the first because it's the one of the outbound proxy
			 */
			if(!branch)
				branch = buf[0];
		}else if(paramscomp(line, "Contact", contact_params, buf)){//Contact header
			if(!expires)
				expires = buf[0];
		}else if(paramscomp(line, "Expires", 0, buf)){//Expires header
			if(!expires)
				expires = buf[0];
		}
		else if(paramscomp(line, "CSeq", 0, buf)){//Expires header
				cseq = buf[0];					
			}
		}

if (debug){
				logger() << "[Sip_Plugin] " << endl;
			logger() << "[Sip_Plugin] " << "################## Useful SIP Info #######################" << endl;
			logger() << "[Sip_Plugin] " << "\tmethod " << method << endl;
			logger() << "[Sip_Plugin] " << "\tuser_name " <<  (from_name ?from_name : "NULL") << endl;
			logger() << "[Sip_Plugin] " << "\tuser_account " <<  (from_account ?from_account : "NULL") << endl;
			logger() << "[Sip_Plugin] " << "\tcode " << code << endl;
			logger() << "[Sip_Plugin] " << "\tbranch " << (branch ? branch : "NULL") << endl;
			logger() << "[Sip_Plugin] " << "\tcseq " << (cseq ? cseq : "NULL") << endl;
			logger() << "[Sip_Plugin] " << "\texpires " << (expires ? expires : "NULL") << endl;
			logger() << "[Sip_Plugin] " << "##########################################################" << endl;
}
    
        
			char boh[]="UNKNOWN";
			from_name= (from_name? from_name : boh);
			from_account= (from_account? from_account :  boh);
	if(outgoing){
		//2 cases:
		//- new registration
		//- renew the registration
		if(branch && method == SipPlugin::REGISTER){
/* //Very verbose...
if (debug)
	cerr << "[SipPlugin]: " << in->source << "(" << ethf->sender << ")" << " REGISTER ";
*/
			//save the 'branch' string (transaction id)
			for(i = 0; i < BRANCH_NUM; i++){
				if(!sipinf->sbranch[i].branch || sipinf->sbranch[i].time + 5 < time(NULL)){
/* //Very verbose...
if (debug)
	cerr << "packet[" << i << "] ";
*/
					if(sipinf->sbranch[i].branch){
						delete []sipinf->sbranch[i].branch;
					}
					sipinf->sbranch[i].time = time(NULL);
					sipinf->sbranch[i].branch = branch;
					branch = 0;
					if(expires)
						sipinf->sbranch[i].expires = atoi(expires);
/* //Very verbose...
if (debug){
				cerr << "branch=" << sipinf->sbranch[i].branch << " expires=" << sipinf->sbranch[i].expires << endl;
}
*/
					break;
				}
			}
		}
		else if(method == SipPlugin::REGISTER){
			logger() << "[Sip_Plugin] " << in->source << "(" << ethf->sender << ")"
				 << " REGISTER without branch parameter (RFC 2543)"; 
			
		}
		
	}
	else{
		/*
		 * HOP by HOP method needs to modify the MAC destination 
		 */
		ethf->receiver = *mac;

		SIPInfo::branch_t *usrbranch = NULL;
		for(i = 0; i < BRANCH_NUM; i++){
			if(branch && sipinf->sbranch[i].branch && !strcmp(sipinf->sbranch[i].branch, branch)){
				usrbranch = &sipinf->sbranch[i];
				break;
			}
		}
		
		if(usrbranch){
/* //Very verbose...
if (debug)
	cerr << "[SipPlugin]: " << in->source << "(" << ethf->sender << ") ";
*/
		}
		
		switch (code) {
			case 100://Trying
					if(usrbranch){
if (debug){
	logger() << "[Sip_Plugin] 100 - Trying, (branch="<< usrbranch->branch <<')' << endl;
}
					}
					break;
				case 200://OK

					if(usrbranch){
if (debug){
	logger() << "[Sip_Plugin] 200 - OK, (branch="<< usrbranch->branch <<')' << endl;
}
						/* 
					 * SIP registration
					 */
					if(expires){
						usrbranch->expires = atoi(expires);
						if(!sipinf->renew_num)
							logger() << "[Sip_Plugin] IP " << in->destination << " (" << *mac 
									 << ") SIP registration ok (expires = " << expires << " seconds)" << endl;
						else
							logger() << "[Sip_Plugin] IP " << in->destination << " (" << *mac  
									 << ") SIP registration renew " << sipinf->renew_num
									 << " ok (expires = " << expires << " seconds)" << endl;
						
						if(sp.getAuthMode(in->source) == SipProviders::PASSIVE){								
							logger() << "[Sip_Plugin] SIP register "<< in->destination <<", user "<<from_name << ", SIP authenticator is "<< in->source << " - PASSIVE mode" << endl;
							psti.authorizeUserFor(in->destination, StatusTableInterface::SIP,from_name,from_account);
							//psti.setStatus(in->destination, StatusTableInterface::AUTHORIZED);
							psti.setExpirationAuth(in->destination, atoi(expires), false);
						}
						else if(sp.getAuthMode(in->source) == SipProviders::ACTIVE){
							logger() << "[Sip_Plugin] SIP register "<< in->destination <<", user "<<from_name <<", SIP authenticator is "<< in->source << " - ACTIVE mode" << endl;
						}
						
						sipinf->renew_num++;
					}
					//SIP de-registration
					else{			
						if(sp.getAuthMode(in->source) == SipProviders::PASSIVE){
						  if(psti.isSIPauthenticated(in->destination)){
							logger() << "[Sip_Plugin] SIP de-register "<< in->destination <<", user "<<from_name <<", SIP authenticator is "<< in->source << " - PASSIVE mode" << endl;
							psti.setStatus(in->destination, StatusTableInterface::TEMPORARY_LEASE, false);						
							psti.removeAuthType(in->destination, StatusTableInterface::SIP);	 
						  }else{
						    	logger() << "[Sip_Plugin] Ignore SIP de-registration: user "<<in->destination <<" not authenticated with SIP" << endl;
						  }
						}
						else if(sp.getAuthMode(in->source) == SipProviders::ACTIVE){
							logger() << "[Sip_Plugin] SIP de-register "<< in->destination <<", user "<<from_name <<", SIP authenticator is "<< in->source << " - ACTIVE mode" << endl;
						}	
						sipinf->renew_num = 0;
						usrbranch->expires = -1;
					}
					delete []usrbranch->branch;
					usrbranch->branch = 0;
				}
				break;
			case 401:/* 
					  * Unauthorized 
					  */
					if(usrbranch){
if (debug){
	logger() << "[Sip_Plugin] 401 - Unauthorized, (branch="<< usrbranch->branch <<')' << endl;
}
						delete []usrbranch->branch;
						usrbranch->branch = 0;
						usrbranch->expires = -1;
					}
					break;
				default:
					if(usrbranch){
if (debug){
	logger() << "[Sip_Plugin] "<< code << " - (some meaning...see RFC), (branch="<< usrbranch->branch <<')'<< endl;
}
						delete []usrbranch->branch;
						usrbranch->branch = 0;
						usrbranch->expires = -1;
					}
					break;
			}
		}
		
		if(branch)
			delete [] branch;
		if(expires)
			delete [] expires;
		if(cseq)
			delete [] cseq;
		if(from_name && (from_name!=boh))
			delete [] from_name;
		if(from_account && (from_account!=boh))
			delete [] from_account;
	/*
	* - or the packet goes from outside to inside (check about trusting of
	* sender has been already made at the beginning of the method...)
	* - or the pair (IP, MAC) is AUTHORIZED
	* - or the SIP method requested by the client is a REGISTER
	*/
		if(!outgoing 
		|| psti.getStatus(in->source) == StatusTableInterface::AUTHORIZED 
		|| method == REGISTER){
			sense[0] = SEND_SAME;
			sense[1] = END_OF_PACKETS;	
			return true;
		}
		return false;
	}
}
