/*##############################################################################
   
   Developed by NetMob Group, University of Trento, Italy

   University of Trento
   DIT - Department of Informatics and Telecommunications 
   Computer Networks and Mobility Research Program
   http://netmob.unitn.it/ 
   
   This file is part of the WilmaGate system, created as part
   of the WILMA Project, sponsored by the Autonomous
   Province of Trento and participated by ITC-irst, the University
   of Trento and Alpikom Spa. More info at

     http://www.wilmaproject.org/		

   To download the project:
      - in tarball format
        http://netmob.unitn.it/wilmagate.html
      - connect to CVS repository
        :pserver:cvs@rtm.unitn.it:/home/cvsroot
   In both previous cases, the download is password protected. To obtain
   the password you must send a mail to 

      wilmagate@dit.unitn.it

################################################################################

   The WilmaGate Licence of Use


   Copyright (c) 2003-2005
                - Universita` di Trento,
                  Dipartimento di Informatica e Telecomunicazioni (DIT)
                  Computer Networks and Mobility Research Program


   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (hereinafter
   the "WilmaGate") directly from the copyright owner, to use the WilmaGate
   without restriction of purpose or time, including without limitation the
   rights to use, copy, modify, and merge the WilmaGate, subject to the
   following conditions:

   The above copyright notice and this permission notice shall be included
   in all copies or substantial portions of the WilmaGate. 

   The WilmaGate shall not be sub-licenced either commercially or free of
   charge, as is or in part or as a part of any other package. 

   The licensee agrees to properly credit WilmaGate and its copyright 
   owners in any technical, scientific or divulgative publication, 
   whatever publishing means are used, that is based entirely or in part 
   on WilmaGate use, as well as in any booklet, web-site or brochure that 
   advertise services based on WilmaGate use. 

   If the WilmaGate is modified, the copyright owner shall be notified.

   THE WilmaGate IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

##############################################################################*/

#include "../Common/utilities.h"
#include <iostream>
#include "status_table.h"
#include <string>

extern "C" {
	#include <syslog.h>
}
 

using namespace std;
using namespace Common;

namespace Gatekeeper {

static const int ROWSPACE = 1000;
static const int BUFFERSPACE = 200;

StatusTable::StatusTable(IPv4Address ipa, IPv4Address minIP, IPv4Address maxIP, const char *fname,time_t timeWin)
{
	int i;
	stl = NULL;
	dtl = NULL;
	
	//Albino here
	currAuth= 0;
	timeWindow=timeWin;	

	this->nPlugin=0;

	IPv4Address ip = ipa;
	for ( i = 0; i < MAX_USER; i++, ++ip ) {
		user[ip.host()].changed = false;
		user[ip.host()].ip = ip;
		user[ip.host()].status = (ip.host() >= minIP.host() && ip.host() <= maxIP.host()) ? FREE : FORBIDDEN;
		user[ip.host()].token = NULL;
		user[ip.host()].name = NULL;
		user[ip.host()].email = NULL;
		user[ip.host()].expireIP = 0;
		user[ip.host()].expireAuth = 0;
		user[ip.host()].dhcpRenewsAuth = false;
		user[ip.host()].url = "";
		//albino here
    		user[ip.host()].IPMode = false;
		//SIP 
		user[ip.host()].authType = NONE;
	}

	// Register this object to the timer
	timer.registerListener (this);

	bool thatsokay = true;
	persistenceFile.open (fname, ios::in);
	if ( persistenceFile.good() ) {
		char buffer[BUFFERSPACE];
		IPv4Address ip;
		for ( i = 0; i < MAX_USER; i++ ) {
			persistenceFile.seekg (i * ROWSPACE, ios::beg);
			if ( persistenceFile.fail() ) {
				logger() << "Not enough rows in file - file reset!\n";
				thatsokay = false;
				break;
			}
			ip.fromBinaryStream (persistenceFile);
			if ( ip != user[i].ip ) {
				logger() << "Stored IP (" << user[i].ip << ") not corresponding to table (" << ip << ")- File reset!\n";
				thatsokay = false;
				break;
			}
			user[i].mac.fromBinaryStream (persistenceFile);
			user[i].token = Token::fromBinaryStream (persistenceFile);
			persistenceFile.read (reinterpret_cast<char*>(&(user[i].status)), sizeof(statusType));
			persistenceFile.read (reinterpret_cast<char*>(&(user[i].expireIP)), sizeof(time_t));
			persistenceFile.read (reinterpret_cast<char*>(&(user[i].expireAuth)), sizeof(time_t));
			persistenceFile.read (reinterpret_cast<char*>(&(user[i].authType)), sizeof(authTypes));
			persistenceFile.read (buffer, BUFFERSPACE);
			if ( *buffer ) {
				delete user[i].name;
				user[i].name = strdup (buffer);
			}
			persistenceFile.read (buffer, BUFFERSPACE);
			if ( *buffer ) {
				delete user[i].email;
				user[i].email = strdup (buffer);
			}
		}
	}
	else
		thatsokay = false;

	persistenceFile.close ();
	persistenceFile.clear ();

	if ( !thatsokay ) {
		logger() << "Status file creation\n";
		persistenceFile.open (fname, ios::out | ios::trunc);
		if ( !persistenceFile )
			logger() << "ERROR: missing persistence functionalities!!\n";
		for ( i = 0; i < MAX_USER; i++ )
			writeRowToFile (i);
	}
	else
		persistenceFile.open (fname, ios::in|ios::out);

	timerTick ();

}

StatusTable::~StatusTable() 
{
	int i, j;
	// Delete for all users
	for ( i=0 ; i<MAX_USER ; i++)
	{
		// Delete each element in the PluginInfo array
		for ( j=0 ; j<this->nPlugin ; j++)
			delete user[i].info[j];
		delete user[i].info;
	}
}

bool StatusTable::setStatus(IPv4Address ip, statusType newStatus, bool newToken)
{	
	int index=ip.host();
	
	ar[index].beginShared();
	if ( user[index].ip == ip ) {


		if ( newStatus < user[index].status /* && newStatus > FREE*/ )	
			user[index].url = "";
		if ( stl )
			stl->statusChanged (ip, user[index].mac, user[index].status, newStatus);
		ar[index].endShared();
		ar[index].beginExclusive();
		user[index].changed = true;
		user[index].status = newStatus;
		if ( newToken ) {
			delete user[index].token;
    			user[index].token = new Token (16);
    		}
		ar[index].endExclusive();
		if ( newStatus >= TEMPORARY_LEASE ) {
			EthernetMACAddress mac = user[index].mac;
			for ( int i = 0; i < MAX_USER; i++ ) {
				if ( i == index )
					continue;
				ar[i].beginShared();
				if ( user[i].mac == mac ) {
					ar[i].endShared();
					ar[i].beginExclusive();
					user[i].status = FREE;
					user[i].authType = NONE;
					user[i].changed = true;
					ar[i].endExclusive();
				}
				else
					ar[i].endShared();
			}
		}
		//albino here
	//	ar[index].beginExclusive();
	//	if ( newStatus == FREE ) {
		//	logger () << "delete data of: "<< user[index].ip <<" - "<< user[index].mac << " from status table" << endl;
			//user[index].name = NULL;
			//user[index].email = NULL;
			//user[index].mac = "00:00:00:00:00:00";	
			//user[index].expireIP = 0;
			//user[index].expireAuth = 0;
			//user[index].dhcpRenewsAuth = false;
      //user[index].IPMode = false;
			//user[index].status = FREE;
      //user[index].token = NULL;
			
	//	}
	//	ar[index].endExclusive();



		return true;
	}
	ar[index].endShared();
	return false;
}


bool StatusTable::setURL(IPv4Address ip, const char *urlRequired)
{
	int index=ip.host();
	// The index of the user in the status table

	ar[index].beginShared();
	if ( user[index].ip == ip )
	{	
		ar[index].endShared();
		ar[index].beginExclusive();
		user[index].changed = true;
/*
		delete [] user[index].url;
		user[index].url = new char[strlen(urlRequired)+1];
		strcpy(user[index].url, urlRequired);	// Save the requested URL
*/
		user[index].url += urlRequired;
		user[index].url += '\n';
		ar[index].endExclusive();
		return true;
	}

	ar[index].endShared();
	return false;
} 


bool StatusTable::setExpirationIP(IPv4Address ip, int t)
{
	int i=ip.host();
	// The index of the user in the status table

	ar[i].beginShared();
	if(user[i].ip == ip)	
					// Check if the user identified by the IP
					// has the right MAC address.
	{
		int newt = time(NULL) + t;
		ar[i].endShared();
		//If DHCP expire ip is rewritten because he is initialized to inf for preauth
		//if ( newt > user[i].expireIP || user[i].IPMode==true  ) {
			
			ar[i].beginExclusive();
			user[i].changed = true;
			user[i].expireIP = newt;
			ar[i].endExclusive();
		//}
		if ( user[i].status == AUTHORIZED && user[i].dhcpRenewsAuth ) {
			setExpirationAuth (ip, t, true);
			logger() << "Auth updated following DHCP req for " << ip << '\n';
		}
		
		return true;
	}

	ar[i].endShared();
	return false;
}


bool StatusTable::setExpirationAuth(IPv4Address ip, int t, bool dhcpRenews /* =false */)
{
	int i=ip.host();
	// The index of the user in the status table
	ar[i].beginShared();
	if(user[i].ip == ip) {
		int newt = dhcpRenews ? (user[i].expireIP + 10) : (time(NULL) + t);
		ar[i].endShared();
		if ( newt > user[i].expireAuth ) {
			ar[i].beginExclusive();
			user[i].changed = true;
			user[i].expireAuth = newt;
			user[i].dhcpRenewsAuth = dhcpRenews;
			ar[i].endExclusive();
		}
		return true;
	}

	ar[i].endShared();
	return false;
}



IPv4Address StatusTable::getFreeIP (EthernetMACAddress &macUser)
{
	userElement *bestE = NULL;
	int bestI = -1;
	statusType bestStatus = AUTHORIZED;
	time_t bestTime = static_cast<time_t>(0x7fffffff);

	int j = rand() % MAX_USER;
	while ( 1 ) {
		for ( int k = 0; k < MAX_USER; k++ ) {
			int i = (j + k) % MAX_USER;
			userElement *e = user + i;
			if ( e->status == FORBIDDEN )
				continue;
			if ( e->mac == macUser && e->status == AUTHORIZED && e->expireAuth == static_cast<time_t>(0x7fffffff) )
				return e->ip;
			ar[i].beginShared();
			if ( e->status == FREE
					|| (e->status == FIRST_SEEN && bestStatus >= FIRST_SEEN && e->expireIP < bestTime)
					|| (e->status > FIRST_SEEN && bestStatus >= e->status && e->mac == macUser)
			) {
				bestE = e;
				bestI = i;
				bestStatus = e->status;
				bestTime = e->expireIP;
			}
			ar[i].endShared ();
		}
		if ( !bestE )
			return static_cast<uint32_t>(0);
		ar[bestI].beginExclusive ();
		if ( bestE->status == bestStatus ) {
			if ( bestE->status == FREE )
				bestE->status = FIRST_SEEN;
			bestE->changed = true;
			bestE->mac = macUser;
			bestE->expireIP = time(NULL) + 10;
			ar[bestI].endExclusive();
			return bestE->ip;
		}
		ar[bestI].endExclusive ();
	}
}		


/*
IPv4Address StatusTable::getFreeIP(EthernetMACAddress &macUser)
{
	bool flag=true;		// flag for stop the for cycle
	int i;
	int userindex=-1;	// index of the user that will be used as IP
	time_t t=time(NULL);// the current time
	time_t mintime=t;

	IPv4Address tmpIP;
	ar[i].beginShared();
	IPv4Address bestIP;
	int bestIndex = -1;
	StatusTableInterface::statusType bestStatus;
	for (i=0; i<MAX_USER && flag ; i++)
	{
		if(user[i].status!=StatusTableInterface::FORBIDDEN)
		{
//			cout<<"!= FORBIDDEN "<<i<<endl;

			if(user[i].mac==macUser || user[i].expireIP==0)
			{
				userindex=i;
				flag=false;
//				cout<<"EX MAC or never used "<<i<<endl;
			}
			else
			{
//				cout<<macUser<<" !ex mac or never used "<<endl;
				if(user[i].status != FREE)
				{	// If the IP address is NOT FREE
//					cout<<macUser<<" in if !=FREE"<<endl;
					if(user[i].expireIP < t)
					{	// If the lease time of DHCP has been expired
						// The status of IP address is set FREE.
//						cout<<macUser<<" expiredIP "<<user[i].expireIP<<endl;
						user[i].status=FREE;
						
						if(user[i].expireIP < mintime)
						{
//							cout<<macUser<<" minimun expiredIP !FREE "<<endl;
							mintime=user[i].expireIP;
							userindex=i;
						}
					}
					else
					{	
//						cout<<macUser<<" not expiredIP "<<user[i].expireIP<<endl;
						if(user[i].expireAuth < t && user[i].expireAuth!=0)
						{
							user[i].status=TEMPORARY_LEASE;
						}
					}
				}
				else
				{
//					cout<<macUser<<" FREE "<<user[i].expireIP<<endl;
					if(user[i].expireIP < mintime)
					{
//						cout<<macUser<<" minimun expiredIP FREE "<<endl;
						mintime=user[i].expireIP;
						userindex=i;
					}
				}
			} // end else (user[i].mac==macUser || user[i].expireIP==0)
		} // end if(user[i].status!=StatusTableInterface::FROBIDDEN) 
//		else
//			cout<<" FORBIDDEN "<<i<<endl;
	} // end for

	ar.endShared();

	if(userindex!=-1)
	{
		ar.beginExclusive();
		user[userindex].changed = true;
		user[userindex].mac = macUser;
		tmpIP = user[userindex].ip;
		user[userindex].status = StatusTableInterface::FIRST_SEEN;
		ar.endExclusive();
		return tmpIP; // a correct IP is found
	}
	else
	{
		// return an 0.0.0.0 IP address: it will be considered as invalid IP
		// all the IP address are user
		//MB tmpIP = IPv4Address();
		return static_cast<uint32_t>(0);
	}
}
*/




IPv4Address StatusTable::getIP(EthernetMACAddress macUser)
{
	for ( int i = 0; i < MAX_USER; i++ )
		if ( user[i].mac == macUser )
			return user[i].ip;
	return static_cast<uint32_t>(0);

/*
	int i=0;
	IPv4Address tmpIP;

	//ar.beginShared();
	for( ; i<MAX_USER && user[i].mac!=macUser ; i++);

	if(i<MAX_USER)
	{
		tmpIP=user[i].ip;
		//ar.endShared();
	}
	else
	{
		//ar.endShared();
		tmpIP=IPv4Address();
	}

	return tmpIP;
	// retrurn 0.0.0.0 if the mac is not in the Status table
*/
}


StatusTableInterface::statusType StatusTable::getStatus(IPv4Address ip)
{
	int index=ip.host();
	statusType tmpStatus=StatusTableInterface::FORBIDDEN;

	ar[index].beginShared();
	if(user[index].ip==ip)
		tmpStatus=user[index].status;
	ar[index].endShared();


	return tmpStatus;
	// return FORBIDDEN if the parameters MAC-IP don't identify a user
}

const char* StatusTable::getURL(IPv4Address ip)
{
	int i=ip.host();
	const char *tmpURL = NULL;

	ar[i].beginShared();
	if(user[i].ip == ip)
		tmpURL = user[i].url.c_str();
	ar[i].endShared();

	return tmpURL;
}


Token* StatusTable::getToken(IPv4Address ip)
{
	int i=ip.host();
	Token* tmpT;

	ar[i].beginShared();
	tmpT = (user[i].ip == ip) ? user[i].token : NULL;
	ar[i].endShared();

	return tmpT;
	// return NULL if the parameters MAC-IP don't identify a user
}


bool StatusTable::validExpirationIP(IPv4Address ip)

{
	int i=ip.host();

	ar[i].beginShared();
	if(user[i].ip==ip && time(NULL)<user[i].expireIP)
	{
		ar[i].endShared();
		return true;
	}
	
	ar[i].endShared();
	return false;
}


bool StatusTable::validExpirationAuth(IPv4Address ip)
{
	int i=ip.host();

	ar[i].beginShared();
	if(user[i].ip==ip && time(NULL)<user[i].expireAuth)
	{
		ar[i].endShared();
		return true;
	}

	ar[i].endShared();
	return false;
}


bool StatusTable::createInfo()
{
	int i=0;
	if (this->nPlugin>0)
	{
		for ( ; i<MAX_USER ; i++)
		{
			user[i].info = new PluginInfo* [nPlugin];
			for ( int j = 0; j < nPlugin; j++ )
				user[i].info[j] = NULL;
		}
		return true;
	}
	else
		return false;
}


bool StatusTable::addPluginInfo(int id, IPv4Address ip, PluginInfo* pi)
{
	int i = ip.host();
	ar[i].beginExclusive();
	user[i].info[id] = pi;
	ar[i].endExclusive();

	return true;
}



PluginInfo* StatusTable::getPluginInfo(int id, IPv4Address ip)
{
	int i = ip.host ();
	ar[i].beginShared();
	PluginInfo* pi = user[i].info[id];
	ar[i].endShared();
	return pi;
}

EthernetMACAddress *StatusTable::getMAC (IPv4Address ip)
{
	int i = ip.host ();
	ar[i].beginShared ();
	EthernetMACAddress *m = (user[i].ip == ip && user[i].status > FREE) ? &(user[i].mac) : NULL;
	ar[i].endShared ();
	return m;
}

bool StatusTable::checkIP (EthernetMACAddress &mac, IPv4Address ip)
{
	int i = ip.host ();
	ar[i].beginShared ();
	bool result = user[i].ip == ip && user[i].status > FREE && user[i].mac == mac;
	ar[i].endShared ();
	return result;
}

void StatusTable::registerListener(StatusTableListener *pointerToReport)
{
	stl = pointerToReport;
}

void StatusTable::writeRowToFile (int i)
{
	if ( persistenceFile.fail() )
		return;
	userElement *ue = user + i;
	persistenceFile.seekp (i * ROWSPACE, ios::beg);
	ue->ip.toBinaryStream (persistenceFile);
	ue->mac.toBinaryStream (persistenceFile);
	if ( ue->token )
		ue->token->toBinaryStream (persistenceFile);
	else
		Token::emptyTokenToBinaryStream (persistenceFile);
	persistenceFile.write (reinterpret_cast<const char*>(&(ue->status)), sizeof(statusType));
	persistenceFile.write (reinterpret_cast<const char*>(&(ue->expireIP)), sizeof(time_t));
	persistenceFile.write (reinterpret_cast<const char*>(&(ue->expireAuth)), sizeof(time_t));
	persistenceFile.write (reinterpret_cast<const char*>(&(ue->authType)), sizeof(authTypes));
	char buffer[BUFFERSPACE];
	if ( ue->name )
		strncpy (buffer, ue->name, BUFFERSPACE);
	else
		*buffer = '\0';
	persistenceFile.write (buffer, BUFFERSPACE);
	if ( ue->email )
		strncpy (buffer, ue->email, BUFFERSPACE);
	else
		*buffer = '\0';
	persistenceFile.write (buffer, BUFFERSPACE);

	persistenceFile.flush ();
}

bool StatusTable::timerTick (void)
{
	char macstring[40];
	time_t t = time (NULL);
	for ( int i = 0; i < MAX_USER; i++ ) {
		userElement *ue = user + i;
		ar[i].beginShared ();
		if ( ue->status > FREE )
			if ( ue->expireIP < t ) {
				ar[i].endShared ();
				logger() << "DHCP expired for " << ue->ip << " (" << ue->mac << ")\n";
				if ( dtl && ue->status < AUTHORIZED )
					dtl->sendRelease (ue->ip, ue->mac);
				logger() << "NEW STATUS is FREE for " << ue->ip << " (" << ue->mac << ")\n";
				syslog (LOG_INFO, "FREE IP %s MAC %s", inet_ntoa (ue->ip.asInaddr()), ue->mac.toString(macstring));
				setStatus (ue->ip, FREE, false);
				clearAuthType(ue->ip);
				ar[i].beginShared ();
			}
		else if ( ue->status == AUTHORIZED && ue->expireAuth < t ) {
			ar[i].endShared ();
			logger() << "Authorization expired for " << ue->ip << " (" << ue->mac << ")\n";
			logger() << "Back to TE_LE for " << ue->ip << " (" << ue->mac << ")\n";
			syslog (LOG_INFO, "ENDAUTH IP %s MAC %s", inet_ntoa (ue->ip.asInaddr()), ue->mac.toString(macstring));
			setStatus (ue->ip, TEMPORARY_LEASE, true);
			clearAuthType(ue->ip);
			ar[i].beginShared ();
		}
		if ( ue->changed ) {
			ue->changed = false;
			writeRowToFile (i);
		}
		ar[i].endShared ();
	}
	return true;
}


void StatusTable::start (void)
{
	timer.start (15);
}

void StatusTable::stop (void)
{
	timer.stop ();
}

void StatusTable::listUsers (ostream &o, statusType l)
{
	for ( int i = 0; i < MAX_USER; i++ ) {
		ar[i].beginShared ();
		if ( user[i].status >= l )
			o << user[i].ip << '\t' << user[i].mac << '\t' << user[i].status << '\n';
		ar[i].endShared ();
	}
}

void StatusTable::setUserData (IPv4Address ip, const char *name, const char *email)
{
	int i = ip.host();
	ar[i].beginShared ();
	if ( user[i].ip != ip ) {
		ar[i].endShared ();
		return;
	}
	ar[i].endShared ();
	ar[i].beginExclusive ();
	user[i].changed = true;
	delete user[i].name;
	delete user[i].email;
	user[i].name = strdup (name);
	user[i].email = strdup (email);
	ar[i].endExclusive ();
}


bool StatusTable::addAuthorization (const char *name, const char *email, IPv4Address ip, EthernetMACAddress &mac)
{
	int i = ip.host();
	
	
	///Adding a record in Authenticated Users Table
	if( (ip=="0.0.0.0") && (currAuth < MAX_USER) ) 
	{	logger() << mac << " pre-authorized MAC"<< "\n";
		authUser[currAuth].mac=mac;
		authUser[currAuth].name = strdup (name);
		authUser[currAuth].email = strdup (email);
    		authUser[currAuth++].expireAuth = static_cast<time_t>(0x7fffffff);

		return true;
	}
	//end Auth User
		

	if ( user[i].ip != ip )
		return false;
		
	user[i].name = strdup (name);
	user[i].email = strdup (email);
	user[i].mac = mac;
	user[i].expireIP = static_cast<time_t>(0x7fffffff);
	user[i].expireAuth = static_cast<time_t>(0x7fffffff);
	user[i].changed = true;
	user[i].status = AUTHORIZED;
	user[i].authType = PERSISTENT;
	
	//albino here
	//user[i].IPMode = false;
	
	logger() << mac << " fixed IP / preauth"<< "\n";




	return true;
}


//albino here
bool StatusTable::isPreAuthenticated(EthernetMACAddress &mac,IPv4Address ip) {
	//logger() << "test if preauth \n";
	int i = ip.host();
	
        ar[i].beginShared();
	/*if(user[i].mac == mac && user[i].name != NULL) { 
     		//logger() << mac << "is already in ST \n"; 
		ar[i].endShared();	
		return true;
	}
  */
	//logger() << "not in ST \n";
	int j=0;
	for (j=0; j<currAuth;j++) {
		if( (authUser[j].mac == mac) /*&& (user[i].name == NULL)*/ )
		{	
			user[ip.host()].ip=ip;	
			user[ip.host()].mac=authUser[j].mac;
			user[ip.host()].name=authUser[j].name;	
			user[ip.host()].email=authUser[j].email;
			user[ip.host()].expireAuth=static_cast<time_t>(0x7fffffff);
      			//after will be overwritten from dhcp plugins (for DHCP clients or static not pre-auth)
			user[ip.host()].expireIP=static_cast<time_t>(0x7fffffff);
			//logger() <<"EXP TIME "<<user[ip.host()].expireAuth<<"\n";
			logger() << "Adding info of Pre-authenticated: " << user[ip.host()].name << "\n" ;
			ar[i].endShared();
			return true;
		}

	}
	ar[i].endShared();
	//logger() << "Is not a preauth \n";
	return false;	

}
//end albino here


bool StatusTable::getUserData (IPv4Address ip, char *name, char *email, time_t *expIP, time_t *expAuth, statusType *status, int bufsize)
{
	int i = ip.host();
	ar[i].beginShared ();
	if ( user[i].ip != ip ) {
		ar[i].endShared ();
		return false;
	}
	if ( user[i].name )
		strncpy (name, user[i].name, bufsize);
	else
		*name = '\0';
	if ( user[i].email )
		strncpy (email, user[i].email, bufsize);
	else
		*email = '\0';
	*expIP = user[i].expireIP;
	*expAuth = user[i].expireAuth;
	*status = user[i].status;
	ar[i].endShared ();
	return true;
}


void StatusTable::registerDHCPService (StatusTableListener *p)
{
  dtl = p;
}

//albino here
void StatusTable::associateMACtoIP(EthernetMACAddress macUser, IPv4Address ipUser)

{
	//	int i = ipUser.host();
		//ar[i].beginShared ();
                user[ipUser.host()].mac=macUser;
		//ar[i].endShared ();
}

//albino here
void StatusTable::setIPMode(IPv4Address ipUser,bool ipMode)
{
    user[ipUser.host()].IPMode=ipMode;
}

/*
void StatusTable::setPresenceCheck(IPv4Address ipUser,time_t nextCheck) {

    user[ipUser.host()].presenceCheck=nextCheck;
}
*/

//######################## Functions added for SIP ##############################
bool StatusTable::addAuthType(IPv4Address ip, authTypes auth)
{
	int index=ip.host();
	// The index of the user in the status table

	ar[index].beginShared();
	if ( user[index].ip == ip )
	{	
		ar[index].endShared();
		ar[index].beginExclusive();
		user[index].changed = true;
		logger() << "Adding Authorization-Type for user " << user[ip.host()].name << ": " << (int)auth << endl ;
		user[index].authType  = authTypes((int) user[index].authType | (int)auth);
		ar[index].endExclusive();
		return true;
	}
	ar[index].endShared();
	return false;
}

bool StatusTable::removeAuthType(IPv4Address ip, authTypes auth)
{	
	char macstring[40];
	int index=ip.host();
	// The index of the user in the status table

	ar[index].beginShared();
	if ( user[index].ip == ip )
	{	
		ar[index].endShared();
		ar[index].beginExclusive();
		user[index].changed = true;
		logger() << "[" << ip <<  ", "<< user[ip.host()].name << ", " << user[ip.host()].email << \
		"]: remove authorization " << (int)auth << endl ;
		syslog (LOG_INFO, "ENDAUTH IP %s MAC %s", inet_ntoa(ip.asInaddr()), user[index].mac.toString(macstring));
		//if authentication we want to remove is present
		if((user[index].authType  & auth)>0)
			user[index].authType = authTypes((int)user[index].authType - (int)(auth));
		ar[index].endExclusive();
		return true;
	}
	ar[index].endShared();
	return false;
}

bool StatusTable::clearAuthType(IPv4Address ip)
{
	char macstring[40];
	int index=ip.host();
	// The index of the user in the status table

	ar[index].beginShared();
	if ( user[index].ip == ip )
	{	
		ar[index].endShared();
		ar[index].beginExclusive();
		user[index].changed = true;
		logger() << "[" << ip <<  ", "<< user[ip.host()].name << ", " << user[ip.host()].email << \
		"]: remove all authorizations" << endl ;
		syslog (LOG_INFO, "ENDAUTH IP %s MAC %s", inet_ntoa(ip.asInaddr()), user[index].mac.toString(macstring));
		user[index].authType = NONE;
		ar[index].endExclusive();
		return true;
	}
	ar[index].endShared();
	return false;
}

bool StatusTable::isHTTPauthenticated(IPv4Address ip)
{
	int index=ip.host();
	int status=0;

	ar[index].beginShared();
	if(user[index].ip==ip)
		status=user[index].authType;
	ar[index].endShared();
	status = status & StatusTableInterface::HTTP; // put all non flags to 0 except HTTP one
	if (status)	// HTTP authentication present
		return true;
	else
		return false;
}

bool StatusTable::isSIPauthenticated(IPv4Address ip)
{
	int index=ip.host();
	int status=0;

	ar[index].beginShared();
	if(user[index].ip==ip)
		status=user[index].authType;
	ar[index].endShared();
	status = status & StatusTableInterface::SIP; // put all non flags to 0 except SIP one
	if (status)	// SIP authentication present
		return true;
	else
		return false;
}

bool StatusTable::isAuthenticated(IPv4Address ip)
{
	int index=ip.host();
	int status=0;

	ar[index].beginShared();
	if(user[index].ip==ip)
		status=user[index].authType;
	ar[index].endShared();
	logger()<< "isAuthenticated called; user authType: " << status << endl;
	if (status)	// user has some authentication still active
		return true;
	else
		return false;
}
bool StatusTable::authorizeUserFor(IPv4Address ip,authTypes authType,const char *name, const char *email)
{
	char macstring[40];
	int index=ip.host();
	int auth_gained;
	ar[index].beginShared();
	if ( user[index].ip == ip )
	{	
		ar[index].endShared();
		if(user[index].ip==ip){
			ar[index].beginExclusive();
				//if user has no active authentication (since we allow only one 
				//type of authorization at a time)
				logger() << "[" << ip << ", " << name << ", " << email \
				<<"]: Authorizationrequest for " << (int)authType << endl;
				logger() << "[" << ip << ", " << name << ", " << email \
				<<"]: Current authorization is: " << user[index].authType << endl;
				if(user[index].authType == NONE){
				if ( stl )
					stl->statusChanged (ip, user[index].mac, user[index].status, AUTHORIZED);
					auth_gained=1;
					user[index].changed = true;
					user[index].status = AUTHORIZED;
					delete user[index].name;
					delete user[index].email;
					delete user[index].token;
					user[index].name = strdup (name);
					user[index].email = strdup (email);
					user[index].token = new Token (16);
					user[index].authType=authType;
				}else{
					auth_gained=0;
					
				}
			ar[index].endExclusive();
			if(auth_gained){
				logger() << "[" << ip << ", " << user[index].name << ", " << user[index].email\
				<<"]: " << (int)authType << " authorization received" << endl ;
				syslog (LOG_INFO, "AUTH IP %s MAC %s NAME %s EMAIL %s",  inet_ntoa(ip.asInaddr()),
				            user[index].mac.toString(macstring), user[index].name, user[index].email);

			}else{
				logger() << "[" << ip << ", " << user[index].name << ", " << user[index].email\
				<<"]: already authorized (renew)" << endl;
			}
			EthernetMACAddress mac = user[index].mac;
			for ( int i = 0; i < MAX_USER; i++ ) {
				if ( i == index )
					continue;
				ar[i].beginShared();
				if ( user[i].mac == mac ) {
					ar[i].endShared();
					ar[i].beginExclusive();
					user[i].status = FREE;
					user[i].changed = true;
					user[index].authType=NONE;
					ar[i].endExclusive();
				}
				else
					ar[i].endShared();
			}
			return true;
		}
	}
	ar[index].endShared();
	return false;
}
//######################## End functions added for SIP ##############################

}
