///
/// \file statustable_listener.h
///	\author Danilo Severina
///
///	The file contains the definition of the interface between the StatusTable and the Report. 
///	The Report class derives from this class and implements the the pure virtual functions
///	that are defined here. The class is used by the Status Table to notify changes in the 
///	Status Table configuration to the Report (that will notify to the Gateway the new 
///	configuration of the IP addresses that are allowed to browse).
///
///	$Id: statustable_listener.h,v 1.3 2005/12/13 17:18:53 cvs Exp $
///

/*##############################################################################
   
   Developed by NetMob Group, University of Trento, Italy

   University of Trento
   DIT - Department of Informatics and Telecommunications 
   Computer Networks and Mobility Research Program
   http://netmob.unitn.it/ 
   
   This file is part of the WilmaGate system, created as part
   of the WILMA Project, sponsored by the Autonomous
   Province of Trento and participated by ITC-irst, the University
   of Trento and Alpikom Spa. More info at

     http://www.wilmaproject.org/		

   To download the project:
      - in tarball format
        http://netmob.unitn.it/wilmagate.html
      - connect to CVS repository
        :pserver:cvs@rtm.unitn.it:/home/cvsroot
   In both previous cases, the download is password protected. To obtain
   the password you must send a mail to 

      wilmagate@dit.unitn.it

################################################################################

   The WilmaGate Licence of Use


   Copyright (c) 2003-2005
                - Universita` di Trento,
                  Dipartimento di Informatica e Telecomunicazioni (DIT)
                  Computer Networks and Mobility Research Program


   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (hereinafter
   the "WilmaGate") directly from the copyright owner, to use the WilmaGate
   without restriction of purpose or time, including without limitation the
   rights to use, copy, modify, and merge the WilmaGate, subject to the
   following conditions:

   The above copyright notice and this permission notice shall be included
   in all copies or substantial portions of the WilmaGate. 

   The WilmaGate shall not be sub-licenced either commercially or free of
   charge, as is or in part or as a part of any other package. 

   The licensee agrees to properly credit WilmaGate and its copyright 
   owners in any technical, scientific or divulgative publication, 
   whatever publishing means are used, that is based entirely or in part 
   on WilmaGate use, as well as in any booklet, web-site or brochure that 
   advertise services based on WilmaGate use. 

   If the WilmaGate is modified, the copyright owner shall be notified.

   THE WilmaGate IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

##############################################################################*/

#ifndef _STATUSTABLE_LISTENER_
#define _STATUSTABLE_LISTENER_

#include <iostream>
#include "../Common/addresses.h"

using namespace std;
using namespace Common;

namespace Gatekeeper {

///
///	The file contains the definition of the interface between the StatusTable and the Report. 
///	The Report class derives from this class and implements the the pure virtual functions
///	that are defined here. The class is used by the Status Table to notify changes in the 
///	Status Table configuration to the Report (that will notify to the Gateway the new 
///	configuration of the IP addresses that are allowed to browse).

class StatusTableListener{
private:

public:

		///
		/// \brief Virtual destructor.
		///
		virtual ~StatusTableListener (void) {}

	///
	/// The function is used by the Status Table to notify to the Report
	///	that the status of some Ip addresses is changed.
	///	The function is defined as pure virtual function and it is implemented in the 
	///	Report class.
	///
	virtual void statusChanged (IPv4Address ip, EthernetMACAddress &mac, StatusTableInterface::statusType oldStatus,
					StatusTableInterface::statusType newStatus) = 0;
	//For DHCP relay plugin
	virtual void sendRelease (IPv4Address ip, EthernetMACAddress &mac) = 0;				

};

}

#endif

