///
/// \file token.h
///	\author Danilo Severina
///
///	The class Token contains teh information about the token that
///	identifies a user. The token is a random sequence of alpha-numeric
///	characters with a choosen length. The length can be chosen when a
///	new token is created.
///
///	$Id: token.h,v 1.3 2005/06/16 13:22:22 cvs Exp $
///

/*##############################################################################
   
   Developed by NetMob Group, University of Trento, Italy

   University of Trento
   DIT - Department of Informatics and Telecommunications 
   Computer Networks and Mobility Research Program
   http://netmob.unitn.it/ 
   
   This file is part of the WilmaGate system, created as part
   of the WILMA Project, sponsored by the Autonomous
   Province of Trento and participated by ITC-irst, the University
   of Trento and Alpikom Spa. More info at

     http://www.wilmaproject.org/		

   To download the project:
      - in tarball format
        http://netmob.unitn.it/wilmagate.html
      - connect to CVS repository
        :pserver:cvs@rtm.unitn.it:/home/cvsroot
   In both previous cases, the download is password protected. To obtain
   the password you must send a mail to 

      wilmagate@dit.unitn.it

################################################################################

   The WilmaGate Licence of Use


   Copyright (c) 2003-2005
                - Universita` di Trento,
                  Dipartimento di Informatica e Telecomunicazioni (DIT)
                  Computer Networks and Mobility Research Program


   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (hereinafter
   the "WilmaGate") directly from the copyright owner, to use the WilmaGate
   without restriction of purpose or time, including without limitation the
   rights to use, copy, modify, and merge the WilmaGate, subject to the
   following conditions:

   The above copyright notice and this permission notice shall be included
   in all copies or substantial portions of the WilmaGate. 

   The WilmaGate shall not be sub-licenced either commercially or free of
   charge, as is or in part or as a part of any other package. 

   The licensee agrees to properly credit WilmaGate and its copyright 
   owners in any technical, scientific or divulgative publication, 
   whatever publishing means are used, that is based entirely or in part 
   on WilmaGate use, as well as in any booklet, web-site or brochure that 
   advertise services based on WilmaGate use. 

   If the WilmaGate is modified, the copyright owner shall be notified.

   THE WilmaGate IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

##############################################################################*/

#ifndef _TOKEN_
#define _TOKEN_


#include <iostream>
#include <cstdlib>

using namespace std;

namespace Gatekeeper {

/// 
///	The class Token contains teh information about the token that
///	identifies a user. The token is a random sequence of alpha-numeric
///	characters with a choosen length. The length can be chosen when a
///	new token is created.
///

class Token{
private:

	///
	///	The function generate a new token object 
	///	in both cases the functions return a pointer to a character array that
	///	identifies the token.
	///
	static char* createToken (int lengthOfToken);
	static char* createToken (Token TokenObject);
	Token (int, const char*);

public:

	///
	///	The token elements of the Token class is a pointer to an array of characters
	///	and identifies a sequence of alpha-numeric characters that are the token.
	///	The element is defined as \e public and \e constan so it can be modified only by the
	///	constructor of the class and other class can access to it but cannot change 
	///	the information that is storaged.
	///
	const char* const token;

	///
	///	The legth element of the class Token defines the length of the token.
	///	To not lose generality, you supposed that tha length of tha token can 
	///	change for one to another.
	///	The element is defined as \e public and \e constant so it can be modified only by the
	///	constructor of the class and other class can access to it but cannot change 
	///	the information that is storaged.
	///
	const int length;

	Token();
	Token (int legthOfToken);
	Token (Token & TokenObject);
	
	~Token();

	inline bool operator== (Token &t);
	inline bool operator== (const char *t);

	ostream &toBinaryStream (ostream &s) const;
	static ostream &emptyTokenToBinaryStream (ostream &s);
	static Token *fromBinaryStream (istream &s);

	friend ostream &operator<< (ostream &o, const Token &t);
};

inline bool Token::operator== (Token &t)
{
	return token && t.token && !strcmp (token, t.token);
}

inline bool Token::operator== (const char *t)
{
	return token && !strcmp (token, t);
}

}

#endif

