/*##############################################################################
   
   Developed by NetMob Group, University of Trento, Italy

   University of Trento
   DIT - Department of Informatics and Telebcommunications 
   Computer Networks and Mobility Research Program
   http://netmob.unitn.it/ 
   
   This file is part of the WilmaGate system, created as part
   of the WILMA Project, sponsored by the Autonomous
   Province of Trento and participated by ITC-irst, the University
   of Trento and Alpikom Spa. More info at

     http://www.wilmaproject.org/		

   To download the project:
      - in tarball format
        http://netmob.unitn.it/wilmagate.html
      - connect to CVS repository
        :pserver:cvs@rtm.unitn.it:/home/cvsroot
   In both previous cases, the download is password protected. To obtain
   the password you must send a mail to 

      wilmagate@dit.unitn.it

################################################################################

   The WilmaGate Licence of Use


   Copyright (c) 2003-2005
                - Universita` di Trento,
                  Dipartimento di Informatica e Telecomunicazioni (DIT)
                  Computer Networks and Mobility Research Program


   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (hereinafter
   the "WilmaGate") directly from the copyright owner, to use the WilmaGate
   without restriction of purpose or time, including without limitation the
   rights to use, copy, modify, and merge the WilmaGate, subject to the
   following conditions:

   The above copyright notice and this permission notice shall be included
   in all copies or substantial portions of the WilmaGate. 

   The WilmaGate shall not be sub-licenced either commercially or free of
   charge, as is or in part or as a part of any other package. 

   The licensee agrees to properly credit WilmaGate and its copyright 
   owners in any technical, scientific or divulgative publication, 
   whatever publishing means are used, that is based entirely or in part 
   on WilmaGate use, as well as in any booklet, web-site or brochure that 
   advertise services based on WilmaGate use. 

   If the WilmaGate is modified, the copyright owner shall be notified.

   THE WilmaGate IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

##############################################################################*/

#include "wtp_table.h"


using namespace std;
using namespace Common;

namespace Gatekeeper {

WTPTable::WTPTable(string newpmtfile)
{
	cpi=NULL;
	if( ReadPMT(newpmtfile) )
	{
		this->nPlugin=0;
		// Register this object to the timer
		timer.registerListener (this);
		timerTick ();
	}
	else
	{
		logger()<<"[CAPWAP] Error when read "<<newpmtfile<<endl;
	}
}

WTPTable::~WTPTable()
{
	 wtps.erase( wtps.begin(), wtps.end() );
}

bool WTPTable::ReadPMT(string newpmtfile)
{
	string newacname,newwtpessid;
	IPv4Address newwtpnetmask, newwtpgateway;
	int newin_udp, newout_udp;
	
	string newwtpname, newwtplocation;
	IPv4Address newwtpip;
	double newwtpchannel, newwtptxp;
	logger()<<"[CAPWAP] Load CAPWAP parameters from "<<newpmtfile<<endl;
	
	// Dirty code...
	char** dc=new char* [2];
	dc[0]=NULL;
	dc[1]=new char [strlen(newpmtfile.c_str())+1];
	strncpy(dc[1],newpmtfile.c_str(),strlen(newpmtfile.c_str())+1);
	Parameters cwpmt (2,dc);
	
	cwpmt >> newacname >> newwtpessid >> newwtpnetmask >> newwtpgateway >> newout_udp >> newin_udp;
	/*
	logger()<<"[CAPWAP]ESSID: "<<newwtpessid<<endl<<"Netmask: "<<newwtpnetmask<<endl<<"Gateway: "<<newwtpgateway<<endl;
	logger()<<"[CAPWAP]Out UDP: "<<newout_udp<<"In UDP: "<<newin_udp<<endl;
	*/
	for ( ; ; ) 
	{
		cwpmt >> newwtpname;
		if ( newwtpname.length() == 0 )
			break;
		cwpmt >> newwtplocation >> newwtpip >> newwtpchannel >> newwtptxp;
		/*logger() <<endl<< "Name: " << newwtpname <<endl
		<< "Location: " <<newwtplocation<<endl
		<< "IP: " <<newwtpip<<endl
		<< "Channel: " <<newwtpchannel<<endl
		<< "Trasmission Power: " <<newwtptxp<<endl;
		*/
		AddPreConfigurationInformation(newwtpip,newwtpname, newwtplocation, uint8_t(newwtpchannel), uint32_t(newwtptxp));
	}
	acname=newacname;
	wtpessid=newwtpessid;
	wtpnetmask=newwtpnetmask;
	wtpgateway=newwtpgateway;
	pmtfile=newpmtfile;
	outudp=newout_udp;
	inudp=newin_udp;
	configurationupdate=false;
	return true;
}

bool WTPTable::SetStatus(IPv4Address wtpip, StatusType newstatus)
{
	for(unsigned int i=0;i<(wtps.size());i++)
	{
		if(wtps[i].ip==wtpip)
		{
			ar[i].beginExclusive();
			wtps[i].status=newstatus;
			ar[i].endExclusive();
			return true;	
		}
			
	}
	return false;
}
bool WTPTable::SetPreConfigure(IPv4Address wtpip,StatusConfiguration newvalue)
{
	for(unsigned int i=0;i<(wtps.size());i++)
	{
		if(wtps[i].ip==wtpip)
		{
			//logger()<<"[CAPWAP]Value: "<<(int)Value<<endl;
			ar[i].beginExclusive();
			wtps[i].preconfigure=newvalue;
			ar[i].endExclusive();
			return true;	
		}
			
	}
	return false;
}

bool WTPTable::AddJoinRequestInformation(IPv4Address ip, EthernetMACAddress mac, uint32_t newsid)
{
	int i=0;
	vector<WTPElement>::iterator index;
	if(wtps.size()==0) //first element
	{
		ar[0].beginExclusive();
		wtps.insert(wtps.begin(),WTPElement());
		wtps[0].ip=ip;
		wtps[0].mac=mac;
		wtps[0].sid=newsid;
		wtps[0].status=WTPTable::JOIN;
		wtps[0].preconfigure=WTPTableInterface::NOPRECONFIGURATION; 
		ar[0].endExclusive();
		return true;
	}
	for( index = wtps.begin(); index < wtps.end(); index++,i++ ) //Search correct position
	{
		if(((wtps[i].ip).asInteger()) == (ip.asInteger()))
		{
			//it has preconfiguration
			ar[i].beginExclusive();
			wtps[i].ip=ip;
			wtps[i].mac=mac;
			wtps[i].sid=newsid;
			wtps[i].status=WTPTable::JOIN;
			ar[i].endExclusive();
			return true;
		}
		
		if( ((wtps[i].ip).asInteger()) > (ip.asInteger()) ) 
		{
			ar[i].beginExclusive();
			wtps.insert(index,WTPElement());
			wtps[i].ip=ip;
			wtps[i].mac=mac;
			wtps[i].sid=newsid;
			wtps[i].status=WTPTable::JOIN;
			wtps[i].preconfigure=WTPTableInterface::NOPRECONFIGURATION;
			ar[i].endExclusive();
			return true;
		}
		if((index+1)==wtps.end())
		{
			ar[i+1].beginExclusive();
			wtps.push_back(WTPElement());
			wtps[i+1].ip=ip;
			wtps[i+1].mac=mac;
			wtps[i+1].sid=newsid;
			wtps[i+1].status=WTPTable::JOIN;
			wtps[i+1].preconfigure=WTPTableInterface::NOPRECONFIGURATION;
			ar[i+1].endExclusive();
			return true;
		}
	}
	return false;
}
	
bool WTPTable::AddConfigurationStatusRequestInformation(
	IPv4Address newip,
	string newwtpname,
	string newwtplocation,
	uint8_t newrid,
	uint8_t newchannel,
	uint32_t newtxpower,
	string newessid)
{
	for(unsigned int i=0;i<(wtps.size());i++)
	{
		if(wtps[i].ip==newip)
		{
			ar[i].beginExclusive();
			wtps[i].name=newwtpname;
			wtps[i].location=newwtplocation;
			wtps[i].channel=newchannel;
			wtps[i].txpower=newtxpower;
			ar[i].endExclusive();
			return true;	
		}
			
	}
	return false;	
}

bool WTPTable::AddPreConfigurationInformation(
	IPv4Address newip, 
	string newwtpname, 
	string newwtplocation, 
	uint8_t newchannel,
	uint32_t newtxpower)
{
	int i=0;
	vector<WTPElement>::iterator index = wtps.begin();
	if(wtps.size()==0) //first element
	{
		ar[0].beginExclusive();
		wtps.insert(wtps.begin(),WTPElement());
		wtps[0].ip=newip;
		wtps[0].name=newwtpname;
		wtps[0].location=newwtplocation;
		wtps[0].channel=newchannel;
		wtps[0].txpower=newtxpower;
		wtps[0].preconfigure=WTPTableInterface::NOACTIVEPRECONFIGURATION;
		ar[0].endExclusive();
		return true;
	}
	for( index = wtps.begin(); index < wtps.end(); index++,i++ ) //Search correct position
	{
		if( ((wtps[i].ip).asInteger()) == (newip.asInteger()) )
		{
			ar[i].beginExclusive();
			wtps[i].ip=newip;
			wtps[i].name=newwtpname;
			wtps[i].location=newwtplocation;
			wtps[i].channel=newchannel;
			wtps[i].txpower=newtxpower;
			if(wtps[i].status==WTPTableInterface::RUN)
			{
				wtps[i].preconfigure=WTPTableInterface::PENDINGPRECONFIGURATION;
			}
			else
			{
				wtps[i].preconfigure=WTPTableInterface::NOACTIVEPRECONFIGURATION;
			}
			ar[i].endExclusive();
			return true;
		}
		
		if( ((wtps[i].ip).asInteger()) > (newip.asInteger()) ) 
		{
			ar[i].beginExclusive();
			wtps.insert(index,WTPElement());
			wtps[i].ip=newip;
			wtps[i].name=newwtpname;
			wtps[i].location=newwtplocation;
			wtps[i].channel=newchannel;
			wtps[i].txpower=newtxpower;
			wtps[i].preconfigure=WTPTableInterface::NOACTIVEPRECONFIGURATION;
			ar[i].endExclusive();
			return true;
		}
		
		if((index+1)==wtps.end())
		{
			ar[i+1].beginExclusive();
			wtps.push_back(WTPElement());
			wtps[i+1].ip=newip;
			wtps[i+1].name=newwtpname;
			wtps[i+1].location=newwtplocation;
			wtps[i+1].channel=newchannel;
			wtps[i+1].txpower=newtxpower;
			wtps[i+1].preconfigure=WTPTableInterface::NOACTIVEPRECONFIGURATION;
			ar[i+1].endExclusive();
			return true;
		}
		
	}
	return false;
}

IPv4Address WTPTable::GetIP(EthernetMACAddress WTPMAC)
{
	IPv4Address BUFIP=IPv4Address();
	for(unsigned int i=0;i<(wtps.size());i++)
	{
		if(wtps[i].mac==WTPMAC)
		{
			ar[i].beginShared();
			BUFIP=wtps[i].ip;
			ar[i].endShared();
		}
		
	}
	return (BUFIP);
}

uint32_t WTPTable::GetSID(IPv4Address WTPIP)
{
	uint32_t BUFSID=0;
	for(unsigned int i=0;i<(wtps.size());i++)
	{
		if(wtps[i].ip==WTPIP)
		{
			ar[i].beginShared();
			BUFSID=wtps[i].sid;
			ar[i].endShared();
		}
	}
	return (BUFSID);
}

EthernetMACAddress WTPTable::GetMAC(IPv4Address WTPIP)
{
	EthernetMACAddress BUFMAC=EthernetMACAddress();
	for(unsigned int i=0;i<(wtps.size());i++)
	{
		if(wtps[i].ip==WTPIP)
		{
			ar[i].beginShared();
			BUFMAC=wtps[i].mac;
			ar[i].endShared();
		}
	}
	return (BUFMAC);
}

WTPTableInterface::StatusType WTPTable::GetStatus(IPv4Address WTPIP)
{
	StatusType BUFStatus=WTPTable::IDLE;	
	for(unsigned int i=0;i<(wtps.size());i++)
	{
		if(wtps[i].ip==WTPIP)
		{
			ar[i].beginShared();
			BUFStatus=wtps[i].status;
			ar[i].endShared();
		}
	}
	return (BUFStatus);
}

string WTPTable::GetLocation(IPv4Address WTPIP)
{
	string BUFLocation="None";	
	for(unsigned int i=0;i<(wtps.size());i++)
	{
		if(wtps[i].ip==WTPIP)
		{
			ar[i].beginShared();
			BUFLocation=wtps[i].location;
			ar[i].endShared();
		}
	}
	return (BUFLocation);
}

string WTPTable::GetWTPName(IPv4Address WTPIP)
{
	string BUFName="None";	
	for(unsigned int i=0;i<(wtps.size());i++)
	{
		if(wtps[i].ip==WTPIP)
		{
			ar[i].beginShared();
			BUFName=wtps[i].name;
			ar[i].endShared();
		}
	}
	return (BUFName);
}

uint8_t WTPTable::GetRadioID(IPv4Address WTPIP)
{
	uint8_t BUFRadioID=0;	
	for(unsigned int i=0;i<(wtps.size());i++)
	{
		if(wtps[i].ip==WTPIP)
		{
			ar[i].beginShared();
			BUFRadioID=wtps[i].rid;
			ar[i].endShared();
		}
	}
	return (BUFRadioID);
}

uint8_t WTPTable::GetChannel(IPv4Address WTPIP)
{
	uint8_t BUFCurrentChannel=0;	
	for(unsigned int i=0;i<(wtps.size());i++)
	{
		if(wtps[i].ip==WTPIP)
		{
			ar[i].beginShared();
			BUFCurrentChannel=wtps[i].channel;
			ar[i].endShared();
		}
	}
	return (BUFCurrentChannel);
}

uint32_t WTPTable::GetTxPower(IPv4Address WTPIP)
{
	uint8_t BUFTxPower=0;	
	for(unsigned int i=0;i<(wtps.size());i++)
	{
		if(wtps[i].ip==WTPIP)
		{
			ar[i].beginShared();
			BUFTxPower=wtps[i].txpower;
			ar[i].endShared();
		}
	}
	return (BUFTxPower);
}

WTPTableInterface::StatusConfiguration WTPTable::GetPreConfigure(IPv4Address WTPIP)
{
	StatusConfiguration BUFPreConfigure;	
	for(unsigned int i=0;i<(wtps.size());i++)
	{
		if(wtps[i].ip==WTPIP)
		{
			ar[i].beginShared();
			BUFPreConfigure=wtps[i].preconfigure;
			ar[i].endShared();
		}
	}
	return (BUFPreConfigure);
}

bool WTPTable::DelWTP(IPv4Address WTPIP)
{
	int i=0;
	for( vector<WTPElement>::iterator id = wtps.begin(); id < wtps.end(); id++,i++)
	{
		if((wtps[i].ip) == (WTPIP))
		{
			ar[i].beginShared();
			wtps.erase(id);
			ar[i].beginShared();
			return true;
		}
	}
	return false;
}

bool WTPTable::RefreshWTP(IPv4Address WTPIP)
{
	int i=0;
	time_t now = time (NULL);
	
	for( vector<WTPElement>::iterator id = wtps.begin(); id < wtps.end(); id++,i++)
	{
		if((wtps[i].ip) == (WTPIP))
		{
			ar[i].beginExclusive();
			wtps[i].fresh=now+300; //5 minutes
			ar[i].endExclusive();
			return true;
		}
	}
	return false;
}

bool WTPTable::ConfigurationUpdate()
{
	configurationupdate=true;
	return true;
}

bool WTPTable::Register_capwap_plugin_B(CapwapPluginB *newpointer)
{
	cpi=newpointer;
	return true;
}

int WTPTable::Getoutudpport()
{
	return(outudp);
}

int WTPTable::Getinudpport()
{
	return(inudp);
}

string WTPTable::GetACName()
{
	return(acname);
}

string WTPTable::GetWTPESSID()
{
	return(wtpessid);
}
IPv4Address WTPTable::GetWTPNetmask()
{
	return(wtpnetmask);
}
IPv4Address WTPTable::GetWTPGateway()
{
	return(wtpgateway);
}

int WTPTable::GetNumberWTP(void)
{
	int numwtp=0;
	numwtp=wtps.size();
	return (numwtp);
}

void WTPTable::start (void)
{
	timer.start (15);
}

void WTPTable::stop (void)
{
	timer.stop ();
}

bool WTPTable::timerTick (void)
{
	time_t t = time (NULL);
	int i = 0;
	vector<WTPElement>::iterator index;
	
	for( index = wtps.begin(); index < wtps.end(); index++,i++ )
	{
		if ( wtps[i].status==WTPTable::RUN ) 
		{
			if(wtps[i].fresh<t)
			{
				if ( wtps[i].preconfigure==WTPTableInterface::NOPRECONFIGURATION )
				{
					ar[i].beginExclusive();
					wtps.erase(index);
					ar[i].endExclusive();
				}
				else
				{
					ar[i].beginExclusive();
					wtps[i].preconfigure=WTPTableInterface::NOACTIVEPRECONFIGURATION;
					wtps[i].status=WTPTable::IDLE;
					ar[i].endExclusive();
				}
				logger()<<"[CAPWAP] WTP: "<<wtps[i].ip<<" is expired!"<<endl;
			}	
		}
	}
	if(configurationupdate) //re-read of pmt file is required
	{
	logger()<<"[CAPWAP] WTP Table re-read file in order to update configuration of WTP."<<endl;
		ReadPMT(pmtfile);
		for( index = wtps.begin(),i = 0; index < wtps.end(); index++,i++ )
		{
		//logger()<<endl<<"Analize wtp: "<<wtps[i].ip<<endl
		//<<" status: "<<wtps[i].status<<endl
		//<<" preconfigure: "<<wtps[i].preconfigure<<endl<<endl;
			if ( wtps[i].preconfigure==WTPTableInterface::PENDINGPRECONFIGURATION )
			{
				//Send Configuration Update Request, submit to capwap plugin B
				cpi->SendCONFIGURATIONUPDATEREQUEST(wtps[i].ip);
			}

		}
		
	}
	return true;
}

}
