///
/// \file wtptable_interface.h
///	\Nicolò Pernigo
///
///	The file contains the definition of functions that work on
///	WTP Table object and that are common to interfaces towards 
/// plugin sub-classes and authentication class.
///
///	$Id: statustable_interface.h,v 1.3 2005/06/16 13:22:22 cvs Exp $
///
/*##############################################################################
   
   Developed by NetMob Group, University of Trento, Italy

   University of Trento
   DIT - Department of Informatics and Telecommunications 
   Computer Networks and Mobility Research Program
   http://netmob.unitn.it/ 
   
   This file is part of the WilmaGate system, created as part
   of the WILMA Project, sponsored by the Autonomous
   Province of Trento and participated by ITC-irst, the University
   of Trento and Alpikom Spa. More info at

     http://www.wilmaproject.org/		

   To download the project:
      - in tarball format
        http://netmob.unitn.it/wilmagate.html
      - connect to CVS repository
        :pserver:cvs@rtm.unitn.it:/home/cvsroot
   In both previous cases, the download is password protected. To obtain
   the password you must send a mail to 

      wilmagate@dit.unitn.it

################################################################################
pwtpti.figure(WTPIP)==2;
   The WilmaGate Licence of Use


   Copyright (c) 2003-2005
                - Universita` di Trento,
                  Dipartimento di Informatica e Telecomunicazioni (DIT)
                  Computer Networks and Mobility Research Program


   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (hereinafter
   the "WilmaGate") directly from the copyright owner, to use the WilmaGate
   without restriction of purpose or time, including without limitation the
   rights to use, copy, modify, and merge the WilmaGate, subject to the
   following conditions:

   The above copyright notice and this permission notice shall be included
   in all copies or substantial portions of the WilmaGate. 

   The WilmaGate shall not be sub-licenced either commercially or free of
   charge, as is or in part or as a part of any other package. 

   The licensee agrees to properly credit WilmaGate and its copyright 
   owners in any technical, scientific or divulgative publication, 
   whatever publishing means are used, that is based entirely or in part 
   on WilmaGate use, as well as in any booklet, web-site or brochure that 
   advertise services based on WilmaGate use. 

   If the WilmaGate is modified, the copyright owner shall be notified.

   THE WilmaGate IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

##############################################################################*/

#ifndef _WTPTABLE_INTERFACE_
#define _WTPTABLE_INTERFACE_


#include <iostream>
#include "../Common/addresses.h"

using namespace std;
using namespace Common;


namespace Gatekeeper {

///
///	The class contains the definition of functions that work on
/// Status Table object and that are common to interfaces towards 
/// plugin sub-classes and authentication class.
///	The classes PluginStatusTableInterface and AuthStatusTableInterface
///	derive from this class: they don't implement the virtual functions that
///	are defined in this class, because these virtual functions are implemented
///	by StatusTable class.
///

class WTPTableInterface{

private:
		

public:

	///
	/// The status of the WTP can be:
	///		- IDLE
	///		- DISCOVERY
	///		- SULKING
	///		- CONFIGURE
	///		- RUN
	///		- RESET
	
	enum StatusType {
	IDLE,
	DISCOVERY,
	JOIN,
	SULKING,
	CONFIGURE,
	RUN,
	RESET};

	enum StatusConfiguration{
	NOPRECONFIGURATION,
	NOACTIVEPRECONFIGURATION,
	ACTIVEPRECONFIGURATION,
	ERRORPRECONFIGURATION,
	PENDINGPRECONFIGURATION};

	///
	/// The function sets all the information derived to Join Request
	///	\param ...
	///	\return \e true if the procedure is correct, \e falses otherwise.
	///
	virtual bool AddJoinRequestInformation(IPv4Address ip, 
	EthernetMACAddress mac, 
	uint32_t newsid) = 0;
	
	///
	/// The function sets all the information derived to Configuration Status Request.
	///	\param ...
	///	\return \e true if the procedure is correct, \e falses otherwise.
	///
	virtual bool AddConfigurationStatusRequestInformation(
	IPv4Address newip,
	string newwtpname,
	string newwtplocation,
	uint8_t newrid,
	uint8_t newchannel,
	uint32_t newtxpower,
	string newessid) = 0;
	
	///
	/// The function sets all the information derived to Pre Configuration.
	///	\param ...
	///	\return \e true if the procedure is correct, \e falses otherwise.
	///
	virtual bool AddPreConfigurationInformation(
	IPv4Address newip, 
	string newwtpname, 
	string newwtplocation, 
	uint8_t newchannel,
	uint32_t newtxpower) = 0;

	virtual bool SetStatus(IPv4Address wtpip, StatusType newstatus) = 0;
	virtual bool SetPreConfigure(IPv4Address wtpip,StatusConfiguration newvalue) = 0;
	virtual bool ConfigurationUpdate() = 0;
	
	virtual int Getoutudpport() = 0;
	virtual int Getinudpport() = 0;
	
	virtual string GetACName() = 0;
	virtual string GetWTPESSID() = 0; 
	virtual IPv4Address GetWTPNetmask() = 0;
	virtual IPv4Address GetWTPGateway() = 0; 
	virtual uint8_t GetRadioID(IPv4Address WTPIP) = 0;
	virtual IPv4Address GetIP(EthernetMACAddress WTPMAC) = 0;
	virtual uint32_t GetSID(IPv4Address WTPIP) = 0;
	virtual EthernetMACAddress GetMAC(IPv4Address WTPIP) = 0;
	virtual StatusType GetStatus(IPv4Address WTPIP) = 0;
	virtual string GetLocation(IPv4Address WTPIP) = 0;
	virtual string GetWTPName(IPv4Address WTPIP) = 0;
	virtual uint8_t GetChannel(IPv4Address WTPIP)  = 0;
	virtual uint32_t GetTxPower(IPv4Address WTPIP) = 0;
	virtual  StatusConfiguration GetPreConfigure(IPv4Address WTPIP) = 0;
	virtual int GetMaxNumberWTP(void) = 0;
	virtual int GetNumberWTP(void) = 0;
	virtual bool DelWTP(IPv4Address WTPIP) = 0;
	virtual bool RefreshWTP(IPv4Address newip) = 0;
	
};

}

#endif

