/// \file Gateway/gateway.cpp
/// \author Mauro Brunato
///
/// \brief Small program for testing classes in the WilmaGate package.
///
/// $Id: gateway.cpp,v 1.12 2005/12/22 11:45:46 cvs Exp $

/*##############################################################################
   
   Developed by NetMob Group, University of Trento, Italy

   University of Trento
   DIT - Department of Informatics and Telecommunications 
   Computer Networks and Mobility Research Program
   http://netmob.unitn.it/ 
   
   This file is part of the WilmaGate system, created as part
   of the WILMA Project, sponsored by the Autonomous
   Province of Trento and participated by ITC-irst, the University
   of Trento and Alpikom Spa. More info at

     http://www.wilmaproject.org/		

   To download the project:
      - in tarball format
        http://netmob.unitn.it/wilmagate.html
      - connect to CVS repository
        :pserver:cvs@rtm.unitn.it:/home/cvsroot
   In both previous cases, the download is password protected. To obtain
   the password you must send a mail to 

      wilmagate@dit.unitn.it

################################################################################

   The WilmaGate Licence of Use


   Copyright (c) 2003-2005
                - Universita` di Trento,
                  Dipartimento di Informatica e Telecomunicazioni (DIT)
                  Computer Networks and Mobility Research Program


   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (hereinafter
   the "WilmaGate") directly from the copyright owner, to use the WilmaGate
   without restriction of purpose or time, including without limitation the
   rights to use, copy, modify, and merge the WilmaGate, subject to the
   following conditions:

   The above copyright notice and this permission notice shall be included
   in all copies or substantial portions of the WilmaGate. 

   The WilmaGate shall not be sub-licenced either commercially or free of
   charge, as is or in part or as a part of any other package. 

   The licensee agrees to properly credit WilmaGate and its copyright 
   owners in any technical, scientific or divulgative publication, 
   whatever publishing means are used, that is based entirely or in part 
   on WilmaGate use, as well as in any booklet, web-site or brochure that 
   advertise services based on WilmaGate use. 

   If the WilmaGate is modified, the copyright owner shall be notified.

   THE WilmaGate IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

##############################################################################*/

#include <iostream>
#include <iomanip>
extern "C" {
#include <unistd.h>
}

#include "parameters.h"
#include "utilities.h"

#include "clientlist.h"
#include "rules.h"
#include "router.h"
#include "netcard.h"
#include "interfaces.h"
#include "layer2capture.h"
#include "layer2send.h"
#include "command.h"

using namespace std;
using namespace Common;
using namespace Gateway;


int main (int argc, char *argv[])
{
	try {

		IPv4Address INNER_IFNET;
		IPv4Address INNER_IFMASK;
		IPv4Address INNER_ARPNET;
		IPv4Address INNER_ARPMASK;
		string INNER_PROP;
		string INNER_PCAP;

		IPv4Address OUTER_IFNET;
		IPv4Address OUTER_IFMASK;
		IPv4Address OUTER_ARPNET;
		IPv4Address OUTER_ARPMASK;
		string OUTER_PROP;
		string OUTER_PCAP;
		IPv4Address OUTER_GATEWAY;

		IPv4Address TUNNEL_IP;
		int TUNNEL_PORT;
		IPv4Address COMMAND_IP;
		int COMMAND_PORT;
		IPv4Address GATEKEEPER_DISPATCHER_IP;
		int GATEKEEPER_DISPATCHER_PORT;
		IPv4Address GATEKEEPER_REPORTER_IP;
		int GATEKEEPER_REPORTER_PORT;

		IPv4Address SINK_IP;
		int SINK_PORT;

		string DNS_SERVERS;
		IPv4Address INITIAL_HTTP_SERVER;
		string PROXY_SERVERS;
		IPv4Address RADIUS_SERVER;

		int SIP_POLICY;
		
		int runmode;
		string LOGGER_FILENAME;
		int TRANSPARENT;
		string PID_FILE;

		Parameters pmt (argc, argv);

		pmt
			>> INNER_IFNET
			>> INNER_IFMASK
			>> INNER_ARPNET
			>> INNER_ARPMASK
			>> INNER_PROP
			>> INNER_PCAP

			>> OUTER_IFNET
			>> OUTER_IFMASK
			>> OUTER_ARPNET
			>> OUTER_ARPMASK
			>> OUTER_PROP
			>> OUTER_PCAP
			>> OUTER_GATEWAY

			>> TUNNEL_IP
			>> TUNNEL_PORT
			>> COMMAND_IP
			>> COMMAND_PORT
			>> GATEKEEPER_DISPATCHER_IP
			>> GATEKEEPER_DISPATCHER_PORT
			>> GATEKEEPER_REPORTER_IP
			>> GATEKEEPER_REPORTER_PORT

			>> SINK_IP
			>> SINK_PORT

			>> DNS_SERVERS
			>> INITIAL_HTTP_SERVER
			>> PROXY_SERVERS
			>> RADIUS_SERVER

			>> SIP_POLICY
			
			>> runmode
			>> LOGGER_FILENAME
			>> TRANSPARENT
			>> PID_FILE;

		if ( runmode ) {
			detachFromConsole (LOGGER_FILENAME.c_str());
			cerr << "\n=\n===========================================\n=\n";
		}

		if ( PID_FILE.length() ) {
			ofstream pidf (PID_FILE.c_str());
			pidf << getpid() << endl;
			pidf.close ();
		}

		logger() << "Gateway program started.\n";

		logger() << "Getting interface settings.\n";
		IPv4Address innerIP, outerIP;
		EthernetMACAddress innerMAC, outerMAC;
		int inneridx = getSettings (INNER_PROP.c_str(), innerIP, innerMAC);
		int outeridx = getSettings (OUTER_PROP.c_str(), outerIP, outerMAC);
		logger() << "Inner: " << innerIP << ' ' << innerMAC << '\n';
		logger() << "Outer: " << outerIP << ' ' << outerMAC << '\n';
		logger() << "Creating user authorization list.\n";
		ClientList list (INNER_IFNET, INNER_IFNET+255);

		logger() << "Creating user list command interface.\n";
		Command cmd (list, COMMAND_IP, COMMAND_PORT, GATEKEEPER_REPORTER_IP, GATEKEEPER_REPORTER_PORT);

		logger() << "Downloading authorization list from Gatekeeper\n";
		if ( ! cmd.getAuthorizations () ) {
			logger() << "Error in getting authorization strings.\n";
			return -5;
		}

		logger() << "Creating rule table.\n";
		RuleTable *rules;
		if ( TRANSPARENT ) {
			rules = new RuleTable (Rule::PASS, Rule::PASS);
			rules->newRule (EthernetFrame::IP_PROTOCOL,
					0xFFFFFFFFUL,
					0xFFFFFFFFUL,
					IPPacket::UDP_PROTOCOL,
					0xFFFF,
					0x0043,
					Rule::INNER,
					Rule::SUBMIT,
					Rule::SUBMIT);
			rules->newRule (EthernetFrame::IP_PROTOCOL,
					0xFFFFFFFFUL,
					0xFFFFFFFFUL,
					IPPacket::UDP_PROTOCOL,
					0xFFFF,
					0x0043,
					Rule::OUTER,
					Rule::DROP,
					Rule::DROP);
		}
		else {
			rules = new RuleTable (Rule::SUBMIT, Rule::PASS);
			while ( PROXY_SERVERS.length() ) {
				IPv4Address ip;
				unsigned int i = PROXY_SERVERS.find (';');
				if ( i == string::npos ) {
					ip = PROXY_SERVERS.c_str();
					PROXY_SERVERS.erase();
				}
				else {
					ip = PROXY_SERVERS.substr(0,i).c_str();
					while ( i<PROXY_SERVERS.length() && (PROXY_SERVERS[i] == ';' || PROXY_SERVERS[i] <= ' ') )
						i++;
					PROXY_SERVERS.erase (0, i);
				}
				logger() << "Adding rule for Proxy " << ip << '\n';
				rules->newRule (EthernetFrame::IP_PROTOCOL,
						0xFFFFFFFFUL,
						ip,
						IPPacket::TCP_PROTOCOL,
						0xFFFF,
						3128,
						Rule::INNER,
						Rule::SUBMIT,
						Rule::PASS);
				rules->newRule (EthernetFrame::IP_PROTOCOL,
						ip,
						0xFFFFFFFFUL,
						IPPacket::TCP_PROTOCOL,
						3128,
						0xFFFF,
						Rule::OUTER,
						Rule::SUBMIT,
						Rule::PASS);
			}

			// DNS queries and responses are authorized for the DNS server
			// also by unauthorized clients
			while ( DNS_SERVERS.length() ) {
				IPv4Address ip;
				unsigned int i = DNS_SERVERS.find (';');
				if ( i == string::npos ) {
					ip = DNS_SERVERS.c_str();
					DNS_SERVERS.erase();
				}
				else {
					ip = DNS_SERVERS.substr(0,i).c_str();
					while ( i<DNS_SERVERS.length() && (DNS_SERVERS[i] == ';' || DNS_SERVERS[i] <= ' ') )
						i++;
					DNS_SERVERS.erase (0, i);
				}
				logger() << "Adding rule for DNS " << ip << '\n';
				rules->newRule (EthernetFrame::IP_PROTOCOL,
						0xFFFFFFFFUL,
						ip,
						IPPacket::UDP_PROTOCOL,
						0xFFFF,
						0x0035,
						Rule::INNER,
						Rule::PASS,
						Rule::PASS);
				rules->newRule (EthernetFrame::IP_PROTOCOL,
						0xFFFFFFFFUL,
						ip,
						IPPacket::TCP_PROTOCOL,
						0xFFFF,
						0x0035,
						Rule::INNER,
						Rule::PASS,
						Rule::PASS);
				rules->newRule (EthernetFrame::IP_PROTOCOL,
						ip,
						0xFFFFFFFFUL,
						IPPacket::UDP_PROTOCOL,
						0x0035,
						0xFFFF,
						Rule::OUTER,
						Rule::PASS,
						Rule::PASS);
				rules->newRule (EthernetFrame::IP_PROTOCOL,
						ip,
						0xFFFFFFFFUL,
						IPPacket::TCP_PROTOCOL,
						0x0035,
						0xFFFF,
						Rule::OUTER,
						Rule::PASS,
						Rule::PASS);
			}

			// DHCP requests coming from clients must always be submitted
			// to the Gatekeeper, while DHCP packets from the outside world
			// must be dropped.
			rules->newRule (EthernetFrame::IP_PROTOCOL,
					0xFFFFFFFFUL,
					0xFFFFFFFFUL,
					IPPacket::UDP_PROTOCOL,
					0xFFFF,
					0x0043,
					Rule::INNER,
					Rule::SUBMIT,
					Rule::SUBMIT);
			rules->newRule (EthernetFrame::IP_PROTOCOL,
					0xFFFFFFFFUL,
					0xFFFFFFFFUL,
					IPPacket::UDP_PROTOCOL,
					0xFFFF,
					0x0043,
					Rule::OUTER,
					Rule::DROP,
					Rule::DROP);

                        // ICMP packets must always pass
                        rules->newRule (EthernetFrame::IP_PROTOCOL,
                                        0xFFFFFFFFUL,
                                        0xFFFFFFFFUL,
                                        IPPacket::ICMP_PROTOCOL,
                                        0xFFFF,
                                        0xFFFF,
                                        Rule::INNER,
                                        Rule::PASS,
                                        Rule::PASS);
                        rules->newRule (EthernetFrame::IP_PROTOCOL,
                                        0xFFFFFFFFUL,
                                        0xFFFFFFFFUL,
                                        IPPacket::ICMP_PROTOCOL,
                                        0xFFFF,
                                        0xFFFF,
                                        Rule::OUTER,
                                        Rule::PASS,
                                        Rule::PASS);

			// Web (port 80) requests directed to the initial HTTP server
			// can freely pass
			rules->newRule (EthernetFrame::IP_PROTOCOL,
					0xFFFFFFFFUL,
					INITIAL_HTTP_SERVER,
					IPPacket::TCP_PROTOCOL,
					0xFFFF,
					80,
					Rule::INNER,
					Rule::PASS,
					Rule::PASS);
			rules->newRule (EthernetFrame::IP_PROTOCOL,
					INITIAL_HTTP_SERVER,
					0xFFFFFFFFUL,
					IPPacket::TCP_PROTOCOL,
					80,
					0xFFFF,
					Rule::OUTER,
					Rule::PASS,
					Rule::PASS);
					
			// SIP traffic (port 5060)
			if(SIP_POLICY){ //submit always to Gatekeeper
				rules->newRule(EthernetFrame::IP_PROTOCOL,
						0xFFFFFFFFUL,
						0xFFFFFFFFUL,
						IPPacket::UDP_PROTOCOL,
						0xFFFF,
						5060,
						Rule::INNER,
						Rule::SUBMIT,
						Rule::SUBMIT);
				rules->newRule(EthernetFrame::IP_PROTOCOL,
					0xFFFFFFFFUL,
					0xFFFFFFFFUL,
					IPPacket::UDP_PROTOCOL,
					5060,
					0xFFFF,
					Rule::OUTER,
					Rule::SUBMIT,
					Rule::SUBMIT);
			}else{ //submit to Gatekeeper only unauthorized packets
				rules->newRule(EthernetFrame::IP_PROTOCOL,
						0xFFFFFFFFUL,
						0xFFFFFFFFUL,
						IPPacket::UDP_PROTOCOL,
						0xFFFF,
						5060,
						Rule::INNER,
						Rule::SUBMIT,
						Rule::PASS);
				rules->newRule(EthernetFrame::IP_PROTOCOL,
					0xFFFFFFFFUL,
					0xFFFFFFFFUL,
					IPPacket::UDP_PROTOCOL,
					5060,
					0xFFFF,
					Rule::OUTER,
					Rule::SUBMIT,
					Rule::PASS);
			
			}	
			/*
			// CAPWAP traffic (port 5061)
			rules->newRule(EthernetFrame::IP_PROTOCOL,
						0xFFFFFFFFUL,
						0xFFFFFFFFUL,
						IPPacket::UDP_PROTOCOL,
						0xFFFF,
						5061,
						Rule::INNER,
						Rule::SUBMIT,
						Rule::SUBMIT);
			rules->newRule(EthernetFrame::IP_PROTOCOL,
					0xFFFFFFFFUL,
					0xFFFFFFFFUL,
					IPPacket::UDP_PROTOCOL,
					5061,
					0xFFFF,
					Rule::OUTER,
					Rule::SUBMIT,
					Rule::SUBMIT);
			*/
		}

		logger() << "Creating router object.\n";
		Router rtr (list, *rules, TUNNEL_IP, TUNNEL_PORT, GATEKEEPER_DISPATCHER_IP, GATEKEEPER_DISPATCHER_PORT,
				SINK_IP, SINK_PORT);

		logger() << "Creating inner network card object.\n";
		NetworkCard innercard (INNER_PCAP, inneridx, rtr, Rule::INNER, innerIP, innerMAC,
					INNER_IFNET, INNER_IFMASK, INNER_ARPNET, INNER_ARPMASK);
		logger() << "Creating outer network card object.\n";
		NetworkCard outercard (OUTER_PCAP, outeridx, rtr, Rule::OUTER, outerIP, outerMAC,
					OUTER_IFNET, OUTER_IFMASK, OUTER_ARPNET, OUTER_ARPMASK);

		logger() << "Looking for gateway...\n";
		outercard.locateGateway (OUTER_GATEWAY);
		logger() << "Gateway is at " << outercard.gatewayMAC() << '\n';

		logger() << "Starting inner network card.\n";
		innercard.start ();

		logger() << "Starting outer network card.\n";
		outercard.start ();

		logger() << "Starting router.\n";
		rtr.start ();

		logger() << "Starting command interface.\n";
		cmd.start ();

		logger() << "Entering waiting loop." << endl;
		innercard.wait ();
	}
	catch ( WilmaGateException &e ) {
		logger() << e << endl;
		return 1;
	}

	return 0;
}
