/// \file rawipsend.h
/// \author Mauro Brunato
///
/// \brief Raw IP socket sending classes.
///
/// $Id: rawipsend.h,v 1.4 2005/06/16 13:22:23 cvs Exp $

/*##############################################################################
   
   Developed by NetMob Group, University of Trento, Italy

   University of Trento
   DIT - Department of Informatics and Telecommunications 
   Computer Networks and Mobility Research Program
   http://netmob.unitn.it/ 
   
   This file is part of the WilmaGate system, created as part
   of the WILMA Project, sponsored by the Autonomous
   Province of Trento and participated by ITC-irst, the University
   of Trento and Alpikom Spa. More info at

     http://www.wilmaproject.org/		

   To download the project:
      - in tarball format
        http://netmob.unitn.it/wilmagate.html
      - connect to CVS repository
        :pserver:cvs@rtm.unitn.it:/home/cvsroot
   In both previous cases, the download is password protected. To obtain
   the password you must send a mail to 

      wilmagate@dit.unitn.it

################################################################################

   The WilmaGate Licence of Use


   Copyright (c) 2003-2005
                - Universita` di Trento,
                  Dipartimento di Informatica e Telecomunicazioni (DIT)
                  Computer Networks and Mobility Research Program


   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (hereinafter
   the "WilmaGate") directly from the copyright owner, to use the WilmaGate
   without restriction of purpose or time, including without limitation the
   rights to use, copy, modify, and merge the WilmaGate, subject to the
   following conditions:

   The above copyright notice and this permission notice shall be included
   in all copies or substantial portions of the WilmaGate. 

   The WilmaGate shall not be sub-licenced either commercially or free of
   charge, as is or in part or as a part of any other package. 

   The licensee agrees to properly credit WilmaGate and its copyright 
   owners in any technical, scientific or divulgative publication, 
   whatever publishing means are used, that is based entirely or in part 
   on WilmaGate use, as well as in any booklet, web-site or brochure that 
   advertise services based on WilmaGate use. 

   If the WilmaGate is modified, the copyright owner shall be notified.

   THE WilmaGate IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

##############################################################################*/

#ifndef _H_RAWIPSEND_
#define _H_RAWIPSEND_

#include "exceptions.h"

namespace Common {
	class IPPacket;
}

using namespace Common;

// commented in netcard.h
namespace Gateway {

	////////////////////////////////////////////////////
	///
	/// \brief Exception raised for critical events in the RawIPSend class.
	///
	class IPSendException:
		public WilmaGateException {
	public:
		///
		/// \brief Constructor.
		///
		/// \param c Subcode of the exception.
		///
		inline IPSendException (int c);
	};

	inline IPSendException::IPSendException (int c):
		WilmaGateException ("Raw IP send interface", c)
	{
	}

	//////////////////////////////////////////////////////////////////////////////////
	///
	/// \brief Raw IP packet sending.
	///
	/// Wrapper class for sending raw IP packets.
	///
	class RawIPSend {
	public:
		///
		/// \brief Error states after object construction.
		///
		enum StatusType {
			GOOD = 0,	///< \brief No error encountered.
			BADOPTION = 1,	///< \brief Header inclusion option not available
			BADOPEN = 2,	///< \brief Could not open the interface.
			BADSEND = 3	///< \brief Could not send
		};
	private:
		///
		/// \brief Raw IP socket.
		///
		int skt;
		///
		/// \brief Address structure for the raw IP socket.
		///
		sockaddr_in addr;
	public:
		///
		/// \brief Constructor.
		///
		RawIPSend (void) throw (IPSendException);
		///
		/// \brief Destructor.
		///
		~RawIPSend (void);
		///
		/// \brief Send an IP packet.
		///
		/// \param p The IP packet, completed with the header.
		/// \param a Destination address of the IP packet.
		/// \param l Total length of the frame.
		///
		void send (const IPPacket *p, in_addr a, int l) throw (IPSendException);
	};

} // namespace Gateway

#endif
