/// \file relay.cpp
/// \author Mauro Brunato
///
/// $Id: relay.cpp,v 1.2 2005/06/16 13:22:23 cvs Exp $

/*##############################################################################
   
   Developed by NetMob Group, University of Trento, Italy

   University of Trento
   DIT - Department of Informatics and Telecommunications 
   Computer Networks and Mobility Research Program
   http://netmob.unitn.it/ 
   
   This file is part of the WilmaGate system, created as part
   of the WILMA Project, sponsored by the Autonomous
   Province of Trento and participated by ITC-irst, the University
   of Trento and Alpikom Spa. More info at

     http://www.wilmaproject.org/		

   To download the project:
      - in tarball format
        http://netmob.unitn.it/wilmagate.html
      - connect to CVS repository
        :pserver:cvs@rtm.unitn.it:/home/cvsroot
   In both previous cases, the download is password protected. To obtain
   the password you must send a mail to 

      wilmagate@dit.unitn.it

################################################################################

   The WilmaGate Licence of Use


   Copyright (c) 2003-2005
                - Universita` di Trento,
                  Dipartimento di Informatica e Telecomunicazioni (DIT)
                  Computer Networks and Mobility Research Program


   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (hereinafter
   the "WilmaGate") directly from the copyright owner, to use the WilmaGate
   without restriction of purpose or time, including without limitation the
   rights to use, copy, modify, and merge the WilmaGate, subject to the
   following conditions:

   The above copyright notice and this permission notice shall be included
   in all copies or substantial portions of the WilmaGate. 

   The WilmaGate shall not be sub-licenced either commercially or free of
   charge, as is or in part or as a part of any other package. 

   The licensee agrees to properly credit WilmaGate and its copyright 
   owners in any technical, scientific or divulgative publication, 
   whatever publishing means are used, that is based entirely or in part 
   on WilmaGate use, as well as in any booklet, web-site or brochure that 
   advertise services based on WilmaGate use. 

   If the WilmaGate is modified, the copyright owner shall be notified.

   THE WilmaGate IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

##############################################################################*/

#include "utilities.h"
#include "relay.h"
#include "packets.h"

using namespace Common;

namespace Gateway {

	Relay::Relay (IPv4Address ip, uint16_t port, IPv4Address sip, uint16_t sport)
	{
		myeth = new (10000) EthernetFrame;
		myip = &(myeth->IPPayload());
		myudp = &(myip->UDPPayload());
		mydhcp = reinterpret_cast<DHCPMessage*>(myudp->payload);
		socket.bind (ip, port);
		socket.setDestination (sip, sport);
		socket.registerListener (*this, 0);
	}

	Relay::~Relay (void)
	{
	}

	void Relay::sendFrame (EthernetFrame *eth, int l, Rule::SenseType sense)
	{
		logger() << "Got packet from " << eth->sender << '\n';
		if ( eth->protocol == htons(EthernetFrame::IP_PROTOCOL)
				&& eth->IPPayload().protocol == IPPacket::UDP_PROTOCOL
				&& eth->IPPayload().UDPPayload().destinationPort == 0x4300U ) {
			logger() << "It's a DHCP request!!\n";

			int dhcplen;
			DHCPMessage *dhcp = reinterpret_cast<DHCPMessage*>(eth->IPPayload().UDPPayload().payload);

			// TODO: Given the received server message stored in dhcp,
			// TODO: create the corresponding DHCP message into mydhcp
			// TODO: and store the message size into dhcplen.

			socket.send (mydhcp, dhcplen);
		}

	}

	void Relay::registerListener (RouterListener *t)
	{
		interface = t;
	}

	Relay::SuggestedAction Relay::failed (int socketID)
	{
		return CONTINUE;
	}

	Relay::SuggestedAction Relay::receivedPacket (uint8_t *message, int size, int socketID, IPv4Address senderIP,
							uint16_t senderPort)
	{
		DHCPMessage *&dhcp = reinterpret_cast<DHCPMessage*>(message);
		int ethlen;

		// Given the DHCP server message dhcp,
		// create the corresponding client ethernet frame into myeth
		// and the corresponding size into ethlen.

		interface->receiveFrame(myeth, ethlen);
		return CONTINUE;
	}

} // namespace Gateway
