/// \file relay.h
/// \author Mauro Brunato
///
/// \brief Classes related to routing and the gatekeeper tunnel
///
/// $Id: relay.h,v 1.2 2005/06/16 13:22:23 cvs Exp $

/*##############################################################################
   
   Developed by NetMob Group, University of Trento, Italy

   University of Trento
   DIT - Department of Informatics and Telecommunications 
   Computer Networks and Mobility Research Program
   http://netmob.unitn.it/ 
   
   This file is part of the WilmaGate system, created as part
   of the WILMA Project, sponsored by the Autonomous
   Province of Trento and participated by ITC-irst, the University
   of Trento and Alpikom Spa. More info at

     http://www.wilmaproject.org/		

   To download the project:
      - in tarball format
        http://netmob.unitn.it/wilmagate.html
      - connect to CVS repository
        :pserver:cvs@rtm.unitn.it:/home/cvsroot
   In both previous cases, the download is password protected. To obtain
   the password you must send a mail to 

      wilmagate@dit.unitn.it

################################################################################

   The WilmaGate Licence of Use


   Copyright (c) 2003-2005
                - Universita` di Trento,
                  Dipartimento di Informatica e Telecomunicazioni (DIT)
                  Computer Networks and Mobility Research Program


   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (hereinafter
   the "WilmaGate") directly from the copyright owner, to use the WilmaGate
   without restriction of purpose or time, including without limitation the
   rights to use, copy, modify, and merge the WilmaGate, subject to the
   following conditions:

   The above copyright notice and this permission notice shall be included
   in all copies or substantial portions of the WilmaGate. 

   The WilmaGate shall not be sub-licenced either commercially or free of
   charge, as is or in part or as a part of any other package. 

   The licensee agrees to properly credit WilmaGate and its copyright 
   owners in any technical, scientific or divulgative publication, 
   whatever publishing means are used, that is based entirely or in part 
   on WilmaGate use, as well as in any booklet, web-site or brochure that 
   advertise services based on WilmaGate use. 

   If the WilmaGate is modified, the copyright owner shall be notified.

   THE WilmaGate IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

##############################################################################*/

#ifndef _RELAY_H_
#define _RELAY_H_

#include "packets.h"
#include "socket.h"
#include "router.h"

using namespace Common;

// Commented in netcard.h
namespace Gateway {

	////////////////////////////////////////////////////////////////////
	///
	/// \brief Relay object
	///
	class Relay:
		public RouterInterface,
		public PacketListener {
	private:
		///
		/// \brief Array of pointers to registered listeners.
		///
		RouterListener *interface;
		///
		/// \brief UDP socket for gatekeeper communications.
		///
		UDPSocket socket;
		///
		/// \brief Pointer to a memory area devoted to an ethernet frame
		///
		EthernetFrame *myeth;
		///
		/// \brief Pointer to a memory area devoted to a DHCP message
		///
		IPPacket *myip;
		///
		/// \brief Pointer to a memory area devoted to a DHCP message
		///
		UDPDatagram *myudp;
		///
		/// \brief Pointer to a memory area devoted to a DHCP message
		///
		DHCPMessage *mydhcp;
	public:
		///
		/// \brief Constructor.
		///
		/// \param ip IP bind address.
		/// \param port UDP bind port.
		/// \param sip IP address of the DHCP server.
		/// \param sport UDP port of the DHCP server.
		///
		Relay (IPv4Address ip, uint16_t port, IPv4Address sip, uint16_t sport);
		///
		/// \brief Virtual destructor.
		///
		virtual ~Relay (void);
		///
		/// \brief Start the UDP socket receiver.
		///
		inline void start (void);
		//
		// Inherited from PacketListener.
		//
		virtual SuggestedAction failed (int socketID);
		virtual SuggestedAction receivedPacket (uint8_t *message, int size, int socketID, IPv4Address senderIP,
							uint16_t senderPort);
		//
		// Inherited from RouterInterface.
		//
		virtual void sendFrame (EthernetFrame *eth, int l, Rule::SenseType sense);
		virtual void registerListener (RouterListener *t);
	};

	inline void Relay::start (void)
	{
		socket.startReceiving ();
	}

} // namespace Gateway

#endif
