#include "net_funcs.h"

int init_tcp_conn(char *server, int port){
	int scktfd,
		i;
	struct hostent *serv_name;
	struct sockaddr_in serv_ip;
	uint8_t *w;
	/*
	 * 
	 */
	if ((scktfd = socket(PF_INET, SOCK_STREAM, 0)) < 0){
		LOG(L_ERR, "ERROR unify init_tcp_conn(): error creating the socket\n");
		return -1;
	}
	/*
	 * 
	 */
	serv_name = gethostbyname(server);
	if (serv_name == NULL) 
	{
		LOG(L_ERR, "ERROR unify init_tcp_conn(): dns resolution failed\n");
		return -1;
    }
    /*
     * 
     */	
	memset((void *)&serv_ip, 0, sizeof(serv_ip));
	serv_ip.sin_family = serv_name->h_addrtype;
	serv_ip.sin_port = htons(port);	
	for (i = 0; serv_name->h_addr_list[i]; ++i) {
		serv_ip.sin_addr.s_addr = *(uint32_t *)(serv_name->h_addr_list[i]);
		w = (uint8_t *)serv_name->h_addr_list[i];
		LOG(L_DBG, "DEBUG unify init_tcp_conn(): try connecting to %d.%d.%d.%d\n", 
				w[0],w[1],w[2],w[3]);
		if(connect(scktfd, (const struct sockaddr *)&serv_ip, sizeof(serv_ip)) < 0)
			LOG(L_DBG, "DEBUG unify init_tcp_conn(): error connecting to %s:%d %s\n",
					server, port, strerror(errno));
		else{
			LOG(L_DBG, "DEBUG unify init_tcp_conn(): connected to %d.%d.%d.%d\n", 
				w[0],w[1],w[2],w[3]);
			return scktfd;
		}
	}
	LOG(L_ERR, "ERROR unify init_tcp_conn(): error connecting to %s:%d\n", server, port);
	return -1;
}

int writen(int fd, void* buf, size_t l){
	size_t nwrite,
		   nres;
	nwrite = 0;
	nres = l;	
	do{
		nwrite = write(fd, (buf+l-nres), nres);
		if(nwrite < 0){
			if(errno == EINTR)
				continue;
			return nres;/* EOF > 0*/
		}
		nres -= nwrite;
	}while(nres > 0);
	return 0;/* OK = 0*/
}

int readn(int fd, void* buf, size_t l){
	size_t nread,
		   nres;
	nread = 0;
	nres = l;
	do{
		nread = read(fd, (buf+l-nres), nres);
		if(nread < 0){
			if(errno == EINTR)
				continue;
			return -1;/* ERROR < 0*/
		}
		else if(nread == 0) 
			return nres;/* EOF > 0*/
		nres -= nread;
	}while(nres > 0);
	return 0;/* OK = 0*/
}

void close_tcp_conn(int scktfd){
	
	if(close(scktfd) < 0)
		LOG(L_ERR, "ERROR unify close_tcp_conn(): socket %d '%s'\n", scktfd, strerror(errno));
	else
		LOG(L_DBG, "DEBUG unify close_tcp_conn(): close %d\n", scktfd);
}
