#include "sem_funcs.h"
/*
 * Semaphores set create
 */
int semcrt(int nsems){
	int semid,
		i;
	if((semid = semget(IPC_PRIVATE, nsems, IPC_CREAT|0666)) < 0){
		LOG(L_ERR, "ERROR unify semcrt(): cannot create semaphores (reason: '%s')\n", strerror(errno));
		return -1;
	}
	/* init semaphore value to 1 */
	for (i = 0; i < nsems; ++i)
		if(semctl(semid, i, SETVAL, 1) < 0){
			LOG(L_ERR, "ERROR unify semcrt(): cannot init semid %d num %d (reason: '%s')\n", semid, i, strerror(errno));
			return -1;
		}		
	return semid;
}
/*
 * Semaphores set remove 
 */
int semrmv(int semid){
	if(semctl(semid, 0, IPC_RMID) < 0){
		LOG(L_ERR, "ERROR unify semrmv(): cannot remove semid %d (reason: '%s')\n", semid, strerror(errno));
		return -1;
	}
	return 0;
}
/*
 * Mutex locking
 */
int mtxlock(int semid, int sem_num){
	struct sembuf semlock = {
			sem_num, /* semaphore number 	*/
			-1,		 /* semaphore operation	*/
			SEM_UNDO /* flag: undo the operation on exit */
	};
	LOG(L_NOTICE, "DEBUG unify mtxLOCK(): semid %d sem_num %d trying...\n", semid, sem_num);
 	if (semop(semid, &semlock, 1) < 0) {
		LOG(L_ERR, "ERROR unify mtxlock(): cannot lock semid %d sem_num %d (reason: '%s')\n", semid, sem_num, strerror(errno));
		return -1;
	}
	LOG(L_NOTICE, "DEBUG unify mtxLOCK(): semid %d sem_num %d OK!\n", semid, sem_num);
	return 0;
}
/*
 * Mutex locking
 */
int mtxunlock(int semid, int sem_num){
	struct sembuf semunlock[2] = {
		{
		sem_num,	/* semaphore number				*/
		0,			/* semaphore operation			*/
		IPC_NOWAIT  /* flag: return error on wait.	*/
		},
		{
		sem_num,	/* semaphore number 	*/
		1,			/* semaphore operation	*/
 		0			/* no operation flag	*/
		}
	};
	LOG(L_NOTICE, "DEBUG unify mtxUNLOCK(): semid %d sem_num %d trying...\n", semid, sem_num);
	if(semop(semid, semunlock, 2) < 0){
		LOG(L_ERR, "ERROR unify mtxunlock(): cannot unlock semid %d sem_num %d (reason: '%s')\n", semid, sem_num, strerror(errno));
		return -1;
	}
	LOG(L_NOTICE, "DEBUG unify mtxUNLOCK(): semid %d sem_num %d OK!\n", semid, sem_num);
	return 0;
}
