#include "shm_funcs.h"

void* shmcomp(char *shm_name, off_t shmsz, int flag, mode_t mode){
    void *p;
    int shmfd;
    /*
     * Open shared memory segment:
     * - creating it if not exists
     */
    if((shmfd = shm_open(shm_name, flag, mode)) < 0){
    	if(errno == EEXIST)
			LOG(L_WARN, "WARNING unify shmcomp(): shared memory segment '%s' already exist!\n", shm_name);
    	else
    		LOG(L_ERR, "ERROR unify shmcomp():"
					" cannot open shared memory segment '%s' (reason = %s)!\n",
					shm_name, strerror(errno));
    	return 0;
    }
    /* 
     * Truncate the file shmfd is open on to shmsz bytes.
     */
    if(flag&O_CREAT)
	    if(ftruncate(shmfd, shmsz)){
			LOG(L_ERR , "ERROR unify shmcomp():"
					" cannot truncate shared memory segment '%s' to %d (reason = %s)!\n",
					shm_name, (int)shmsz, strerror(errno));
			return 0;
	    }
    /* 
     * Map addresses: 
     * - PROT_WRITE: Page can be written.
	 * - PROT_READ:	 Page can be read.
	 * - MAP_SHARED: Share changes 
     */
    if((p = mmap(0, shmsz, PROT_WRITE|PROT_READ, MAP_SHARED, shmfd, 0)) == MAP_FAILED){
		LOG(L_ERR , "ERROR unify shmcomp():"
				" cannot map shared memory segment '%s' (reason = %s)!\n", shm_name, strerror(errno));
		return 0;
    }
    /* 
     * Erase shared memory segment
     */
    if(flag&O_CREAT)
    	memset((void *) p, 0, shmsz);
    return p;
}
